---------------------------------------------------------------------
-- TITLE: division module addtional component in multiplier.vhd
-- AUTHOR: Rene Doss
-- Company Dossmatik GmbH
-- DATE CREATED: 2.4.2013
-- FILENAME: divide.vhd
--
-- COPYRIGHT: Creative Commons CC BY-NC 3.0 with exception 
-- commercial applicants have to pay a licence fee
-- 
-- DESCRIPTION:
-- Implements divison
---------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

use work.mlite_pack.all;

entity division is
  
  
  port (

    cpu_clk   : in  std_logic;
    reset     : in  std_logic;
    dividend  : in  std_logic_vector(31 downto 0);
    divisor   : in  std_logic_vector(31 downto 0);
    mult_func : in  mult_function_type;
    div_zero  : out std_logic;
    div_busy  : out std_logic := '0';
    div_rdy   : out std_logic := '0';
    quotient  : out std_logic_vector(31 downto 0);
    reminder  : out std_logic_vector(31 downto 0)
    );
end division;

architecture Behavioral of division is
  signal div_counter : unsigned (5 downto 0) := "000000";

  signal divis    : std_logic_vector(31 downto 0);
  signal remind   : std_logic_vector(32 downto 0);
  signal speicher : std_logic_vector(31 downto 0);
  -- speicher has dual function
  -- start dividend end hold quotient (save registers) 
  signal sign     : std_logic;

  signal sub : unsigned(32 downto 0);
  
begin

  sub <= unsigned(remind)-resize(unsigned(divis), 33);

  process(cpu_clk)
  begin
    if rising_edge(cpu_clk) then
      if div_counter = "00001" then
        reminder <= remind(32 downto 1);

        if sign = '0' then
          quotient <= speicher;
        else
          quotient <= std_logic_vector(0- signed(speicher));
        end if;
      end if;
    end if;
  end process;


  process(cpu_clk)
  begin
    if rising_edge(cpu_clk) then
      if div_counter /= "000000" then
        div_counter <= div_counter-1;
        --calculation     
        if sub(32) = '0' then
          remind   <= std_logic_vector(sub(31 downto 0)) & speicher(31);
          speicher <= speicher(30 downto 0)& '1';
        else
          remind   <= remind(31 downto 0) & speicher(31);
          speicher <= speicher(30 downto 0)& '0';
        end if;
        
      else
        -- start division    
        if mult_func(7) = '1' then      --div od divu
          div_counter <= "100010";      --34
          remind      <= (others => '0');
          if mult_func(4) = '1' then    --div
            sign <= dividend(31) xor divisor(31);
            if dividend(31) = '1' then
              speicher <= std_logic_vector(-signed(dividend));
            else
              speicher <= dividend;
            end if;

            if divisor(31) = '1' then
              divis <= std_logic_vector(-signed(divisor));
            else
              divis <= divisor;
            end if;
            
          else                          --divu
            sign     <= '0';
            speicher <= dividend;
            divis    <= divisor;
          end if;
          
        end if;
      end if;
      if reset = '1' then
        div_counter <= (others => '0');
      end if;
    end if;

  end process;


  process(cpu_clk)
  begin
    if rising_edge(cpu_clk) then
      if div_counter = "100001" then
        
        div_busy <= '1';
      end if;
      if div_counter = "00001" then
        div_busy <= '0';
        div_rdy  <= '1';
      else
        div_rdy <= '0';
      end if;
      if reset='1' then
        div_rdy<='0';
        div_busy<='0';
      end if;
    end if;
  end process;

  process(cpu_clk)
  begin
    if rising_edge(cpu_clk) then
      if mult_func(7) = '1' then        --div od divu
        if divisor = X"00000000" then
          div_zero <= '1';
        else
          div_zero <= '0';
        end if;
      end if;
      if reset = '1' then
        div_zero <= '0';
      end if;
    end if;
  end process;
  
end Behavioral;
