---------------------------------------------------------------------
-- TITLE: Instruction execute
-- AUTHOR: Rene Doss
-- Company Dossmatik GmbH
-- DATE CREATED: 27.6.2011
-- FILENAME: inst_execute.vhd
-- PROJECT: MAIS CPU core
--
-- COPYRIGHT: Creative Commons CC BY-NC 3.0 with exception 
-- commercial applicants have to pay a licence fee
--
-- DESCRIPTION:
-- create bus stall
---------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use work.mlite_pack.all;


entity inst_arbiter is
  
  
  port (

    cpu_clk        : in     std_logic;
    Reset          : in     std_logic;
    BREADY         : in     std_logic;
    BACTIVE        : in     std_logic;
    decode         : in     decode_rec;
    bus_wait       : buffer std_logic;
    stall          : buffer std_logic

    );
end inst_arbiter;

architecture Behavioral of inst_arbiter is
  signal stall_counter : std_logic_vector(3 downto 0);

begin



 


-- load
  process(cpu_clk)
  begin
    if rising_edge(cpu_clk) then
      if BREADY = '1' then
        stall_counter <= '0' & stall_counter(3 downto 1);
      end if;
      if stall = '0' and  BREADY = '1' then  ---read
        if decode.mem_source(7 downto 6) = "11" then
          stall_counter <= "0111";
        end if;
      end if;

      if Reset = '1' then
        stall_counter <= (others => '0');
      end if;
    end if;
  end process;


  process(cpu_clk)
  begin
    if rising_edge(cpu_clk) then
      if BREADY = '0' and BACTIVE = '1' and stall_counter = "0000" then
        bus_wait <= '1';
      else
        bus_wait <= '0';
      end if;
      if Reset = '1' then
        bus_wait <= '0';
      end if;
    end if;
  end process;


  stall <= stall_counter(0);

end Behavioral;

