---------------------------------------------------------------------
-- TITLE: Program Counter
-- AUTHOR: Rene Doss
-- Company Dossmatik GmbH
-- DATE CREATED: 27.6.2011
-- FILENAME: inst_fetch.vhd
-- PROJECT: MAIS CPU core
--
-- COPYRIGHT: Creative Commons CC BY-NC 3.0 with exception 
-- commercial applicants have to pay a licence fee
--
-- DESCRIPTION:
-- Implements the Program Counter logic 
---------------------------------------------------------------------


library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use work.mlite_pack.all;


--! this is the PC-Counter logic


entity inst_pc is
  generic (
    ISR_addr : std_logic_vector(31 downto 0);
    SYS_addr : std_logic_vector(31 downto 0));

  port(cpu_clk            : in  std_logic;   --! cpu clock
       reset              : in  std_logic;   --! reset PC to 0x0000_0000  
       jump_ISR           : in  std_logic;
       syscall            : in  std_logic;
       op_break           : in  std_logic;   --! op break
       stall              : in  std_logic;   --! stall
       stall_mult         : in  std_logic;
       debug_pc_hold      : in  std_logic;
       breakC             : in  std_logic;   --! stall_mult 
       exception_pipeline : in  unsigned (2 downto 0);
       BREADY             : in  std_logic;   --! HREADY on Bus
       BACTIVE            : in  std_logic;
       imem_valid         : in  std_logic;
       branch             : in  branch_rec;  --! control the program counter   
       PC_reg             : out std_logic_vector(31 downto 0) := (others => '0');
       PC_plus4           : out std_logic_vector(31 downto 0);
       PC_plus4_d1        : out std_logic_vector(31 downto 0)
       );
end;


architecture Behavioral of inst_pc is
  signal PC_inc    : std_logic_vector(31 downto 2);  --! program counter++ reg
  signal PC        : std_logic_vector(31 downto 2);
  signal pc_inc_d1 : std_logic_vector(31 downto 2);

begin

  pc_inc   <= std_logic_vector(unsigned(PC)+1);  --increment pc
  PC_reg   <= PC & "00";
  PC_plus4 <= pc_inc & "00";
  PC_plus4_d1<=pc_inc_d1 &"00";

  process(cpu_clk)
  begin
    if rising_edge(cpu_clk) then
      if stall = '1' or op_break = '1' or stall_mult = '1' then
      else
        pc_inc_d1 <= PC_inc;
      end if;
    end if;
  end process;


  process(cpu_clk)
  begin
    if rising_edge(cpu_clk) then

      if op_break = '0' and stall = '0' and breakC = '0' and debug_pc_hold = '0' then
        if imem_valid = '1' and not(BREADY = '0' and BACTIVE = '1') then
          PC <= PC_inc;
        end if;
        if branch.branch_execute = '1' and exception_pipeline = "000" then
          PC <= branch.branch_address;
        end if;    
      end if;

      if (branch.branch_execute = '1') and (exception_pipeline = "000") then
        PC <= branch.branch_address;
      end if;

      if op_break = '1' then
        pc <= pc_inc_d1;
      end if;

      if syscall = '1' then
        PC <= SYS_addr (31 downto 2);
      end if;

      if jump_ISR = '1' then
        PC <= ISR_addr (31 downto 2);
      end if;

      if reset = '1' then
        pc <= (others => '0');
      end if;
      
    end if;
  end process;
  

end;  --Behavioral
