---------------------------------------------------------------------
-- TITLE: load_access
--
-- AUTHOR: Rene Doss
-- Company Dossmatik GmbH
-- DATE CREATED: 2.8.2011
-- FILENAME: load_access.vhd
-- PROJECT: MAIS CPU core
--
-- COPYRIGHT: Creative Commons CC BY-NC 3.0 with exception 
-- commercial applicants have to pay a licence fee
-- 
-- DESCRIPTION:
-- Implements the  stage write back
---------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use work.mlite_pack.all;



entity load_access is
  port (
    cpu_clk : in std_logic;
    reset   : in std_logic;
--BBUS    
    BADDR   : in std_logic_vector(31 downto 0);
    BWDATA  : in std_logic_vector(31 downto 0);
    BRD     : in std_logic;
    BRDATA  : in std_logic_vector(31 downto 0);
    BREADY  : in std_logic;

-- from execute
    load_index : in  std_logic_vector (4 downto 0);
    IR_load    : in  std_logic_vector (2 downto 0);
--output
    loadto_reg : out write_reg_rec
    );
end load_access;



architecture Behavioral of load_access is
  signal load_index_d1 : std_logic_vector(4 downto 0);
  signal BWDATA_d1     : std_logic_vector(31 downto 0);
  signal BADDR_d1      : std_logic_vector(31 downto 0);
  signal IR_load_d1    : std_logic_vector (2 downto 0);

  signal a_number, b_number : unsigned(1 downto 0);
  
begin

  process(cpu_clk)
    --bus phase one
  begin
    if rising_edge(cpu_clk) then
      if BRD='1' and BREADY = '1' then
        a_number      <= a_number+1;
        load_index_d1 <= load_index;
        BWDATA_d1     <= BWDATA;
        IR_load_d1    <= IR_load;
        BADDR_d1      <= BADDR;
      end if;
      if reset = '1' then
        a_number <= (others => '0');
      end if;
    end if;
  end process;


  process(cpu_clk)
    --bus phase two
  begin
    if rising_edge(cpu_clk) then
      if b_number /= a_number and BREADY = '1' then
        b_number <= b_number+1;

        case IR_load_d1 is
          when "000" =>                 --000 lb
            if BADDR_d1 (1 downto 0) = "00" then
              loadto_reg.data <= std_logic_vector(resize(signed(BRDATA(31 downto 24)), 32));
            end if;
            if BADDR_d1 (1 downto 0) = "01" then
              loadto_reg.data <= std_logic_vector(resize(signed(BRDATA(23 downto 16)), 32));
            end if;
            if BADDR_d1 (1 downto 0) = "10" then
              loadto_reg.data <= std_logic_vector(resize(signed(BRDATA(15 downto 8)), 32));
            end if;
            if BADDR_d1 (1 downto 0) = "11" then
              loadto_reg.data <= std_logic_vector(resize(signed(BRDATA(7 downto 0)), 32));
            end if;
            
          when "001" =>                 --001 lh
            if BADDR_d1 (1 downto 0) = "00" then
              loadto_reg.data <= std_logic_vector(resize(signed(BRDATA(31 downto 16)), 32));
            end if;
            if BADDR_d1 (1 downto 0) = "10" then
              loadto_reg.data <= std_logic_vector(resize(signed(BRDATA(15 downto 00)), 32));
            end if;
          when "010" =>                 --010 lwl
            if BADDR_d1 (1 downto 0) = "00" then
              loadto_reg.data <= BRDATA;
            end if;
            if BADDR_d1 (1 downto 0) = "01" then
              loadto_reg.data <= BRDATA (23 downto 0) & BWDATA_d1(7 downto 0);
            end if;
            if BADDR_d1 (1 downto 0) = "10" then
              loadto_reg.data <= BRDATA (15 downto 0) & BWDATA_d1(15 downto 0);
            end if;
            if BADDR_d1 (1 downto 0) = "11" then
              loadto_reg.data <= BRDATA (7 downto 0) & BWDATA_d1(23 downto 0);
            end if;
          when "011" =>                 --011 lw
            loadto_reg.data <= BRDATA (31 downto 0);
          when "100" =>                 --100 lbu
            if BADDR_d1 (1 downto 0) = "00" then
              loadto_reg.data <= std_logic_vector(resize(unsigned(BRDATA(31 downto 24)), 32));
            end if;
            if BADDR_d1 (1 downto 0) = "01" then
              loadto_reg.data <= std_logic_vector(resize(unsigned(BRDATA(23 downto 16)), 32));
            end if;
            if BADDR_d1 (1 downto 0) = "10" then
              loadto_reg.data <= std_logic_vector(resize(unsigned(BRDATA(15 downto 8)), 32));
            end if;
            if BADDR_d1 (1 downto 0) = "11" then
              loadto_reg.data <= std_logic_vector(resize(unsigned(BRDATA(7 downto 0)), 32));
            end if;
            
          when "101" =>                 --101 lhu
            if BADDR_d1 (1 downto 0) = "00" then
              loadto_reg.data <= std_logic_vector(resize(unsigned(BRDATA(31 downto 16)), 32));
            else
              loadto_reg.data <= std_logic_vector(resize(unsigned(BRDATA(15 downto 0)), 32));
            end if;
          when "110" =>                 --110 lwr
            if BADDR_d1 (1 downto 0) = "00" then
              loadto_reg.data <= BWDATA_d1(31 downto 8) & BRDATA (31 downto 24);
            end if;
            if BADDR_d1 (1 downto 0) = "01" then
              loadto_reg.data <= BWDATA_d1(31 downto 16) & BRDATA (31 downto 16);
            end if;
            if BADDR_d1 (1 downto 0) = "10" then
              loadto_reg.data <= BWDATA_d1(31 downto 24) & BRDATA (31 downto 8);
            end if;
            if BADDR_d1 (1 downto 0) = "11" then
              loadto_reg.data <= BRDATA;
            end if;
          when others => null;
        end case;
        --write status nach vorne mitgeben 
        loadto_reg.wr    <= '1';
        loadto_reg.index <= load_index_d1;
      else
        loadto_reg.wr <= '0';
      end if;
      if reset = '1' then
        b_number <= (others => '0');
      end if;
    end if;
  end process;

end Behavioral;

