---------------------------------------------------------------------
-- TITLE: Instruction execute
-- AUTHOR: Rene Doss
-- Company Dossmatik GmbH
-- DATE CREATED: 27.6.2011
-- FILENAME: multiplier.vhd
-- PROJECT: MAIS CPU core
-- COPYRIGHT: see Mais Processor Manual
-- 
-- DESCRIPTION:
-- Implements multiplication
---------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

use work.mlite_pack.all;

entity multiplier_HW is
  generic(multiplier : string := "generic";
          divider    : string := "generic");

  port (

    cpu_clk    : in  std_logic;
    reset      : in  std_logic;
    path_a     : in  std_logic_vector(31 downto 0);
    path_b     : in  std_logic_vector(31 downto 0);
    decode     : in  decode_rec;
    mult_ready : out std_logic                     := '0';
    mult_akku  : out std_logic_vector(63 downto 0) := (others => '0')
    );
end multiplier_HW;

architecture Behavioral of multiplier_HW is
  signal mul_execute : std_logic_vector (9 downto 0) := "0000000000";
  signal div_busy    : std_logic                     := '0';
  signal div_rdy     : std_logic                     := '0';
  signal quotient    : std_logic_vector(31 downto 0);
  signal reminder    : std_logic_vector(31 downto 0);


--todo  div_zero not in use!!!!

  signal d1_mult_akku , d2_mult_akku, d3_mult_akku : std_logic_vector(63 downto 0) := (others => '0');
  signal path_a_d1, path_a_d2                      : std_logic_vector(31 downto 0);
  signal path_b_d1, path_b_d2                      : std_logic_vector(31 downto 0);


  component division is
    port (
      cpu_clk   : in  std_logic;
      reset     : in  std_logic;
      dividend  : in  std_logic_vector(31 downto 0);
      divisor   : in  std_logic_vector(31 downto 0);
      mult_func : in  mult_function_type;
      div_zero  : out std_logic;
      div_busy  : out std_logic := '0';
      div_rdy   : out std_logic := '0';
      quotient  : out std_logic_vector(31 downto 0);
      reminder  : out std_logic_vector(31 downto 0)
      );
  end component;
  
begin

----------------------------------------------------76543210
--    constant mult_read_low  :mult_function_type:="00000001";
--    constant mult_write_low :mult_function_type:="00000010";
--    constant mult_read_hi   :mult_function_type:="00000100";
--    constant mult_write_hi  :mult_function_type:="00001000";
--    constant mult_mult      :mult_function_type:="01010000";
--    constant mult_multu     :mult_function_type:="01100000";
--    constant mult_div       :mult_function_type:="10010000";
--    constant mult_divu      :mult_function_type:="10100000";

--mult/div busy (activity)
  process (cpu_clk)
  begin
    if rising_edge(cpu_clk) then
      if div_busy='0' then
         mult_ready <= mul_execute(1) or div_rdy;  --used in inst_fetch
      else
        mult_ready <='0';
      end if;
      ---mult or multu or div or divu
      if decode.mult_func(4) = '1' or decode.mult_func(5) = '1' then
        mul_execute <= (4 => '1', others => '0');
      else
        -- shift right
        mul_execute <= '0' & mul_execute(mul_execute'high downto 1);
      end if;
    end if;
  end process;

--state machine mult/div
  process(cpu_clk)
  begin
    if rising_edge(cpu_clk) then
      if mul_execute(0) = '1' then
        mult_akku <= d1_mult_akku;
      end if;

      if div_rdy = '1' then
        mult_akku(31 downto 0)  <= quotient;
        mult_akku(63 downto 32) <= reminder;
      end if;

      if decode.mult_func(1) = '1' then  --mult_write_low
        mult_akku(31 downto 0) <= path_a;
      end if;
      if decode.mult_func(3) = '1' then  --mult_write_high
        mult_akku(63 downto 32) <= path_a;
      end if;
    end if;
  end process;



  mult : if (multiplier = "generic") generate
    process(cpu_clk)
    begin
      if rising_edge(cpu_clk) then
        path_a_d1    <= path_a;
        path_b_d1    <= path_b;
        path_a_d2    <= path_a_d1;
        path_b_d2    <= path_b_d1;
        d3_mult_akku <= std_logic_vector(signed(path_a_d2)*signed(path_b_d2));
        d2_mult_akku <= d3_mult_akku;
        d1_mult_akku <= d2_mult_akku;
      end if;
    end process;
  end generate mult;





  div : if (multiplier = "generic") generate
    divide : division port map(
      cpu_clk   => cpu_clk,
      reset     => reset,
      dividend  => path_a,
      divisor   => path_b,
      mult_func => decode.mult_func,
      div_zero  => open,
      div_busy  => div_busy,
      div_rdy   => div_rdy,
      quotient  => quotient,
      reminder  => reminder
      );

    
  end generate div;

end Behavioral;

