---------------------------------------------------------------------
-- TITLE: branch
-- AUTHOR: Rene Doss
-- Company Dossmatik GmbH
-- DATE CREATED: 27.6.2011
-- FILENAME: control.vhd
-- PROJECT: MAIS CPU core
--
-- COPYRIGHT: Creative Commons CC BY-NC 3.0 with exception 
-- commercial applicants have to pay a licence fee
-- 
-- DESCRIPTION:
-- Implements  the control of the logic
---------------------------------------------------------------------


library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use work.mlite_pack.all;

package branch_pack is

  function branch_f (

    branch_func : in branch_function_type;
    PC_branch   : in std_logic_vector (31 downto 2);
    opcode_d1   : in std_logic_vector (31 downto 0);
    path_a      : in std_logic_vector (31 downto 0);
    pc_plus4    : in std_logic_vector (31 downto 2);
    compare     : in compare_type;
    CPint       : in CPint_rec)
    return branch_rec;
  
end;

package body branch_pack is

  function branch_f(

    branch_func : in branch_function_type;
    PC_branch   : in std_logic_vector (31 downto 2);
    opcode_d1   : in std_logic_vector (31 downto 0);
    path_a      : in std_logic_vector (31 downto 0);
    pc_plus4    : in std_logic_vector (31 downto 2);
    compare     : in compare_type;
    CPint       : in CPint_rec)
    return branch_rec
    
  is
    variable branch : branch_rec;

    
  begin
    branch.branch_execute := '0';


    --branch to address offset is shifted left 2bits and added to PC
    branch.branch_address := PC_branch(31 downto 2);

    if branch_func(7) = '1' then
      --JALR and JR
      branch.branch_execute := '1';
      branch.branch_address := path_a (31 downto 2);  --reg-rs-data
    end if;

    if branch_func(6) = '1' then
      --J and JAL
      --jump in the current 256MB aligned region
      --the upper bit are corresponding of addressing
      -- in the delay slot   
      branch.branch_execute := '1';
      branch.branch_address := pc_plus4(31 downto 28) & opcode_d1(25 downto 0);
    end if;

    if path_a = X"00000000" then
      if branch_func(3) = '1' then
        branch.branch_execute := '1';
      end if;
    end if;
    -- bit 2  [rs]<0
    if signed(path_a) < 0 then
      if branch_func(2) = '1' then
        branch.branch_execute := '1';
      end if;
    end if;
    if branch_func(1) = '1' then
      if compare(1) = branch_func(0) then
        branch.branch_execute := '1';
      else
        branch.branch_execute := '0';
      end if;
    end if;

    if branch_func(4) = '1' then
      --   if to_integer(signed(reg_rs_data))>0 then
      if compare(2) = '1' then
        branch.branch_execute := '1';
      end if;
    end if;

    if branch_func(10) = '1' then       --eret
      branch.branch_address := CPint.EPC;
      branch.branch_execute := '1';
    end if;

    return branch;
  end branch_f;

end;





