---------------------------------------------------------------------
-- title: instruction execute
-- author: rene doss
-- company dossmatik gmbh
-- date created: 27.6.2011
-- filename: control.vhd
-- project: mais cpu core
--
-- COPYRIGHT: Creative Commons CC BY-NC 3.0 with exception 
-- commercial applicants have to pay a licence fee
-- 
-- description :
-- implements  the control of the logic
---------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use work.mlite_pack.all;
package control_pack is
  function control_f (ir : in std_logic_vector (31 downto 0))
    return control_rec
;
end;

package body control_pack is

  function control_f (ir : in std_logic_vector (31 downto 0))
    return control_rec
  is
    alias op                 : std_logic_vector (5 downto 0) is ir (31 downto 26);
    alias func               : std_logic_vector (5 downto 0) is ir (5 downto 0);
    variable branch_function : branch_function_type;
    variable arith_function  : arith_function_type;
    variable shift_function  : shift_function_type;
    variable logic_function  : logic_function_type;
    variable mult_function   : mult_function_type;
    variable dataflow_v      : dataflow_type;
    variable mem_source      : mem_source_type;
    variable data_input      : data_in_type;
    variable sys_call        : std_logic;
    variable re_store        : std_logic;
-- var for data_in
    variable control_out     : control_rec;
    
  begin
    re_store        := '0';
    sys_call        := '0';
    branch_function := branch_no;
    mem_source      := (others => '0');
    arith_function  := (others => '0');
    shift_function  := (others => '0');
    logic_function  := (others => '0');
    dataflow_v      := (others => '0');
    data_input      := (others => '0');
    mult_function   := (others => '0');

    case op is
      when "000000" =>
--special
        case func is
          when "000000" =>
--sll   r[rd]=r[rt]<<sa;
            data_input     := reg_sa;
            dataflow_v     := "00001010";
            shift_function := "000";
          when "000010" =>
--srl   r[rd]=u[rt]>>sa;
            data_input     := reg_sa;
            dataflow_v     := "00001010";
            shift_function := "001";
          when "000011" =>
--sra   r[rd]=r[rt]>>sa;
            data_input     := reg_sa;
            dataflow_v     := "00001010";
            shift_function := "011";
          when "000100" =>
--sllv  r[rd]=r[rt]<<r[rs];
            data_input     := reg_reg;
            dataflow_v     := "00001010";
            shift_function := "100";
          when "000110" =>
--srlv  r[rd]=u[rt]>>r[rs];
            data_input     := reg_reg;
            dataflow_v     := "00001010";
            shift_function := "101";
          when "000111" =>
--srav  r[rd]=r[rt]>>r[rs];
            data_input     := reg_reg;
            dataflow_v     := "00001010";
            shift_function := "111";
          when "001000" =>
--jr    s->pc_next=r[rs];
--jump register
            data_input      := reg_reg;
            branch_function := jump_rs;
          when "001001" =>
--jalr  r[rd]=s->pc_next; s->pc_next=r[rs];
--jump and link register rs
            data_input      := reg_reg;
            branch_function := jump_al_rs;
            dataflow_v      := "00000000";
          when "001010" =>
--movz  if(!r[rt]) r[rd]=r[rs]; /*iv*/
          when "001011" =>
--movn  if(r[rt]) r[rd]=r[rs];  /*iv*/
          when "001100" =>
--syscall                                
            sys_call := '1';
          when "001101" =>


--break s->wakeup=1;
          when "001111" =>
--sync  s->wakeup=1;


          when "010000" =>

--mfhi  r[rd]=s->hi;
            mult_function := mult_read_hi;
            data_input    := reg_reg;
            dataflow_v    := "01000000";
          when "010001" =>
--mthi  s->hi=r[rs];
            mult_function := mult_write_hi;

          when "010010" =>
--mflo  r[rd]=s->lo;
            mult_function := mult_read_low;
            data_input    := reg_reg;
            dataflow_v    := "01000000";
          when "010011" =>
--mtlo  s->lo=r[rs];
            mult_function := mult_write_low;


          when "011000" =>
--mult  s->lo=r[rs]*r[rt]; s->hi=0;
            mult_function := mult_mult;
          when "011001" =>
--multu s->lo=r[rs]*r[rt]; s->hi=0;
            mult_function := mult_multu;
          when "011010" =>
--div   s->lo=r[rs]/r[rt]; s->hi=r[rs]%r[rt];
            mult_function := mult_div;
          when "011011" =>
--divu  s->lo=r[rs]/r[rt]; s->hi=r[rs]%r[rt];
            mult_function := mult_divu;
            
          when "100000" =>
--add   r[rd]=r[rs]+r[rt];
            data_input     := reg_reg;
            dataflow_v     := "00001001";
            arith_function := "0001";
          when "100001" =>
--addu  r[rd]=r[rs]+r[rt];
            data_input     := reg_reg;
            dataflow_v     := "00001001";
            arith_function := "0001";
          when "100010" =>
--sub   r[rd]=r[rs]-r[rt];
            data_input     := reg_reg;
            dataflow_v     := "00001001";
            arith_function := "0010";
          when "100011" =>
--subu  r[rd]=r[rs]-r[rt];
            data_input     := reg_reg;
            dataflow_v     := "00001001";
            arith_function := "0010";
          when "100100" =>
--and   r[rd]=r[rs]&r[rt];
            data_input     := reg_reg;
            dataflow_v     := "00001100";
            logic_function := "00010";
          when "100101" =>
--or    r[rd]=r[rs]|r[rt];
            data_input     := reg_reg;
            dataflow_v     := "00001100";
            logic_function := "00001";
          when "100110" =>
--xor   r[rd]=r[rs]^r[rt];
            data_input     := reg_reg;
            dataflow_v     := "00001100";
            logic_function := "00100";
          when "100111" =>
--nor   r[rd]=~(r[rs]|r[rt]);
            data_input     := reg_reg;
            dataflow_v     := "00001100";
            logic_function := "01000";
          when "101010" =>
--slt   r[rd]=r[rs]<r[rt];
            data_input := reg_reg;
            dataflow_v := "00011000";
          when "101011" =>
--sltu  r[rd]=u[rs]<u[rt];
            data_input := reg_reg;
            dataflow_v := "00101000";
          when "101101" =>
--daddu r[rd]=r[rs]+u[rt]; mips iii
          when "110001" =>
--tgeu   trap  mips ii
          when "110010" =>
--tlt
          when "110011" =>
--tltu
          when "110100" =>
--teq 
          when "110110" =>
--tne 
          when others   =>
        end case;
      when "000001" =>
--regimm
        case ir (20 downto 16) is
          when "00000" =>
--bltz    branch=r[rs]<0;
--branch on less then equal zero
            branch_function := branch_ltz;
            data_input      := rs_zero;
            dataflow_v      := "00000000";
          when "00001" =>
--bgez    branch=r[rs]>=0;
--branch on greater then equal zero
            data_input      := rs_zero;
            branch_function := branch_gez;
            dataflow_v      := "00000000";
          when "00010" =>
--bltzl   lbranch=r[rs]<0;
            data_input      := rs_zero;
            branch_function := branch_ltzal;
            dataflow_v      := "00000000";
          when "00011" =>
--bgezl   lbranch=r[rs]>=0;
            branch_function := branch_gezal;
            data_input      := rs_zero;
            dataflow_v      := "00000000";
          when "10000" =>
--bltzal  r[31]=s->pc_next; branch=r[rs]<0;
            data_input      := rs_zero;
            branch_function := branch_ltzal;
            dataflow_v      := "00000000";
          when "10001" =>
--bgezal  r[31]=s->pc_next; branch=r[rs]>=0;
            data_input      := rs_zero;
            branch_function := branch_gezal;
            dataflow_v      := "00000000";
          when "10010" =>
--bltzall r[31]=s->pc_next; lbranch=r[rs]<0;
            data_input      := rs_zero;
            branch_function := branch_ltzal;
            dataflow_v      := "00000000";
          when "10011" =>
--bgezall r[31]=s->pc_next; lbranch=r[rs]>=0;
-- ? possible not correct. i can not distinguish  bgezal and bgzall
            data_input      := rs_zero;
            branch_function := branch_gezal;
            dataflow_v      := "00000000";
          when others =>
        end case;
      when "000011" =>
--jal    r[31]=pc_next; jump and link
        branch_function := jump_al;
        dataflow_v      := "00000000";
      when "000010" =>
--j      jump; 
        branch_function := jump;
        dataflow_v      := "00000000";

--  two register instruction      
      when "000100" =>
--beq    branch=r[rs]==r[rt];
        data_input      := reg_reg;
        branch_function := branch_eq;
        dataflow_v      := "00000000";
      when "000101" =>
--bne    branch=r[rs]!=r[rt];
--also pseudo instruction bnez -> bne r[rs]=r[zero];
        data_input      := reg_reg;
        branch_function := branch_ne;
        dataflow_v      := "00000000";
      when "000110" =>
--blez   branch=r[rs]<=0;
        data_input      := reg_reg;
        branch_function := branch_lez;
        dataflow_v      := "00000000";
      when "000111" =>
--bgtz   branch=r[rs]>0;
        data_input      := reg_reg;
        branch_function := branch_gtz;
        dataflow_v      := "00000000";
      when "001000" =>
--addi   r[rt]=r[rs]+(short)imm;
        data_input     := reg_imm;
        dataflow_v     := "00001001";
        arith_function := "0001";
      when "001001" =>
--addiu  u[rt]=r[rs]+(short)imm;
        data_input     := reg_imm;
        dataflow_v     := "00001001";
        arith_function := "0001";
      when "001010" =>
--slti   r[rt]=r[rs]<(short)imm;
        data_input := reg_imm;
        data_input := reg_imm;
        dataflow_v := "00011000";
      when "001011" =>
--sltiu  u[rt]=u[rs]<(unsigned long)(short)imm;
        data_input := reg_imm;
        data_input := reg_imm;
        dataflow_v := "00101000";
      when "001100" =>
--andi   r[rt]=r[rs]&imm;
        data_input     := reg_imm;
        dataflow_v     := "00001100";
        logic_function := "00010";
      when "001101" =>
--ori    r[rt]=r[rs]|imm;
        data_input     := reg_imm;
        dataflow_v     := "00001100";
        logic_function := "00001";
      when "001110" =>
--xori   r[rt]=r[rs]^imm;
        data_input     := reg_imm;
        dataflow_v     := "00001100";
        logic_function := "00100";
      when "001111" =>
--lui    r[rt]=(imm<<16);
        data_input     := reg_imm;
        dataflow_v     := "00001100";
        logic_function := "10000";

--cop0                          
      when "010000" =>

        if ir (25 downto 21) = "00000" then

--move from cp0
          data_input     := cp0_input;
          dataflow_v     := "00001100";
          logic_function := "00001";

--path_a or r[zero]->r[rt]
        end if;

--Optimierung zusammenfassung eret und rfe moeglich                             
        if ir (5 downto 0) = "011000" then
--eret exception return
          branch_function := eret;
          re_store        := '1';
        end if;
        if ir (5 downto 0) = "010000" then
--rfe  restore from exception                   
          re_store := '1';
        end if;




--                      when "010001" =>
--cop1
--                      when "010010" =>
--cop2
--                      when "010011" =>

--cop3
-- branch likely 
      when "010100" =>
--beql   lbranch=r[rs]==r[rt];
        data_input      := reg_reg;
        branch_function := branch_eq;
        dataflow_v      := "00000000";
      when "010101" =>
--bnel   lbranch=r[rs]!=r[rt];
        data_input      := reg_reg;
        branch_function := branch_ne;
        data_input      := reg_reg;
        dataflow_v      := "00000000";
      when "010110" =>
--blezl  lbranch=r[rs]<=0;
        data_input      := reg_reg;
        branch_function := branch_lez;
        dataflow_v      := "00000000";
      when "010111" =>
--bgtzl  lbranch=r[rs]>0;
        data_input      := reg_reg;
        branch_function := branch_gtz;
        dataflow_v      := "00000000";
      when "011100" =>
-- clo count leading 
        data_input := reg_reg;
        dataflow_v := "00001001";
        if ir(0) = '1' then
          --count leading ones
          arith_function := "0100";
        else
          --scount leading zeros
          arith_function := "1000";
        end if;

--load and store instructions      
--address=(short)imm+r[rs];
      when "100000" =>
--lb     r[rt]=*(signed char*)ptr;
        data_input     := reg_imm;
        arith_function := "0001";
        dataflow_v     := "10000001";
        mem_source     := "11001001";
      when "100001" =>
--lh     r[rt]=*(signed short*)ptr;
        data_input     := reg_imm;
        arith_function := "0001";
        dataflow_v     := "10000001";
        mem_source     := "11001010";
      when "100010" =>
--lwl    //not implemented
        data_input     := reg_imm;
        arith_function := "0001";
        dataflow_v     := "10000001";
        mem_source     := "11010100";
      when "100011" =>
--lw     r[rt]=*(long*)ptr;
        data_input     := reg_imm;
        arith_function := "0001";
        dataflow_v     := "10000001";
        mem_source     := "11000100";
      when "100100" =>
--lbu    r[rt]=*(unsigned char*)ptr;
        data_input     := reg_imm;
        arith_function := "0001";
        dataflow_v     := "10000001";
        mem_source     := "11010001";
      when "100101" =>
--lhu    r[rt]=*(unsigned short*)ptr;
        data_input     := reg_imm;
        arith_function := "0001";
        dataflow_v     := "10000001";
        mem_source     := "11000010";
      when "100110" =>
--lwr   
        data_input     := reg_imm;
        arith_function := "0001";
        dataflow_v     := "10000001";
        mem_source     := "11100100";
      when "101000" =>
--sb     *(char*)ptr=(char)r[rt];
        data_input     := reg_imm;
        arith_function := "0001";
        dataflow_v     := "10000001";
        mem_source     := "10000001";
      when "101001" =>
--sh     *(short*)ptr=(short)r[rt];
        data_input     := reg_imm;
        arith_function := "0001";
        dataflow_v     := "10000001";
        mem_source     := "10000010";
      when "101010" =>
--swl   
        data_input     := reg_imm;
        arith_function := "0001";
        dataflow_v     := "10000001";
        mem_source     := "10010000";


--  mem_source := mem_write32;  --address=(short)imm+r[rs];
      when "101011" =>
--sw     *(long*)ptr=r[rt];
        data_input     := reg_imm;
        arith_function := "0001";
        dataflow_v     := "10000001";
        mem_source     := "10001100";


--  mem_source := mem_write32;  --address=(short)imm+r[rs];
      when "101110" =>
--swr    //not implemented
        data_input     := reg_imm;
        arith_function := "0001";
        dataflow_v     := "10000001";
        mem_source     := "10100000";
      when "101111" =>
--cache
      when "110000" =>
--ll     r[rt]=*(long*)ptr;
      when "110001" =>
--lwc1 
      when "110010" =>
--lwc2 
      when "110011" =>
--lwc3 
      when "110101" =>
--ldc1 
      when "110110" =>
--ldc2 
      when "110111" =>
--ldc3 
      when "111000" =>
--sc     *(long*)ptr=r[rt]; r[rt]=1;
      when "111001" =>
--swc1 
      when "111010" =>
--swc2 
      when "111011" =>
--swc3 
      when "111101" =>
--sdc1 
      when "111110" =>
--sdc2 
      when "111111" =>
--sdc3 
      when others   =>
    end case;





    control_out.syscall    := sys_call;
    control_out.restore    := re_store;
    control_out.arith_func := arith_function;
    control_out.shift_func := shift_function;
    control_out.logic_func := logic_function;
    control_out.mult_func  := mult_function;



    control_out.data_in        := data_input;
    control_out.branch_func    := branch_function;
    control_out.dataflow       := dataflow_v;
    control_out.mem_source_out := mem_source;
    return control_out;
  end;
  --logic
end;

