library ieee;
use ieee.STD_LOGIC_1164.all;

use ieee.numeric_std.all;

use work.SPB.all;

entity bus_dummy is
  generic(memory_type : string                       := "DEFAULT";
          sel         : integer := 15);

  port(BCLK     : in  std_logic;
       BRESET   : in  std_logic;
       BBUS_in  : in  SPB_in;
       BBUS_out : out SPB_out);
end;  --entity 

architecture logic of bus_dummy is
  
  constant addresswidth : integer := 8;
  type     ram_byte is array (0 to (2**(addresswidth-1)))
    of std_logic_vector (7 downto 0);


--32 bit
  signal RAM_BLOCK0 : ram_byte;
  signal RAM_BLOCK1 : ram_byte;
  signal RAM_BLOCK2 : ram_byte;
  signal RAM_BLOCK3 : ram_byte;


  signal waitstate : std_logic_vector (4 downto 0) := "11111";

begin

  ram_proc : process(BCLK)

  begin
    if rising_edge(BCLK) then

      if BBUS_in.BWR = '1' and BBUS_in.BREADY = '1' then
        if BBUS_in.BSEL(SEL) = '1' then
          if BBUS_in.BMASK(3) = '1' then
            RAM_BLOCK0(to_integer(unsigned(BBUS_in.BADDR(addresswidth-1 downto 2)))) <= BBUS_in.BWDATA(31 downto 24);
          end if;
          if BBUS_in.BMASK(2) = '1' then
            RAM_BLOCK1(to_integer(unsigned(BBUS_in.BADDR(addresswidth-1 downto 2)))) <= BBUS_in.BWDATA(23 downto 16);
          end if;
          if BBUS_in.BMASK(1) = '1' then
            RAM_BLOCK2(to_integer(unsigned(BBUS_in.BADDR(addresswidth-1 downto 2)))) <= BBUS_in.BWDATA(15 downto 8);
          end if;
          if BBUS_in.BMASK(0) = '1' then
            RAM_BLOCK3(to_integer(unsigned(BBUS_in.BADDR(addresswidth-1 downto 2)))) <= BBUS_in.BWDATA(7 downto 0);
          end if;
        end if;
      end if;
    end if;
  end process;


  process(BCLK)
  begin
    if rising_edge(BCLK) then
      waitstate <= '1' & waitstate(4 downto 1);
      if BBUS_in.BACTIVE = '1' and BBUS_in.BREADY = '1' and BBUS_in.BSEL(SEL) = '1' then
        case BBUS_IN.BADDR(9 downto 8) is
          when "00"   => waitstate <= "11111";
          when "01"   => waitstate <= "11110";
          when "10"   => waitstate <= "11100";
          when "11"   => waitstate <= "11000";
          when others => null;
        end case;
      end if;

      if BRESET = '1' then
        waitstate <= "11111";
      end if;
    end if;
  end process;

  BBUS_out.BREADYOUT <= waitstate(0);





  process(BCLK)
  begin
    if rising_edge(BCLK) then
      BBUS_OUT.BRDATA <= RAM_BLOCK0 (to_integer(unsigned(BBUS_IN.BADDR(addresswidth-1 downto 2))))
                                &RAM_BLOCK1 (to_integer(unsigned(BBUS_IN.BADDR(addresswidth-1 downto 2))))
                                &RAM_BLOCK2 (to_integer(unsigned(BBUS_IN.BADDR(addresswidth-1 downto 2))))
                                &RAM_BLOCK3 (to_integer(unsigned(BBUS_IN.BADDR(addresswidth-1 downto 2))));
    end if;
  end process;
end;
