---------------------------------------------------------------------
-- title: package
-- author: rene doss
-- company dossmatik gmbh
-- date created: 1.9.2013
-- filename: pkg_mlite.vhd
-- project: mais cpu core
--
-- COPYRIGHT: Creative Commons CC BY-NC 3.0 with exception 
-- commercial applicants have to pay a licence fee
-- 
-- description :
-- simple pipelined bus
---------------------------------------------
library ieee;
use ieee.std_logic_1164.all;

package SPB is
  function BSEL_f (higher_addr : in std_logic_vector(31 downto 28) )
     return std_logic_vector;

  constant sel_size : positive := 16; --return width of BSEL_f

  type SPB_in is
  record
    BADDR   : std_logic_vector (31 downto 0);
    BSEL    : std_logic_vector (sel_size-1 downto 0);
    BACTIVE : std_logic;
    BRD     : std_logic;                --READ
    BWR     : std_logic;                --WRITE
    BMASK   : std_logic_vector (3 downto 0);
    BWDATA : std_logic_vector (31 downto 0);
    BREADY  : std_logic;
    
  end record;

  type SPB_out is
  record
    BRDATA    : std_logic_vector (31 downto 0);
    BREADYOUT : std_logic;
  end record;

end;

package body SPB is

function BSEL_f (higher_addr : in std_logic_vector(31 downto 28) )
    return std_logic_vector
  is
  
    variable var    : std_logic_vector(15 downto 0);
    
  begin
  
    case higher_addr(31 downto 28) is
      when "0000" =>    var:="0000000000000001";  --0
      when "0001" =>    var:="0000000000000010";  --1
      when "0010" =>    var:="0000000000000100";  --2
      when "0011" =>    var:="0000000000001000";  --3
      when "0100" =>    var:="0000000000010000";  --4
      when "0101" =>    var:="0000000000100000";  --5
      when "0110" =>    var:="0000000001000000";  --6    
      when "0111" =>    var:="0000000010000000";  --7
      when "1000" =>    var:="0000000100000000";  --8
      when "1001" =>    var:="0000001000000000";  --9
      when "1010" =>    var:="0000010000000000";  --A
      when "1011" =>    var:="0000100000000000";  --B
      when "1100" =>    var:="0001000000000000";  --C
      when "1101" =>    var:="0010000000000000";  --D
      when "1110" =>    var:="0100000000000000";  --E    
      when "1111" =>    var:="1000000000000000";  --F   
      when others =>    var:="0000000000000000";                                            
    end case;

    return var;
  end;
  

end;
