---------------------------------------------------------------------
-- TITLE:  pkg_SPB_devices.vhd                                     --
-- AUTHOR:  autmatic generated by component collector tool         --
-- Company Dossmatik GmbH                                          --
-- DATE CREATED:22.4.2013 - 13:3
-- FILENAME:  pkg_SPB_devices.vhd                                  --
-- PROJECT: MAIS CPU core    
--
-- COPYRIGHT: Creative Commons CC BY-NC 3.0 with exception 
-- commercial applicants have to pay a licence fee
--                                      --
-- DESCRIPTION:                                                    --
---------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use work.SPB.all;


package SPBdevice is



  component MAIS_CPU is
    generic (IRQ_number  : integer;
              ISR_addr   : std_logic_vector(31 downto 0);
              SYS_addr   : std_logic_vector(31 downto 0);
              multiplier : string := "generic");

    port(bclk            : in std_logic;
         breset          : in std_logic;
         IRQ             : in std_logic_vector(IRQ_number-1 downto 0);

         ---Instruction RAM
         imem_data       : in  std_logic_vector(31 downto 0);
         imem_valid      : in  std_logic;
         imem_address    : out std_logic_vector(31 downto 0);
         ---SPBlight BUS
         BBUS_in         : out SPB_in;
         BBUS_out        : in  SPB_out);
  end component;
  

  component SPB_memory
    port(
      BCLK         : in  std_logic;
      BBUS_in      : in  SPB_in;
      BBUS_out     : out SPB_out;
      imem_data    : out std_logic_vector(31 downto 0);
      imem_valid   : out std_logic;
      imem_address : in  std_logic_vector(31 downto 0)
      );
  end component;


  component bus_dummy
    generic(memory_type  : string                       := "DEFAULT";
            sel          : integer := 15);      
    port(BCLK            : in  std_logic;
         BRESET          : in  std_logic;
         BBUS_in         : in  SPB_in;
         BBUS_out        : out SPB_out);
  end component;     



  component SPB_sseg is
    generic(
      clk_freq : integer;
      BSEL     : integer);

    port(BCLK     : in  std_logic;
         BBUS_in  : in  SPB_in;
         BBUS_out : out SPB_out;

         ---Display
         sseg : out std_logic_vector (7 downto 0)

         );
  end component;

  component logic_analyser is

  generic (depth           : integer:=10;
           sel             : integer;
           number_of_wires : integer:=128);

  port(
    BCLK          : in  std_logic;
    Hreset        : in  std_logic;
    BBUS_in       : in  SPB_in;
    BBUS_out      : out SPB_out;

    trig_ch       : in std_logic_vector(3 downto 0);
    wires         : in  std_logic_vector (127 downto 0) -- have to equal number_of_wires
    );
end component;

  component SPB_spi_master is
    generic(
      slavenumber       : integer  := 1;
      BCLK_freq         : integer;
      BSEL              : integer;
      preset_speed      : std_logic_vector (7 downto 0) := "00000100";
      preset_shiftwidth : std_logic_vector (7 downto 0) := "00001000"
      );


    port(BCLK     : in  std_logic;
         BRESET   : in  std_logic;
         BBUS_in  : in  SPB_in;
         BBUS_out : out SPB_out;

         -- possibile interrupt source 
         spi_irq : out std_logic;

         --Hardwarepins SPI
         SPI_clk : out std_logic;       --SPI Clock
         SS      : out std_logic_vector(slavenumber-1 downto 0);  --SS slave select
         miso    : in  std_logic;       --master in slave out
         mosi    : out std_logic        --master out slave in 
         );
  end component;


  component SPB_I2C_master is
    generic(
      BCLK_freq   : integer;
      BSEL        : integer
    );

    port(BCLK     : in  std_logic;
         BRESET   : in  std_logic;
         BBUS_in  : in  SPB_in;
         BBUS_out : out SPB_out;

         -- possibile interrupt source 
         I2C_irq  : out std_logic;

         --Hardwarepins 
         SCL      : out std_logic;         --SCL Clock
         SDA      : inout  std_logic       --SDA
         );
  end component;  --SPB_I2C_master




  component SPB_uart is
    generic(
      sim      : string := "no";
      BSEL     : integer;
      baudrate : integer;
      clk_freq : integer);

    port(BCLK     : in  std_logic;
         BRESET   : in  std_logic;
         BBUS_in  : in  SPB_in;
         BBUS_out : out SPB_out;

         uart_irq : out std_logic;

         -- possibility for interrupt source

         --Hardwarepin
         rx : in  std_logic;
         tx : out std_logic
         --normaly connected to
         --RS232 level converter
         );
  end component;



  component SPB_vga_char is
    generic(
      BSEL : integer);

    port(BCLK     : in  std_logic;
         BBUS_in  : in  SPB_in;
         BBUS_out : out SPB_out;


         clk25     : in  std_logic;     --! clock for VGA
         --VGA pins
         blue_out  : out std_logic;
         red_out   : out std_logic;
         green_out : out std_logic;
         hs_out    : out std_logic;
         vs_out    : out std_logic
         );
  end component;


end;
