---------------------------------------------------------------------
-- TITLE: Uart 
-- AUTHOR: Rene Doss
-- Company Dossmatik GmbH
-- DATE CREATED: 27.11.2011
-- FILENAME: UART_RX_8N1.vhd
-- PROJECT: MAIS CPU core
--
-- COPYRIGHT: Creative Commons CC BY-NC 3.0 with exception 
-- commercial applicants have to pay a licence fee
-- 
-- DESCRIPTION:
-- Implements  receive timing
---           
---------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.all;
use ieee.numeric_std.all;

--Uart receiver without parity

entity Uart_8N1_RX is

  generic (clk_freq : integer;
           baudrate : integer);

  port(
    clk           : in  std_logic;
    reset         : in  std_logic;
    receive_data  : out std_logic_vector(7 downto 0);
    receive_ack   : in  std_logic;
    receive_empty : out std_logic;
    rx            : in  std_logic
    );
end Uart_8N1_RX;


architecture Behavioral of Uart_8N1_RX is

  type   state_type is (idle, start, data0, data1, data2, data3, data4, data5, data6, data7, stop);
  signal state, nextstate : state_type;

  signal puffer : std_logic_vector (7 downto 0);

--FIFO
  type RAM is array (0 to 63) of std_logic_vector (7 downto 0);

  signal fifo : RAM := (
                       others => x"00");
  signal nextwrite  : unsigned(5 downto 0);
  signal nextread   : unsigned(5 downto 0);
  signal write_fifo : std_logic;

  signal tick_counter : unsigned (10 downto 0);

  constant tick : integer := clk_freq/baudrate;


begin
  
  receive_data  <= fifo(to_integer(nextread));
  receive_empty <= '1' when nextread = nextwrite else '0';


  process (clk)
  begin
    if rising_edge(clk) then
      if reset = '1' then
        nextread <= (others => '0');
      elsif receive_ack = '1' then
        nextread <= nextread+1;
      end if;
    end if;
  end process;

  process(clk)
  begin
    if rising_edge(clk)then
      state <= nextstate;
    end if;
  end process;

  process(clk)
  begin
    if rising_edge(clk) then
      if reset = '1' then
        nextwrite <= (others => '0');
        
        
      elsif state = stop and nextstate = idle then
        fifo(to_integer(nextwrite)) <= puffer;
        nextwrite                   <= nextwrite+1;
      end if;
    end if;
  end process;


  process(clk)
  begin
    
    if (clk'event and clk = '1') then
      tick_counter <= tick_counter + 1;

      case state is
--schaut nach ob auf RX etwas passiert          
        when idle =>
          tick_counter <= (others => '0');
          if (rx = '0') then
            nextstate <= start;
          else
            nextstate <= idle;
          end if;
--synchronisert auf eine halbe Verschiebung             
        when start =>
          if (tick_counter = tick/2) then
            tick_counter <= (others => '0');
            nextstate    <= data0;
          end if;
          
        when data0 =>
          if (tick_counter = tick) then
            puffer (0)   <= rx;
            tick_counter <= (others => '0');
            nextstate    <= data1;
          end if;
        when data1 =>
          if (tick_counter = tick) then
            puffer (1)   <= rx;
            tick_counter <= (others => '0');
            nextstate    <= data2;
          end if;
        when data2 =>
          if (tick_counter = tick) then
            puffer (2)   <= rx;
            tick_counter <= (others => '0');
            nextstate    <= data3;
          end if;
        when data3 =>
          if (tick_counter = tick) then
            puffer(3)    <= rx;
            tick_counter <= (others => '0');
            nextstate    <= data4;
          end if;
        when data4 =>
          if (tick_counter = tick) then
            puffer (4)   <= rx;
            tick_counter <= (others => '0');
            nextstate    <= data5;
          end if;
        when data5 =>
          if (tick_counter = tick) then
            puffer (5)   <= rx;
            tick_counter <= (others => '0');
            nextstate    <= data6;
          end if;
        when data6 =>
          if (tick_counter = tick) then
            puffer (6)   <= rx;
            tick_counter <= (others => '0');
            nextstate    <= data7;
          end if;
        when data7 =>
          if (tick_counter = tick) then
            puffer (7)   <= rx;
            tick_counter <= (others => '0');
            nextstate    <= stop;
          end if;
        when stop =>
          if (tick_counter = tick) then
            tick_counter <= (others => '0');
            nextstate    <= idle;
          end if;

      end case;

    end if;


  end process;
end Behavioral;

