---------------------------------------------------------------------
-- TITLE: Uart 
-- AUTHOR: Rene Doss
-- Company Dossmatik GmbH
-- DATE CREATED: 27.11.2011
-- FILENAME: uart.vhd
-- PROJECT: MAIS CPU core
--
-- COPYRIGHT: Creative Commons CC BY-NC 3.0 with exception 
-- commercial applicants have to pay a licence fee
-- 
-- DESCRIPTION:
-- Implements  Uart top
---           
---------------------------------------------------------------------
library ieee;

use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use work.mlite_pack.all;
use work.SPB.all;


entity SPB_uart is
  generic(
    sim         : string := "no";
    log_file    : string := "UNUSED";
    clk_freq    : integer;
    baudrate    : integer;
    BSEL        : integer);

  port(BCLK     : in  std_logic;
       BRESET   : in std_logic;
       BBUS_in  : in  SPB_in;
       BBUS_out : out SPB_out;
       -- possibility for interrupt source    
       uart_irq : out std_logic;
       --Hardwarepin
       rx       : in  std_logic;
       tx       : out std_logic
       --normaly connected to
       --RS232 level converter
       );
end;  --entity uart



architecture logic of SPB_uart is
 
  signal write_en      : std_logic;
  signal send_busy     : std_logic;
  signal send_empty    : std_logic;
  signal UART_out      : std_logic_vector (7 downto 0);
  
  signal receive_ack   : std_logic;
  signal receive_empty : std_logic;
  signal receive_data  : std_logic_vector (7 downto 0);

  signal line_length   : natural := 0;


  component Uart_8N1_TX is
    generic(
      clk_freq    : integer;
      baudrate    : integer);
    port(
      clk         : in std_logic;    ---CPU_CLK            
      reset       : in std_logic;

      send_data   : in  std_logic_vector(7 downto 0);
      write_en    : in  std_logic;
      send_busy   : out std_logic;
      send_empty  : out std_logic;
      tx          : out std_logic);
  end component;

  component Uart_8N1_RX is
    generic (
      clk_freq : integer;
      baudrate : integer);

    port(
      clk           : in  std_logic;
      reset         : in  std_logic;
      receive_data  : out std_logic_vector(7 downto 0);
      receive_ack   : in  std_logic;
      receive_empty : out std_logic;
      rx            : in  std_logic
      );
  end component;


begin
  
  process(BCLK)
  begin
    if rising_edge(Bclk) then
      uart_irq <= not receive_empty;
    end if;
  end process;


  process(BCLK)  --write
  begin    
    if rising_edge (BCLK) then
      write_en <= '0';
      if BBUS_in.BSEL(BSEL) = '1'  and BBUS_in.BWR = '1' then
        if BBUS_in.BREADY = '1' and BBUS_in.BADDR(3) = '1' then
          write_en<='1';
          UART_OUT<=BBUS_in.BWDATA(7 downto 0);
        end if;
      end if;
    end if;
  end process;
         
 BBUS_out.BREADYOUT<='1';
  process(BCLK) --read
  begin
    if rising_edge(BCLK) then
        receive_ack <= '0';
      if BBUS_in.BSEL(BSEL) = '1' and BBUS_in.BRD = '1' then
        if  BBUS_in.BADDR(2) = '1' then
          BBUS_out.BRDATA <="0000000" & send_empty & "0000000" & send_busy & X"00" & "0000000" & receive_empty;
        end if;
        if  BBUS_in.BADDR(2) = '0' then
          BBUS_out.BRDATA <= X"000000"& receive_data;
          if BBUS_in.BREADY = '1' then 
            receive_ack <= '1';
          end if;
        end if;
      end if;
    end if;
  end process;
      


  Uart_tx : Uart_8N1_TX
    generic map(clk_freq => clk_freq,
                baudrate => baudrate)

    port map(
      clk       => bclk,
      reset     => BRESET,
      send_data => uart_out,
      write_en  => write_en,
      send_busy => send_busy,
      send_empty=> send_empty,      
      tx        => tx);

  Uart_rx : Uart_8N1_RX
    generic map(
      clk_freq => clk_freq,
      baudrate => baudrate)

    port map(
      clk           => Bclk,
      reset         => Breset,
      receive_data  => receive_data,
      receive_ack   => receive_ack,
      receive_empty => receive_empty,
      rx            => rx);



--  uart_logger :
--  if log_file /= "UNUSED" and sim = "yes" generate
--    uart_proc : process(bclk)
--      file store_file        : text open write_mode is log_file;
--      variable hex_file_line : line;
--      variable c             : character;
--      variable index         : integer;  --natural;
--      
--    begin
--      if rising_edge(bclk) then
--        if write_en = '1' then
--          index := to_integer(unsigned(uart_out (7 downto 0)));
--          if index /= 10 then
--            c           := character'val(index);
--            write(hex_file_line, c);
--            line_length <= line_length + 1;
--          end if;
--          if index = 10 or line_length > 39 then
--
--            writeline(store_file, hex_file_line);
--            line_length <= 0;
--          end if;
--        end if;
--      end if;
--    end process;  --uart_proc
--  end generate;  --uart_logger


end;  --architecture logic
