library ieee;
use ieee.STD_LOGIC_1164.all;

use ieee.numeric_std.all;

use work.SPB.all;

entity SPB_memory is
   generic(memory_type : string := "DEFAULT");
   port(BCLK          : in std_logic;
        BBUS_in       : in SPB_in;
        BBUS_out      : out SPB_out;

        imem_address  : in std_logic_vector(31 downto 0);
        imem_data     : out std_logic_vector(31 downto 0);
        imem_valid    : out std_logic);
end; --entity 
architecture logic of SPB_memory is
 
constant addresswidth : integer:= 13;
type ram_byte is array (0 to (2**(addresswidth-1)))
         of std_logic_vector (7 downto 0);

signal BADDR_d1: std_logic_vector (31 downto 0);
signal write_d1: std_logic;
signal BRDATA  : std_logic_vector (31 downto 0);

--block0  bit 31...24
signal RAM_BLOCK0:ram_byte:=(
X"40",X"3c",X"36",X"34",X"34",X"34",X"34",X"40",X"24",X"24",X"24",X"40",X"34",X"a2",X"34",X"a2",
X"34",X"a2",X"34",X"a2",X"a2",X"a2",X"0c",X"34",X"a2",X"34",X"a2",X"34",X"a2",X"34",X"a2",X"a2",
X"a2",X"34",X"a2",X"34",X"34",X"00",X"00",X"a2",X"a2",X"a2",X"34",X"a2",X"34",X"34",X"00",X"00",
X"a2",X"a2",X"a2",X"34",X"a2",X"34",X"a2",X"34",X"a2",X"34",X"a2",X"34",X"a2",X"a2",X"a2",X"34",
X"a2",X"34",X"34",X"00",X"a2",X"a2",X"a2",X"34",X"a2",X"34",X"20",X"a2",X"a2",X"a2",X"34",X"a2",
X"34",X"24",X"a2",X"a2",X"a2",X"34",X"a2",X"34",X"34",X"00",X"a2",X"a2",X"a2",X"24",X"24",X"24",
X"34",X"a2",X"34",X"34",X"00",X"20",X"a2",X"00",X"20",X"a2",X"24",X"00",X"20",X"a2",X"00",X"20",
X"a2",X"24",X"00",X"20",X"a2",X"00",X"20",X"a2",X"a2",X"a2",X"34",X"a2",X"34",X"28",X"20",X"a2",
X"28",X"20",X"a2",X"a2",X"a2",X"34",X"a2",X"34",X"2c",X"20",X"a2",X"2c",X"20",X"a2",X"a2",X"a2",
X"34",X"a2",X"34",X"34",X"00",X"20",X"a2",X"00",X"20",X"a2",X"a2",X"a2",X"34",X"a2",X"34",X"34",
X"00",X"a2",X"a2",X"a2",X"34",X"a2",X"34",X"34",X"00",X"a2",X"a2",X"a2",X"0c",X"34",X"a2",X"34",
X"a2",X"34",X"a2",X"34",X"a2",X"a2",X"a2",X"24",X"34",X"a2",X"34",X"a3",X"00",X"3c",X"34",X"af",
X"83",X"a2",X"3c",X"34",X"af",X"83",X"83",X"83",X"83",X"a2",X"a2",X"a2",X"a2",X"20",X"a3",X"83",
X"a2",X"a2",X"a2",X"34",X"a2",X"00",X"3c",X"34",X"af",X"34",X"a3",X"93",X"a2",X"3c",X"34",X"af",
X"93",X"93",X"93",X"93",X"a2",X"a2",X"a2",X"a2",X"20",X"a3",X"93",X"a2",X"a2",X"a2",X"34",X"a2",
X"00",X"3c",X"34",X"ac",X"84",X"a2",X"84",X"a2",X"a2",X"a2",X"34",X"a2",X"00",X"3c",X"34",X"ac",
X"84",X"a2",X"84",X"a2",X"a2",X"a2",X"34",X"a2",X"00",X"24",X"ac",X"34",X"8c",X"a2",X"a2",X"a2",
X"34",X"a2",X"34",X"a2",X"a2",X"a2",X"34",X"a2",X"00",X"34",X"a4",X"80",X"a2",X"80",X"a2",X"a2",
X"a2",X"34",X"a2",X"00",X"3c",X"34",X"ac",X"80",X"a2",X"80",X"a2",X"80",X"a2",X"80",X"a2",X"a2",
X"a2",X"0c",X"34",X"a2",X"34",X"a2",X"34",X"a2",X"a2",X"a2",X"00",X"24",X"af",X"af",X"3c",X"34",
X"ab",X"83",X"a2",X"83",X"a2",X"83",X"a2",X"83",X"a2",X"a2",X"a2",X"34",X"a2",X"34",X"a2",X"34",
X"a2",X"a2",X"a2",X"24",X"ac",X"3c",X"34",X"bb",X"83",X"a2",X"83",X"a2",X"83",X"a2",X"83",X"a2",
X"a2",X"a2",X"34",X"a2",X"34",X"a2",X"34",X"a2",X"a2",X"a2",X"34",X"a2",X"34",X"3c",X"34",X"af",
X"3c",X"34",X"8b",X"a2",X"00",X"a2",X"00",X"a2",X"00",X"a2",X"a2",X"a2",X"34",X"a2",X"34",X"3c",
X"34",X"af",X"3c",X"34",X"8b",X"a2",X"00",X"a2",X"00",X"a2",X"00",X"a2",X"a2",X"a2",X"34",X"a2",
X"34",X"3c",X"34",X"af",X"34",X"8b",X"a2",X"00",X"a2",X"00",X"a2",X"00",X"a2",X"a2",X"a2",X"34",
X"a2",X"34",X"3c",X"34",X"af",X"3c",X"34",X"8b",X"a2",X"00",X"a2",X"00",X"a2",X"00",X"a2",X"a2",
X"a2",X"34",X"a2",X"3c",X"34",X"af",X"3c",X"34",X"9b",X"a2",X"00",X"a2",X"00",X"a2",X"00",X"a2",
X"a2",X"a2",X"34",X"a2",X"3c",X"34",X"af",X"3c",X"34",X"9b",X"a2",X"00",X"a2",X"00",X"a2",X"00",
X"a2",X"a2",X"a2",X"34",X"a2",X"3c",X"34",X"af",X"3c",X"34",X"9b",X"a2",X"00",X"a2",X"00",X"a2",
X"00",X"a2",X"a2",X"a2",X"34",X"a2",X"3c",X"34",X"af",X"3c",X"34",X"9b",X"a2",X"00",X"a2",X"00",
X"a2",X"00",X"a2",X"a2",X"a2",X"0c",X"34",X"a2",X"34",X"a2",X"34",X"a2",X"34",X"a2",X"34",X"a2",
X"34",X"a2",X"a2",X"a2",X"34",X"a2",X"34",X"34",X"10",X"a2",X"a2",X"a2",X"a2",X"a2",X"34",X"a2",
X"34",X"34",X"34",X"34",X"34",X"34",X"04",X"a2",X"a2",X"10",X"a2",X"a2",X"a2",X"03",X"a2",X"a2",
X"a2",X"a2",X"34",X"a2",X"34",X"34",X"34",X"34",X"34",X"34",X"34",X"10",X"a2",X"a2",X"10",X"a2",
X"a2",X"a2",X"a2",X"a2",X"34",X"a2",X"34",X"34",X"34",X"34",X"34",X"34",X"3c",X"34",X"34",X"04",
X"a2",X"a2",X"04",X"a2",X"a2",X"04",X"00",X"a2",X"a2",X"a2",X"a2",X"34",X"a2",X"34",X"34",X"34",
X"34",X"34",X"34",X"3c",X"34",X"04",X"00",X"a2",X"04",X"00",X"a2",X"10",X"a2",X"a2",X"a2",X"03",
X"a2",X"a2",X"a2",X"a2",X"34",X"a2",X"34",X"34",X"34",X"34",X"34",X"3c",X"34",X"1c",X"a2",X"a2",
X"1c",X"a2",X"a2",X"a2",X"a2",X"a2",X"34",X"a2",X"34",X"34",X"34",X"34",X"34",X"3c",X"34",X"18",
X"a2",X"a2",X"18",X"a2",X"a2",X"18",X"00",X"a2",X"a2",X"a2",X"a2",X"34",X"a2",X"34",X"34",X"34",
X"34",X"34",X"34",X"3c",X"34",X"34",X"04",X"a2",X"a2",X"04",X"a2",X"a2",X"04",X"00",X"a2",X"a2",
X"a2",X"a2",X"34",X"3c",X"a2",X"34",X"34",X"34",X"34",X"34",X"34",X"3c",X"34",X"04",X"00",X"a2",
X"04",X"a2",X"10",X"a2",X"a2",X"a2",X"03",X"a2",X"a2",X"a2",X"a2",X"34",X"a2",X"34",X"34",X"34",
X"34",X"34",X"34",X"34",X"14",X"a2",X"a2",X"14",X"a2",X"a2",X"a2",X"a2",X"a2",X"34",X"a2",X"34",
X"34",X"34",X"08",X"a2",X"a2",X"a2",X"a2",X"a2",X"34",X"a2",X"34",X"34",X"34",X"34",X"34",X"34",
X"0c",X"a2",X"a2",X"10",X"a2",X"a2",X"a2",X"03",X"a2",X"a2",X"a2",X"a2",X"34",X"a2",X"34",X"34",
X"34",X"34",X"34",X"34",X"3c",X"24",X"00",X"a2",X"a2",X"10",X"a2",X"a2",X"a2",X"03",X"a2",X"a2",
X"a2",X"a2",X"34",X"a2",X"34",X"34",X"34",X"3c",X"24",X"00",X"a2",X"a2",X"a2",X"a2",X"a2",X"34",
X"a2",X"34",X"00",X"a2",X"a2",X"a2",X"0c",X"34",X"a2",X"34",X"a2",X"34",X"a2",X"34",X"a2",X"34",
X"a2",X"a2",X"a2",X"34",X"a2",X"3c",X"34",X"00",X"00",X"a2",X"a2",X"a2",X"34",X"a2",X"3c",X"34",
X"34",X"00",X"00",X"a2",X"a2",X"a2",X"34",X"a2",X"3c",X"34",X"00",X"a2",X"3c",X"00",X"20",X"a2",
X"a2",X"a2",X"34",X"a2",X"3c",X"34",X"34",X"00",X"a2",X"34",X"3c",X"00",X"20",X"a2",X"a2",X"a2",
X"34",X"a2",X"3c",X"34",X"00",X"a2",X"3c",X"00",X"a2",X"a2",X"a2",X"34",X"a2",X"3c",X"34",X"34",
X"00",X"a2",X"34",X"3c",X"00",X"a2",X"a2",X"a2",X"0c",X"34",X"a2",X"34",X"a2",X"34",X"a2",X"34",
X"a2",X"34",X"a2",X"a2",X"a2",X"34",X"a2",X"34",X"34",X"00",X"a2",X"a2",X"a2",X"34",X"a2",X"34",
X"30",X"a2",X"a2",X"a2",X"34",X"a2",X"3c",X"00",X"a2",X"a2",X"a2",X"34",X"a2",X"3c",X"34",X"3c",
X"34",X"00",X"a2",X"a2",X"a2",X"34",X"a2",X"34",X"34",X"00",X"a2",X"a2",X"a2",X"34",X"a2",X"34",
X"34",X"a2",X"a2",X"a2",X"34",X"a2",X"34",X"34",X"00",X"a2",X"a2",X"a2",X"34",X"a2",X"34",X"38",
X"a2",X"a2",X"a2",X"34",X"a2",X"34",X"a2",X"34",X"a2",X"34",X"a2",X"a2",X"a2",X"34",X"a2",X"34",
X"34",X"00",X"20",X"a2",X"a2",X"a2",X"82",X"11",X"00",X"03",X"00",X"00",others=>X"00");

signal RAM_BLOCK1:ram_byte:=(
X"80",X"14",X"94",X"17",X"16",X"15",X"18",X"94",X"02",X"03",X"04",X"11",X"02",X"82",X"02",X"82",
X"02",X"82",X"02",X"82",X"97",X"95",X"00",X"02",X"82",X"02",X"82",X"02",X"82",X"02",X"82",X"97",
X"95",X"02",X"82",X"04",X"03",X"64",X"00",X"82",X"97",X"95",X"02",X"82",X"04",X"03",X"83",X"00",
X"82",X"97",X"95",X"02",X"82",X"02",X"82",X"02",X"82",X"02",X"82",X"02",X"82",X"97",X"95",X"02",
X"82",X"04",X"03",X"64",X"82",X"97",X"95",X"02",X"82",X"04",X"82",X"82",X"97",X"95",X"02",X"82",
X"04",X"85",X"85",X"97",X"95",X"02",X"82",X"03",X"04",X"64",X"82",X"97",X"95",X"02",X"03",X"04",
X"02",X"82",X"02",X"03",X"43",X"85",X"85",X"62",X"85",X"85",X"02",X"43",X"85",X"85",X"62",X"85",
X"85",X"03",X"43",X"85",X"85",X"62",X"85",X"85",X"97",X"95",X"02",X"82",X"02",X"44",X"85",X"85",
X"44",X"85",X"85",X"97",X"95",X"02",X"82",X"02",X"44",X"85",X"85",X"44",X"85",X"85",X"97",X"95",
X"02",X"82",X"02",X"03",X"43",X"85",X"85",X"62",X"85",X"85",X"97",X"95",X"02",X"82",X"03",X"04",
X"64",X"82",X"97",X"95",X"02",X"82",X"03",X"04",X"64",X"82",X"97",X"95",X"00",X"02",X"82",X"02",
X"82",X"02",X"82",X"02",X"82",X"97",X"95",X"19",X"02",X"82",X"02",X"22",X"18",X"03",X"63",X"23",
X"22",X"82",X"02",X"42",X"22",X"24",X"25",X"26",X"27",X"84",X"85",X"86",X"87",X"e7",X"27",X"22",
X"87",X"97",X"95",X"02",X"82",X"18",X"03",X"63",X"23",X"02",X"22",X"22",X"82",X"02",X"42",X"22",
X"24",X"25",X"26",X"27",X"84",X"85",X"86",X"87",X"e7",X"27",X"22",X"87",X"97",X"95",X"02",X"82",
X"18",X"03",X"63",X"43",X"44",X"84",X"42",X"82",X"97",X"95",X"02",X"82",X"18",X"03",X"63",X"43",
X"44",X"84",X"42",X"82",X"97",X"95",X"02",X"82",X"18",X"03",X"43",X"03",X"42",X"82",X"97",X"95",
X"02",X"82",X"02",X"82",X"97",X"95",X"02",X"82",X"18",X"02",X"82",X"83",X"83",X"82",X"82",X"97",
X"95",X"02",X"82",X"18",X"03",X"63",X"43",X"44",X"84",X"44",X"84",X"44",X"84",X"42",X"82",X"97",
X"95",X"00",X"02",X"82",X"02",X"82",X"02",X"82",X"97",X"95",X"18",X"03",X"03",X"03",X"03",X"63",
X"03",X"02",X"82",X"02",X"82",X"02",X"82",X"02",X"82",X"97",X"95",X"02",X"82",X"02",X"82",X"02",
X"82",X"97",X"95",X"03",X"43",X"03",X"63",X"03",X"04",X"84",X"04",X"84",X"04",X"84",X"02",X"82",
X"97",X"95",X"02",X"82",X"02",X"82",X"02",X"82",X"97",X"95",X"02",X"82",X"02",X"03",X"63",X"03",
X"02",X"42",X"02",X"82",X"02",X"82",X"02",X"82",X"02",X"82",X"97",X"95",X"02",X"82",X"02",X"03",
X"63",X"03",X"02",X"42",X"02",X"82",X"02",X"82",X"02",X"82",X"02",X"82",X"97",X"95",X"02",X"82",
X"02",X"03",X"63",X"03",X"42",X"02",X"82",X"02",X"82",X"02",X"82",X"02",X"82",X"97",X"95",X"02",
X"82",X"02",X"03",X"63",X"03",X"02",X"42",X"02",X"82",X"02",X"82",X"02",X"82",X"02",X"82",X"97",
X"95",X"02",X"82",X"03",X"63",X"03",X"02",X"42",X"02",X"82",X"02",X"82",X"02",X"82",X"02",X"82",
X"97",X"95",X"02",X"82",X"03",X"63",X"03",X"02",X"42",X"02",X"82",X"02",X"82",X"02",X"82",X"02",
X"82",X"97",X"95",X"02",X"82",X"03",X"63",X"03",X"02",X"42",X"02",X"82",X"02",X"82",X"02",X"82",
X"02",X"82",X"97",X"95",X"02",X"82",X"03",X"63",X"03",X"02",X"42",X"02",X"82",X"02",X"82",X"02",
X"82",X"02",X"82",X"97",X"95",X"00",X"02",X"82",X"02",X"82",X"02",X"82",X"02",X"82",X"02",X"82",
X"02",X"82",X"97",X"95",X"02",X"82",X"0a",X"0b",X"00",X"8a",X"8a",X"8b",X"97",X"95",X"02",X"82",
X"0a",X"0b",X"0c",X"0d",X"0e",X"0f",X"11",X"8a",X"8d",X"00",X"8e",X"8f",X"8b",X"e0",X"8c",X"96",
X"97",X"95",X"02",X"82",X"0a",X"0b",X"0c",X"0d",X"02",X"03",X"04",X"43",X"8a",X"8b",X"64",X"8c",
X"96",X"8d",X"97",X"95",X"02",X"82",X"0a",X"0b",X"0c",X"0d",X"0f",X"02",X"03",X"63",X"04",X"61",
X"8a",X"8b",X"41",X"8c",X"96",X"01",X"00",X"8f",X"8d",X"97",X"95",X"02",X"82",X"0a",X"0b",X"0c",
X"0d",X"0e",X"0f",X"03",X"63",X"71",X"00",X"8a",X"11",X"00",X"8d",X"00",X"8e",X"8f",X"8b",X"e0",
X"8c",X"96",X"97",X"95",X"02",X"82",X"0a",X"0b",X"0c",X"0d",X"02",X"03",X"63",X"60",X"8a",X"8b",
X"40",X"8c",X"96",X"8d",X"97",X"95",X"02",X"82",X"0a",X"0b",X"0c",X"0d",X"02",X"03",X"63",X"40",
X"8a",X"8b",X"60",X"8c",X"96",X"00",X"00",X"96",X"8d",X"97",X"95",X"02",X"82",X"0a",X"0b",X"0c",
X"0d",X"0e",X"02",X"03",X"63",X"04",X"40",X"8a",X"8b",X"60",X"8c",X"96",X"80",X"00",X"8d",X"8e",
X"97",X"95",X"02",X"03",X"82",X"0a",X"0b",X"0c",X"0d",X"0e",X"0f",X"03",X"63",X"10",X"00",X"8a",
X"70",X"8b",X"00",X"8e",X"8f",X"8c",X"e0",X"8d",X"96",X"97",X"95",X"02",X"82",X"0a",X"0b",X"0c",
X"0d",X"02",X"03",X"04",X"64",X"8a",X"8b",X"43",X"8c",X"96",X"8d",X"97",X"95",X"02",X"82",X"0a",
X"0b",X"0f",X"00",X"8a",X"8f",X"8b",X"97",X"95",X"02",X"82",X"0a",X"0b",X"0c",X"0d",X"0e",X"0f",
X"00",X"8a",X"8d",X"00",X"8e",X"8f",X"8b",X"e0",X"8c",X"8f",X"97",X"95",X"02",X"82",X"0a",X"0b",
X"0c",X"0d",X"0e",X"0f",X"03",X"63",X"60",X"8a",X"8d",X"00",X"8e",X"8f",X"8b",X"e0",X"8c",X"96",
X"97",X"95",X"02",X"82",X"0a",X"0b",X"0f",X"03",X"63",X"60",X"8a",X"8f",X"8b",X"97",X"95",X"02",
X"82",X"02",X"00",X"82",X"97",X"95",X"00",X"02",X"82",X"02",X"82",X"02",X"82",X"02",X"82",X"02",
X"82",X"97",X"95",X"02",X"82",X"02",X"42",X"02",X"03",X"83",X"97",X"95",X"02",X"82",X"02",X"42",
X"03",X"62",X"03",X"83",X"97",X"95",X"02",X"82",X"02",X"42",X"02",X"83",X"02",X"02",X"63",X"83",
X"97",X"95",X"02",X"82",X"02",X"42",X"03",X"62",X"83",X"03",X"02",X"62",X"63",X"83",X"97",X"95",
X"02",X"82",X"02",X"42",X"02",X"83",X"02",X"02",X"83",X"97",X"95",X"02",X"82",X"02",X"42",X"03",
X"62",X"84",X"03",X"02",X"62",X"83",X"97",X"95",X"00",X"02",X"82",X"02",X"82",X"02",X"82",X"02",
X"82",X"02",X"82",X"97",X"95",X"02",X"82",X"02",X"03",X"43",X"84",X"97",X"95",X"02",X"82",X"02",
X"44",X"84",X"97",X"95",X"02",X"82",X"02",X"02",X"83",X"97",X"95",X"02",X"82",X"02",X"42",X"03",
X"63",X"43",X"84",X"97",X"95",X"02",X"82",X"02",X"03",X"43",X"84",X"97",X"95",X"02",X"82",X"02",
X"44",X"84",X"97",X"95",X"02",X"82",X"02",X"03",X"43",X"84",X"97",X"95",X"02",X"82",X"02",X"44",
X"84",X"97",X"95",X"02",X"82",X"02",X"82",X"02",X"82",X"02",X"82",X"97",X"95",X"02",X"82",X"02",
X"04",X"00",X"84",X"84",X"97",X"95",X"8a",X"40",X"00",X"e0",X"00",X"00",others=>X"00");

signal RAM_BLOCK2:ram_byte:=(
X"60",X"20",X"00",X"00",X"00",X"00",X"0f",X"b8",X"00",X"00",X"00",X"b8",X"00",X"00",X"00",X"00",
X"00",X"00",X"00",X"00",X"00",X"00",X"03",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",
X"00",X"00",X"00",X"00",X"00",X"00",X"10",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"10",
X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",
X"00",X"00",X"00",X"10",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",
X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"10",X"00",X"00",X"00",X"00",X"00",X"00",
X"00",X"00",X"00",X"00",X"20",X"00",X"00",X"20",X"00",X"00",X"ff",X"20",X"00",X"00",X"20",X"00",
X"00",X"ff",X"20",X"00",X"00",X"20",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",
X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",
X"00",X"00",X"00",X"00",X"20",X"00",X"00",X"20",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",
X"10",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"10",X"00",X"00",X"00",X"03",X"00",X"00",X"00",
X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"20",X"00",X"00",X"00",X"00",X"10",X"41",X"43",X"00",
X"00",X"00",X"42",X"44",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",
X"00",X"00",X"00",X"00",X"00",X"10",X"41",X"43",X"00",X"00",X"00",X"00",X"00",X"42",X"44",X"00",
X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",
X"10",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"10",X"00",X"00",X"00",
X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"10",X"00",X"00",X"00",X"00",X"00",X"00",X"00",
X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"20",X"41",X"00",X"00",X"00",X"00",X"00",X"00",
X"00",X"00",X"00",X"10",X"41",X"43",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",
X"00",X"03",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"10",X"ff",X"00",X"00",X"41",X"43",
X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",
X"00",X"00",X"00",X"00",X"00",X"41",X"43",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",
X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"44",X"42",X"00",
X"47",X"47",X"00",X"00",X"12",X"00",X"12",X"00",X"12",X"00",X"00",X"00",X"00",X"00",X"00",X"45",
X"43",X"00",X"47",X"47",X"00",X"00",X"12",X"00",X"12",X"00",X"12",X"00",X"00",X"00",X"00",X"00",
X"00",X"44",X"44",X"00",X"42",X"00",X"00",X"12",X"00",X"12",X"00",X"12",X"00",X"00",X"00",X"00",
X"00",X"00",X"49",X"48",X"00",X"00",X"42",X"00",X"00",X"12",X"00",X"12",X"00",X"12",X"00",X"00",
X"00",X"00",X"00",X"41",X"48",X"00",X"44",X"42",X"00",X"00",X"12",X"00",X"12",X"00",X"12",X"00",
X"00",X"00",X"00",X"00",X"42",X"48",X"00",X"44",X"47",X"00",X"00",X"12",X"00",X"12",X"00",X"12",
X"00",X"00",X"00",X"00",X"00",X"43",X"41",X"00",X"44",X"47",X"00",X"00",X"12",X"00",X"12",X"00",
X"12",X"00",X"00",X"00",X"00",X"00",X"44",X"42",X"00",X"33",X"47",X"00",X"00",X"12",X"00",X"12",
X"00",X"12",X"00",X"00",X"00",X"03",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",
X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",
X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",
X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",
X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"ff",X"12",X"00",X"00",
X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",
X"00",X"00",X"00",X"ff",X"12",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",
X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"ff",X"12",X"00",X"00",X"00",
X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"ff",X"12",X"00",
X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",
X"00",X"00",X"00",X"ff",X"12",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",
X"00",X"00",X"00",X"99",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"ff",X"12",X"00",X"00",X"00",
X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",
X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",
X"00",X"00",X"02",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",
X"02",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",
X"00",X"00",X"00",X"00",X"00",X"0b",X"f8",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",
X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"0c",X"00",X"00",X"00",X"00",X"00",X"00",X"00",
X"00",X"00",X"00",X"00",X"00",X"00",X"03",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",
X"00",X"00",X"00",X"00",X"00",X"40",X"42",X"1a",X"1e",X"00",X"00",X"00",X"00",X"00",X"40",X"42",
X"00",X"18",X"1e",X"00",X"00",X"00",X"00",X"00",X"40",X"42",X"1c",X"00",X"84",X"1e",X"ff",X"00",
X"00",X"00",X"00",X"00",X"40",X"42",X"00",X"18",X"00",X"00",X"84",X"18",X"ff",X"00",X"00",X"00",
X"00",X"00",X"40",X"42",X"1c",X"00",X"84",X"1e",X"00",X"00",X"00",X"00",X"00",X"40",X"42",X"00",
X"20",X"00",X"00",X"84",X"18",X"00",X"00",X"00",X"03",X"00",X"00",X"00",X"00",X"00",X"00",X"00",
X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"07",X"60",X"20",X"00",X"00",X"00",X"00",X"00",X"07",
X"60",X"00",X"00",X"00",X"00",X"00",X"00",X"1c",X"00",X"00",X"00",X"00",X"00",X"f0",X"f0",X"0f",
X"0f",X"20",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"20",X"00",X"00",X"00",X"00",X"00",X"00",
X"00",X"00",X"00",X"00",X"00",X"00",X"f0",X"f0",X"20",X"00",X"00",X"00",X"00",X"00",X"f0",X"f0",
X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",
X"00",X"00",X"00",X"00",X"00",X"00",X"ff",X"ff",X"00",X"00",X"00",X"00",others=>x"00");

signal RAM_BLOCK3:ram_byte:=(
X"00",X"00",X"08",X"0d",X"58",X"0a",X"80",X"00",X"00",X"00",X"00",X"00",X"54",X"00",X"65",X"00",
X"73",X"00",X"74",X"00",X"00",X"00",X"c6",X"4d",X"00",X"75",X"00",X"6c",X"00",X"74",X"00",X"00",
X"00",X"61",X"00",X"0d",X"05",X"18",X"12",X"00",X"00",X"00",X"62",X"00",X"c3",X"03",X"1a",X"12",
X"00",X"00",X"00",X"41",X"00",X"72",X"00",X"69",X"00",X"74",X"00",X"68",X"00",X"00",X"00",X"61",
X"00",X"3c",X"05",X"20",X"00",X"00",X"00",X"62",X"00",X"3c",X"05",X"00",X"00",X"00",X"63",X"00",
X"32",X"0f",X"00",X"00",X"00",X"64",X"00",X"05",X"3c",X"20",X"00",X"00",X"00",X"00",X"00",X"00",
X"69",X"00",X"0a",X"0c",X"2a",X"40",X"00",X"2a",X"42",X"00",X"f0",X"2a",X"42",X"00",X"2a",X"44",
X"00",X"ff",X"2a",X"44",X"00",X"2a",X"46",X"00",X"00",X"00",X"6a",X"00",X"0a",X"0c",X"40",X"00",
X"08",X"42",X"00",X"00",X"00",X"6b",X"00",X"0a",X"0c",X"40",X"00",X"08",X"42",X"00",X"00",X"00",
X"6c",X"00",X"0a",X"0c",X"2a",X"40",X"00",X"2a",X"42",X"00",X"00",X"00",X"6d",X"00",X"46",X"05",
X"22",X"00",X"00",X"00",X"6e",X"00",X"46",X"05",X"22",X"00",X"00",X"00",X"c6",X"4c",X"00",X"6f",
X"00",X"61",X"00",X"64",X"00",X"00",X"00",X"00",X"61",X"00",X"41",X"00",X"25",X"42",X"fc",X"10",
X"00",X"00",X"43",X"45",X"04",X"04",X"05",X"06",X"07",X"00",X"00",X"00",X"00",X"01",X"00",X"00",
X"00",X"00",X"00",X"62",X"00",X"25",X"42",X"fc",X"10",X"41",X"00",X"00",X"00",X"43",X"45",X"04",
X"04",X"05",X"06",X"07",X"00",X"00",X"00",X"00",X"01",X"00",X"00",X"00",X"00",X"00",X"63",X"00",
X"25",X"41",X"42",X"10",X"10",X"00",X"12",X"00",X"00",X"00",X"64",X"00",X"25",X"41",X"42",X"10",
X"10",X"00",X"12",X"00",X"00",X"00",X"65",X"00",X"25",X"41",X"10",X"00",X"10",X"00",X"00",X"00",
X"66",X"00",X"41",X"00",X"00",X"00",X"67",X"00",X"25",X"42",X"10",X"10",X"00",X"11",X"00",X"00",
X"00",X"68",X"00",X"25",X"42",X"44",X"10",X"10",X"00",X"11",X"00",X"12",X"00",X"13",X"00",X"00",
X"00",X"c6",X"53",X"00",X"57",X"00",X"4c",X"00",X"00",X"00",X"25",X"ff",X"00",X"04",X"42",X"44",
X"02",X"00",X"00",X"01",X"00",X"02",X"00",X"03",X"00",X"00",X"00",X"53",X"00",X"57",X"00",X"52",
X"00",X"00",X"00",X"00",X"00",X"42",X"44",X"07",X"04",X"00",X"05",X"00",X"06",X"00",X"07",X"00",
X"00",X"00",X"4c",X"00",X"57",X"00",X"4c",X"00",X"00",X"00",X"61",X"00",X"00",X"43",X"41",X"00",
X"47",X"41",X"00",X"00",X"03",X"00",X"03",X"00",X"03",X"00",X"00",X"00",X"62",X"00",X"00",X"44",
X"42",X"00",X"47",X"41",X"01",X"00",X"03",X"00",X"03",X"00",X"03",X"00",X"00",X"00",X"63",X"00",
X"00",X"43",X"43",X"00",X"41",X"02",X"00",X"03",X"00",X"03",X"00",X"03",X"00",X"00",X"00",X"64",
X"00",X"00",X"47",X"44",X"00",X"43",X"41",X"03",X"00",X"03",X"00",X"03",X"00",X"03",X"00",X"00",
X"00",X"66",X"00",X"47",X"49",X"00",X"43",X"80",X"00",X"00",X"03",X"00",X"03",X"00",X"03",X"00",
X"00",X"00",X"6b",X"00",X"41",X"49",X"00",X"43",X"80",X"01",X"00",X"03",X"00",X"03",X"00",X"03",
X"00",X"00",X"00",X"6c",X"00",X"42",X"48",X"00",X"45",X"80",X"02",X"00",X"03",X"00",X"03",X"00",
X"03",X"00",X"00",X"00",X"6b",X"00",X"43",X"41",X"00",X"45",X"80",X"03",X"00",X"03",X"00",X"03",
X"00",X"03",X"00",X"00",X"00",X"c6",X"42",X"00",X"72",X"00",X"61",X"00",X"6e",X"00",X"63",X"00",
X"68",X"00",X"00",X"00",X"61",X"00",X"41",X"42",X"02",X"00",X"00",X"00",X"00",X"00",X"62",X"00",
X"41",X"42",X"43",X"44",X"45",X"58",X"05",X"00",X"00",X"06",X"00",X"00",X"00",X"08",X"00",X"00",
X"00",X"00",X"63",X"00",X"41",X"42",X"43",X"44",X"64",X"7b",X"7b",X"05",X"00",X"00",X"02",X"00",
X"00",X"00",X"00",X"00",X"64",X"00",X"41",X"42",X"43",X"44",X"58",X"64",X"ff",X"34",X"7b",X"05",
X"00",X"00",X"02",X"00",X"00",X"02",X"00",X"00",X"00",X"00",X"00",X"65",X"00",X"41",X"42",X"43",
X"44",X"45",X"58",X"ff",X"34",X"08",X"00",X"00",X"05",X"00",X"00",X"06",X"00",X"00",X"00",X"08",
X"00",X"00",X"00",X"00",X"66",X"00",X"41",X"42",X"43",X"44",X"64",X"ff",X"34",X"05",X"00",X"00",
X"02",X"00",X"00",X"00",X"00",X"00",X"67",X"00",X"41",X"42",X"43",X"44",X"64",X"ff",X"34",X"05",
X"00",X"00",X"02",X"00",X"00",X"02",X"00",X"00",X"00",X"00",X"00",X"68",X"00",X"41",X"42",X"43",
X"44",X"45",X"64",X"ff",X"34",X"00",X"05",X"00",X"00",X"02",X"00",X"00",X"02",X"00",X"00",X"00",
X"00",X"00",X"69",X"99",X"00",X"41",X"42",X"43",X"44",X"45",X"58",X"ff",X"34",X"07",X"00",X"00",
X"04",X"00",X"06",X"00",X"00",X"00",X"08",X"00",X"00",X"00",X"00",X"6a",X"00",X"41",X"42",X"43",
X"44",X"64",X"7b",X"7b",X"05",X"00",X"00",X"02",X"00",X"00",X"00",X"00",X"00",X"6b",X"00",X"41",
X"42",X"58",X"d5",X"00",X"00",X"00",X"00",X"00",X"6c",X"00",X"41",X"42",X"43",X"44",X"45",X"58",
X"e6",X"00",X"00",X"06",X"00",X"00",X"00",X"08",X"00",X"00",X"00",X"00",X"6d",X"00",X"41",X"42",
X"43",X"44",X"45",X"58",X"00",X"f0",X"09",X"00",X"00",X"06",X"00",X"00",X"00",X"08",X"00",X"00",
X"00",X"00",X"6e",X"00",X"41",X"42",X"58",X"00",X"30",X"08",X"00",X"00",X"00",X"00",X"00",X"6f",
X"00",X"41",X"00",X"00",X"00",X"00",X"c6",X"53",X"00",X"68",X"00",X"69",X"00",X"66",X"00",X"74",
X"00",X"00",X"00",X"61",X"00",X"41",X"43",X"00",X"02",X"00",X"00",X"00",X"62",X"00",X"41",X"43",
X"08",X"04",X"02",X"00",X"00",X"00",X"63",X"00",X"41",X"43",X"03",X"00",X"00",X"43",X"80",X"00",
X"00",X"00",X"64",X"00",X"41",X"43",X"10",X"07",X"00",X"19",X"00",X"07",X"80",X"00",X"00",X"00",
X"65",X"00",X"41",X"43",X"02",X"00",X"00",X"42",X"00",X"00",X"00",X"66",X"00",X"41",X"43",X"10",
X"06",X"00",X"19",X"00",X"06",X"00",X"00",X"00",X"c6",X"4c",X"00",X"6f",X"00",X"67",X"00",X"69",
X"00",X"63",X"00",X"00",X"00",X"61",X"00",X"41",X"f3",X"24",X"00",X"00",X"00",X"62",X"00",X"41",
X"f3",X"00",X"00",X"00",X"63",X"00",X"41",X"02",X"00",X"00",X"00",X"64",X"00",X"ff",X"8e",X"0f",
X"30",X"27",X"00",X"00",X"00",X"65",X"00",X"40",X"01",X"25",X"00",X"00",X"00",X"66",X"00",X"40",
X"01",X"00",X"00",X"00",X"67",X"00",X"43",X"02",X"26",X"00",X"00",X"00",X"68",X"00",X"43",X"02",
X"00",X"00",X"00",X"44",X"00",X"6f",X"00",X"6e",X"00",X"65",X"00",X"00",X"00",X"70",X"00",X"7a",
X"3b",X"0d",X"01",X"00",X"00",X"00",X"fc",X"fe",X"00",X"08",X"00",X"00",others=>x"00");

  attribute RAM_STYLE: string;
  attribute RAM_STYLE of RAM_BLOCK0: signal is "block";
  attribute RAM_STYLE of RAM_BLOCK1: signal is "block";
  attribute RAM_STYLE of RAM_BLOCK2: signal is "block";
  attribute RAM_STYLE of RAM_BLOCK3: signal is "block";

type ram_halfword is array (0 to (2**(addresswidth-1)))
         of std_logic_vector (15 downto 0);

--high block  bit 31...16
signal HIGHRAM:ram_halfword:=(
X"4080",X"3c14",X"3694",X"3417",X"3416",X"3415",X"3418",X"4094",
X"2402",X"2403",X"2404",X"4011",X"3402",X"a282",X"3402",X"a282",
X"3402",X"a282",X"3402",X"a282",X"a297",X"a295",X"0c00",X"3402",
X"a282",X"3402",X"a282",X"3402",X"a282",X"3402",X"a282",X"a297",
X"a295",X"3402",X"a282",X"3404",X"3403",X"0064",X"0000",X"a282",
X"a297",X"a295",X"3402",X"a282",X"3404",X"3403",X"0083",X"0000",
X"a282",X"a297",X"a295",X"3402",X"a282",X"3402",X"a282",X"3402",
X"a282",X"3402",X"a282",X"3402",X"a282",X"a297",X"a295",X"3402",
X"a282",X"3404",X"3403",X"0064",X"a282",X"a297",X"a295",X"3402",
X"a282",X"3404",X"2082",X"a282",X"a297",X"a295",X"3402",X"a282",
X"3404",X"2485",X"a285",X"a297",X"a295",X"3402",X"a282",X"3403",
X"3404",X"0064",X"a282",X"a297",X"a295",X"2402",X"2403",X"2404",
X"3402",X"a282",X"3402",X"3403",X"0043",X"2085",X"a285",X"0062",
X"2085",X"a285",X"2402",X"0043",X"2085",X"a285",X"0062",X"2085",
X"a285",X"2403",X"0043",X"2085",X"a285",X"0062",X"2085",X"a285",
X"a297",X"a295",X"3402",X"a282",X"3402",X"2844",X"2085",X"a285",
X"2844",X"2085",X"a285",X"a297",X"a295",X"3402",X"a282",X"3402",
X"2c44",X"2085",X"a285",X"2c44",X"2085",X"a285",X"a297",X"a295",
X"3402",X"a282",X"3402",X"3403",X"0043",X"2085",X"a285",X"0062",
X"2085",X"a285",X"a297",X"a295",X"3402",X"a282",X"3403",X"3404",
X"0064",X"a282",X"a297",X"a295",X"3402",X"a282",X"3403",X"3404",
X"0064",X"a282",X"a297",X"a295",X"0c00",X"3402",X"a282",X"3402",
X"a282",X"3402",X"a282",X"3402",X"a282",X"a297",X"a295",X"2419",
X"3402",X"a282",X"3402",X"a322",X"0018",X"3c03",X"3463",X"af23",
X"8322",X"a282",X"3c02",X"3442",X"af22",X"8324",X"8325",X"8326",
X"8327",X"a284",X"a285",X"a286",X"a287",X"20e7",X"a327",X"8322",
X"a287",X"a297",X"a295",X"3402",X"a282",X"0018",X"3c03",X"3463",
X"af23",X"3402",X"a322",X"9322",X"a282",X"3c02",X"3442",X"af22",
X"9324",X"9325",X"9326",X"9327",X"a284",X"a285",X"a286",X"a287",
X"20e7",X"a327",X"9322",X"a287",X"a297",X"a295",X"3402",X"a282",
X"0018",X"3c03",X"3463",X"ac43",X"8444",X"a284",X"8442",X"a282",
X"a297",X"a295",X"3402",X"a282",X"0018",X"3c03",X"3463",X"ac43",
X"8444",X"a284",X"8442",X"a282",X"a297",X"a295",X"3402",X"a282",
X"0018",X"2403",X"ac43",X"3403",X"8c42",X"a282",X"a297",X"a295",
X"3402",X"a282",X"3402",X"a282",X"a297",X"a295",X"3402",X"a282",
X"0018",X"3402",X"a482",X"8083",X"a283",X"8082",X"a282",X"a297",
X"a295",X"3402",X"a282",X"0018",X"3c03",X"3463",X"ac43",X"8044",
X"a284",X"8044",X"a284",X"8044",X"a284",X"8042",X"a282",X"a297",
X"a295",X"0c00",X"3402",X"a282",X"3402",X"a282",X"3402",X"a282",
X"a297",X"a295",X"0018",X"2403",X"af03",X"af03",X"3c03",X"3463",
X"ab03",X"8302",X"a282",X"8302",X"a282",X"8302",X"a282",X"8302",
X"a282",X"a297",X"a295",X"3402",X"a282",X"3402",X"a282",X"3402",
X"a282",X"a297",X"a295",X"2403",X"ac43",X"3c03",X"3463",X"bb03",
X"8304",X"a284",X"8304",X"a284",X"8304",X"a284",X"8302",X"a282",
X"a297",X"a295",X"3402",X"a282",X"3402",X"a282",X"3402",X"a282",
X"a297",X"a295",X"3402",X"a282",X"3402",X"3c03",X"3463",X"af03",
X"3c02",X"3442",X"8b02",X"a282",X"0002",X"a282",X"0002",X"a282",
X"0002",X"a282",X"a297",X"a295",X"3402",X"a282",X"3402",X"3c03",
X"3463",X"af03",X"3c02",X"3442",X"8b02",X"a282",X"0002",X"a282",
X"0002",X"a282",X"0002",X"a282",X"a297",X"a295",X"3402",X"a282",
X"3402",X"3c03",X"3463",X"af03",X"3442",X"8b02",X"a282",X"0002",
X"a282",X"0002",X"a282",X"0002",X"a282",X"a297",X"a295",X"3402",
X"a282",X"3402",X"3c03",X"3463",X"af03",X"3c02",X"3442",X"8b02",
X"a282",X"0002",X"a282",X"0002",X"a282",X"0002",X"a282",X"a297",
X"a295",X"3402",X"a282",X"3c03",X"3463",X"af03",X"3c02",X"3442",
X"9b02",X"a282",X"0002",X"a282",X"0002",X"a282",X"0002",X"a282",
X"a297",X"a295",X"3402",X"a282",X"3c03",X"3463",X"af03",X"3c02",
X"3442",X"9b02",X"a282",X"0002",X"a282",X"0002",X"a282",X"0002",
X"a282",X"a297",X"a295",X"3402",X"a282",X"3c03",X"3463",X"af03",
X"3c02",X"3442",X"9b02",X"a282",X"0002",X"a282",X"0002",X"a282",
X"0002",X"a282",X"a297",X"a295",X"3402",X"a282",X"3c03",X"3463",
X"af03",X"3c02",X"3442",X"9b02",X"a282",X"0002",X"a282",X"0002",
X"a282",X"0002",X"a282",X"a297",X"a295",X"0c00",X"3402",X"a282",
X"3402",X"a282",X"3402",X"a282",X"3402",X"a282",X"3402",X"a282",
X"3402",X"a282",X"a297",X"a295",X"3402",X"a282",X"340a",X"340b",
X"1000",X"a28a",X"a28a",X"a28b",X"a297",X"a295",X"3402",X"a282",
X"340a",X"340b",X"340c",X"340d",X"340e",X"340f",X"0411",X"a28a",
X"a28d",X"1000",X"a28e",X"a28f",X"a28b",X"03e0",X"a28c",X"a296",
X"a297",X"a295",X"3402",X"a282",X"340a",X"340b",X"340c",X"340d",
X"3402",X"3403",X"3404",X"1043",X"a28a",X"a28b",X"1064",X"a28c",
X"a296",X"a28d",X"a297",X"a295",X"3402",X"a282",X"340a",X"340b",
X"340c",X"340d",X"340f",X"3402",X"3c03",X"3463",X"3404",X"0461",
X"a28a",X"a28b",X"0441",X"a28c",X"a296",X"0401",X"0000",X"a28f",
X"a28d",X"a297",X"a295",X"3402",X"a282",X"340a",X"340b",X"340c",
X"340d",X"340e",X"340f",X"3c03",X"3463",X"0471",X"0000",X"a28a",
X"0411",X"0000",X"a28d",X"1000",X"a28e",X"a28f",X"a28b",X"03e0",
X"a28c",X"a296",X"a297",X"a295",X"3402",X"a282",X"340a",X"340b",
X"340c",X"340d",X"3402",X"3c03",X"3463",X"1c60",X"a28a",X"a28b",
X"1c40",X"a28c",X"a296",X"a28d",X"a297",X"a295",X"3402",X"a282",
X"340a",X"340b",X"340c",X"340d",X"3402",X"3c03",X"3463",X"1840",
X"a28a",X"a28b",X"1860",X"a28c",X"a296",X"1800",X"0000",X"a296",
X"a28d",X"a297",X"a295",X"3402",X"a282",X"340a",X"340b",X"340c",
X"340d",X"340e",X"3402",X"3c03",X"3463",X"3404",X"0440",X"a28a",
X"a28b",X"0460",X"a28c",X"a296",X"0480",X"0000",X"a28d",X"a28e",
X"a297",X"a295",X"3402",X"3c03",X"a282",X"340a",X"340b",X"340c",
X"340d",X"340e",X"340f",X"3c03",X"3463",X"0410",X"0000",X"a28a",
X"0470",X"a28b",X"1000",X"a28e",X"a28f",X"a28c",X"03e0",X"a28d",
X"a296",X"a297",X"a295",X"3402",X"a282",X"340a",X"340b",X"340c",
X"340d",X"3402",X"3403",X"3404",X"1464",X"a28a",X"a28b",X"1443",
X"a28c",X"a296",X"a28d",X"a297",X"a295",X"3402",X"a282",X"340a",
X"340b",X"340f",X"0800",X"a28a",X"a28f",X"a28b",X"a297",X"a295",
X"3402",X"a282",X"340a",X"340b",X"340c",X"340d",X"340e",X"340f",
X"0c00",X"a28a",X"a28d",X"1000",X"a28e",X"a28f",X"a28b",X"03e0",
X"a28c",X"a28f",X"a297",X"a295",X"3402",X"a282",X"340a",X"340b",
X"340c",X"340d",X"340e",X"340f",X"3c03",X"2463",X"0060",X"a28a",
X"a28d",X"1000",X"a28e",X"a28f",X"a28b",X"03e0",X"a28c",X"a296",
X"a297",X"a295",X"3402",X"a282",X"340a",X"340b",X"340f",X"3c03",
X"2463",X"0060",X"a28a",X"a28f",X"a28b",X"a297",X"a295",X"3402",
X"a282",X"3402",X"0000",X"a282",X"a297",X"a295",X"0c00",X"3402",
X"a282",X"3402",X"a282",X"3402",X"a282",X"3402",X"a282",X"3402",
X"a282",X"a297",X"a295",X"3402",X"a282",X"3c02",X"3442",X"0002",
X"0003",X"a283",X"a297",X"a295",X"3402",X"a282",X"3c02",X"3442",
X"3403",X"0062",X"0003",X"a283",X"a297",X"a295",X"3402",X"a282",
X"3c02",X"3442",X"0002",X"a283",X"3c02",X"0002",X"2063",X"a283",
X"a297",X"a295",X"3402",X"a282",X"3c02",X"3442",X"3403",X"0062",
X"a283",X"3403",X"3c02",X"0062",X"2063",X"a283",X"a297",X"a295",
X"3402",X"a282",X"3c02",X"3442",X"0002",X"a283",X"3c02",X"0002",
X"a283",X"a297",X"a295",X"3402",X"a282",X"3c02",X"3442",X"3403",
X"0062",X"a284",X"3403",X"3c02",X"0062",X"a283",X"a297",X"a295",
X"0c00",X"3402",X"a282",X"3402",X"a282",X"3402",X"a282",X"3402",
X"a282",X"3402",X"a282",X"a297",X"a295",X"3402",X"a282",X"3402",
X"3403",X"0043",X"a284",X"a297",X"a295",X"3402",X"a282",X"3402",
X"3044",X"a284",X"a297",X"a295",X"3402",X"a282",X"3c02",X"0002",
X"a283",X"a297",X"a295",X"3402",X"a282",X"3c02",X"3442",X"3c03",
X"3463",X"0043",X"a284",X"a297",X"a295",X"3402",X"a282",X"3402",
X"3403",X"0043",X"a284",X"a297",X"a295",X"3402",X"a282",X"3402",
X"3444",X"a284",X"a297",X"a295",X"3402",X"a282",X"3402",X"3403",
X"0043",X"a284",X"a297",X"a295",X"3402",X"a282",X"3402",X"3844",
X"a284",X"a297",X"a295",X"3402",X"a282",X"3402",X"a282",X"3402",
X"a282",X"3402",X"a282",X"a297",X"a295",X"3402",X"a282",X"3402",
X"3404",X"0000",X"2084",X"a284",X"a297",X"a295",X"828a",X"1140",
X"0000",X"03e0",X"0000",X"0000",others=>X"0000");

--low block  bit 15...0
signal LOWRAM:ram_halfword:=(
X"6000",X"2000",X"0008",X"000d",X"0058",X"000a",X"0f80",X"b800",
X"0000",X"0000",X"0000",X"b800",X"0054",X"0000",X"0065",X"0000",
X"0073",X"0000",X"0074",X"0000",X"0000",X"0000",X"03c6",X"004d",
X"0000",X"0075",X"0000",X"006c",X"0000",X"0074",X"0000",X"0000",
X"0000",X"0061",X"0000",X"000d",X"0005",X"0018",X"1012",X"0000",
X"0000",X"0000",X"0062",X"0000",X"00c3",X"0003",X"001a",X"1012",
X"0000",X"0000",X"0000",X"0041",X"0000",X"0072",X"0000",X"0069",
X"0000",X"0074",X"0000",X"0068",X"0000",X"0000",X"0000",X"0061",
X"0000",X"003c",X"0005",X"1020",X"0000",X"0000",X"0000",X"0062",
X"0000",X"003c",X"0005",X"0000",X"0000",X"0000",X"0063",X"0000",
X"0032",X"000f",X"0000",X"0000",X"0000",X"0064",X"0000",X"0005",
X"003c",X"1020",X"0000",X"0000",X"0000",X"0000",X"0000",X"0000",
X"0069",X"0000",X"000a",X"000c",X"202a",X"0040",X"0000",X"202a",
X"0042",X"0000",X"fff0",X"202a",X"0042",X"0000",X"202a",X"0044",
X"0000",X"ffff",X"202a",X"0044",X"0000",X"202a",X"0046",X"0000",
X"0000",X"0000",X"006a",X"0000",X"000a",X"000c",X"0040",X"0000",
X"0008",X"0042",X"0000",X"0000",X"0000",X"006b",X"0000",X"000a",
X"000c",X"0040",X"0000",X"0008",X"0042",X"0000",X"0000",X"0000",
X"006c",X"0000",X"000a",X"000c",X"202a",X"0040",X"0000",X"202a",
X"0042",X"0000",X"0000",X"0000",X"006d",X"0000",X"0046",X"0005",
X"1022",X"0000",X"0000",X"0000",X"006e",X"0000",X"0046",X"0005",
X"1022",X"0000",X"0000",X"0000",X"03c6",X"004c",X"0000",X"006f",
X"0000",X"0061",X"0000",X"0064",X"0000",X"0000",X"0000",X"2000",
X"0061",X"0000",X"0041",X"0000",X"1025",X"4142",X"43fc",X"0010",
X"0000",X"0000",X"4243",X"4445",X"0004",X"0004",X"0005",X"0006",
X"0007",X"0000",X"0000",X"0000",X"0000",X"0001",X"0000",X"0000",
X"0000",X"0000",X"0000",X"0062",X"0000",X"1025",X"4142",X"43fc",
X"0010",X"0041",X"0000",X"0000",X"0000",X"4243",X"4445",X"0004",
X"0004",X"0005",X"0006",X"0007",X"0000",X"0000",X"0000",X"0000",
X"0001",X"0000",X"0000",X"0000",X"0000",X"0000",X"0063",X"0000",
X"1025",X"0041",X"0042",X"0010",X"0010",X"0000",X"0012",X"0000",
X"0000",X"0000",X"0064",X"0000",X"1025",X"0041",X"0042",X"0010",
X"0010",X"0000",X"0012",X"0000",X"0000",X"0000",X"0065",X"0000",
X"1025",X"0041",X"0010",X"0000",X"0010",X"0000",X"0000",X"0000",
X"0066",X"0000",X"0041",X"0000",X"0000",X"0000",X"0067",X"0000",
X"2025",X"4142",X"0010",X"0010",X"0000",X"0011",X"0000",X"0000",
X"0000",X"0068",X"0000",X"1025",X"4142",X"4344",X"0010",X"0010",
X"0000",X"0011",X"0000",X"0012",X"0000",X"0013",X"0000",X"0000",
X"0000",X"03c6",X"0053",X"0000",X"0057",X"0000",X"004c",X"0000",
X"0000",X"0000",X"1025",X"ffff",X"0000",X"0004",X"4142",X"4344",
X"0002",X"0000",X"0000",X"0001",X"0000",X"0002",X"0000",X"0003",
X"0000",X"0000",X"0000",X"0053",X"0000",X"0057",X"0000",X"0052",
X"0000",X"0000",X"0000",X"0000",X"0000",X"4142",X"4344",X"0007",
X"0004",X"0000",X"0005",X"0000",X"0006",X"0000",X"0007",X"0000",
X"0000",X"0000",X"004c",X"0000",X"0057",X"0000",X"004c",X"0000",
X"0000",X"0000",X"0061",X"0000",X"0000",X"4443",X"4241",X"0000",
X"4747",X"4741",X"0000",X"0000",X"1203",X"0000",X"1203",X"0000",
X"1203",X"0000",X"0000",X"0000",X"0062",X"0000",X"0000",X"4544",
X"4342",X"0000",X"4747",X"4741",X"0001",X"0000",X"1203",X"0000",
X"1203",X"0000",X"1203",X"0000",X"0000",X"0000",X"0063",X"0000",
X"0000",X"4443",X"4443",X"0000",X"4241",X"0002",X"0000",X"1203",
X"0000",X"1203",X"0000",X"1203",X"0000",X"0000",X"0000",X"0064",
X"0000",X"0000",X"4947",X"4844",X"0000",X"0043",X"4241",X"0003",
X"0000",X"1203",X"0000",X"1203",X"0000",X"1203",X"0000",X"0000",
X"0000",X"0066",X"0000",X"4147",X"4849",X"0000",X"4443",X"4280",
X"0000",X"0000",X"1203",X"0000",X"1203",X"0000",X"1203",X"0000",
X"0000",X"0000",X"006b",X"0000",X"4241",X"4849",X"0000",X"4443",
X"4780",X"0001",X"0000",X"1203",X"0000",X"1203",X"0000",X"1203",
X"0000",X"0000",X"0000",X"006c",X"0000",X"4342",X"4148",X"0000",
X"4445",X"4780",X"0002",X"0000",X"1203",X"0000",X"1203",X"0000",
X"1203",X"0000",X"0000",X"0000",X"006b",X"0000",X"4443",X"4241",
X"0000",X"3345",X"4780",X"0003",X"0000",X"1203",X"0000",X"1203",
X"0000",X"1203",X"0000",X"0000",X"0000",X"03c6",X"0042",X"0000",
X"0072",X"0000",X"0061",X"0000",X"006e",X"0000",X"0063",X"0000",
X"0068",X"0000",X"0000",X"0000",X"0061",X"0000",X"0041",X"0042",
X"0002",X"0000",X"0000",X"0000",X"0000",X"0000",X"0062",X"0000",
X"0041",X"0042",X"0043",X"0044",X"0045",X"0058",X"0005",X"0000",
X"0000",X"0006",X"0000",X"0000",X"0000",X"0008",X"0000",X"0000",
X"0000",X"0000",X"0063",X"0000",X"0041",X"0042",X"0043",X"0044",
X"0064",X"007b",X"007b",X"0005",X"0000",X"0000",X"0002",X"0000",
X"0000",X"0000",X"0000",X"0000",X"0064",X"0000",X"0041",X"0042",
X"0043",X"0044",X"0058",X"0064",X"ffff",X"1234",X"007b",X"0005",
X"0000",X"0000",X"0002",X"0000",X"0000",X"0002",X"0000",X"0000",
X"0000",X"0000",X"0000",X"0065",X"0000",X"0041",X"0042",X"0043",
X"0044",X"0045",X"0058",X"ffff",X"1234",X"0008",X"0000",X"0000",
X"0005",X"0000",X"0000",X"0006",X"0000",X"0000",X"0000",X"0008",
X"0000",X"0000",X"0000",X"0000",X"0066",X"0000",X"0041",X"0042",
X"0043",X"0044",X"0064",X"ffff",X"1234",X"0005",X"0000",X"0000",
X"0002",X"0000",X"0000",X"0000",X"0000",X"0000",X"0067",X"0000",
X"0041",X"0042",X"0043",X"0044",X"0064",X"ffff",X"1234",X"0005",
X"0000",X"0000",X"0002",X"0000",X"0000",X"0002",X"0000",X"0000",
X"0000",X"0000",X"0000",X"0068",X"0000",X"0041",X"0042",X"0043",
X"0044",X"0045",X"0064",X"ffff",X"1234",X"0000",X"0005",X"0000",
X"0000",X"0002",X"0000",X"0000",X"0002",X"0000",X"0000",X"0000",
X"0000",X"0000",X"0069",X"9999",X"0000",X"0041",X"0042",X"0043",
X"0044",X"0045",X"0058",X"ffff",X"1234",X"0007",X"0000",X"0000",
X"0004",X"0000",X"0006",X"0000",X"0000",X"0000",X"0008",X"0000",
X"0000",X"0000",X"0000",X"006a",X"0000",X"0041",X"0042",X"0043",
X"0044",X"0064",X"007b",X"007b",X"0005",X"0000",X"0000",X"0002",
X"0000",X"0000",X"0000",X"0000",X"0000",X"006b",X"0000",X"0041",
X"0042",X"0058",X"02d5",X"0000",X"0000",X"0000",X"0000",X"0000",
X"006c",X"0000",X"0041",X"0042",X"0043",X"0044",X"0045",X"0058",
X"02e6",X"0000",X"0000",X"0006",X"0000",X"0000",X"0000",X"0008",
X"0000",X"0000",X"0000",X"0000",X"006d",X"0000",X"0041",X"0042",
X"0043",X"0044",X"0045",X"0058",X"0000",X"0bf0",X"f809",X"0000",
X"0000",X"0006",X"0000",X"0000",X"0000",X"0008",X"0000",X"0000",
X"0000",X"0000",X"006e",X"0000",X"0041",X"0042",X"0058",X"0000",
X"0c30",X"0008",X"0000",X"0000",X"0000",X"0000",X"0000",X"006f",
X"0000",X"0041",X"0000",X"0000",X"0000",X"0000",X"03c6",X"0053",
X"0000",X"0068",X"0000",X"0069",X"0000",X"0066",X"0000",X"0074",
X"0000",X"0000",X"0000",X"0061",X"0000",X"4041",X"4243",X"1a00",
X"1e02",X"0000",X"0000",X"0000",X"0062",X"0000",X"4041",X"4243",
X"0008",X"1804",X"1e02",X"0000",X"0000",X"0000",X"0063",X"0000",
X"4041",X"4243",X"1c03",X"0000",X"8400",X"1e43",X"ff80",X"0000",
X"0000",X"0000",X"0064",X"0000",X"4041",X"4243",X"0010",X"1807",
X"0000",X"0019",X"8400",X"1807",X"ff80",X"0000",X"0000",X"0000",
X"0065",X"0000",X"4041",X"4243",X"1c02",X"0000",X"8400",X"1e42",
X"0000",X"0000",X"0000",X"0066",X"0000",X"4041",X"4243",X"0010",
X"2006",X"0000",X"0019",X"8400",X"1806",X"0000",X"0000",X"0000",
X"03c6",X"004c",X"0000",X"006f",X"0000",X"0067",X"0000",X"0069",
X"0000",X"0063",X"0000",X"0000",X"0000",X"0061",X"0000",X"0741",
X"60f3",X"2024",X"0000",X"0000",X"0000",X"0062",X"0000",X"0741",
X"60f3",X"0000",X"0000",X"0000",X"0063",X"0000",X"0041",X"1c02",
X"0000",X"0000",X"0000",X"0064",X"0000",X"f0ff",X"f08e",X"0f0f",
X"0f30",X"2027",X"0000",X"0000",X"0000",X"0065",X"0000",X"0040",
X"0001",X"2025",X"0000",X"0000",X"0000",X"0066",X"0000",X"0040",
X"0001",X"0000",X"0000",X"0000",X"0067",X"0000",X"f043",X"f002",
X"2026",X"0000",X"0000",X"0000",X"0068",X"0000",X"f043",X"f002",
X"0000",X"0000",X"0000",X"0044",X"0000",X"006f",X"0000",X"006e",
X"0000",X"0065",X"0000",X"0000",X"0000",X"0070",X"0000",X"007a",
X"003b",X"000d",X"0001",X"0000",X"0000",X"0000",X"fffc",X"fffe",
X"0000",X"0008",X"0000",X"0000",others=>X"0000");

 begin

 imem_valid <='1';
 ram_proc: process(BCLK)

 begin
   if rising_edge(BCLK) then

     if BBUS_in.BWR = '1' and BBUS_in.BREADY='1' then
       if BBUS_in.BSEL(0) = '1' then
         if BBUS_in.BMASK(3)='1' then
           RAM_BLOCK0(to_integer(unsigned( BBUS_in.BADDR(addresswidth-1 downto 2))))<=BBUS_in.BWDATA(31 downto 24);
         end if;
         if BBUS_in.BMASK(2)='1' then
           RAM_BLOCK1(to_integer(unsigned( BBUS_in.BADDR(addresswidth-1 downto 2))))<=BBUS_in.BWDATA(23 downto 16);
         end if;
         if BBUS_in.BMASK(1)='1' then
           RAM_BLOCK2(to_integer(unsigned( BBUS_in.BADDR(addresswidth-1 downto 2))))<=BBUS_in.BWDATA(15 downto 8);
         end if;
         if BBUS_in.BMASK(0)='1' then
           RAM_BLOCK3(to_integer(unsigned( BBUS_in.BADDR(addresswidth-1 downto 2))))<=BBUS_in.BWDATA(7 downto 0);
         end if; 
       end if;
     end if;
   end if;
 end process; 
 
 
 BBUS_out.BREADYOUT<='1';

 process(BCLK) 
  begin
   if rising_edge(BCLK) then
     imem_data<=         HIGHRAM (to_integer(unsigned( imem_address(addresswidth-1 downto 2))))
        & LOWRAM (to_integer(unsigned( imem_address(addresswidth-1 downto 2))));
   end if;
  end process; 
 
 
 process(BCLK) 
  begin
   if rising_edge(BCLK) then
     BRDATA<=         RAM_BLOCK0 (to_integer(unsigned( BBUS_IN.BADDR(addresswidth-1 downto 2))))
        &RAM_BLOCK1 (to_integer(unsigned( BBUS_IN.BADDR(addresswidth-1 downto 2))))
        &RAM_BLOCK2 (to_integer(unsigned( BBUS_IN.BADDR(addresswidth-1 downto 2))))
        &RAM_BLOCK3 (to_integer(unsigned( BBUS_IN.BADDR(addresswidth-1 downto 2))));
   end if;
  end process; 

  BBUS_OUT.BRDATA<=BRDATA;

 process(BCLK) 
   begin 
     if rising_edge(BCLK) then  
       BADDR_d1<=BBUS_IN.BADDR; 
       if BBUS_in.BWR = '1' and BBUS_in.BSEL(0) ='1' then 
         write_d1<='1';
       else
         write_d1<='0';
       end if;
     end if;
   end process;

 process(BCLK)
   begin
     if rising_edge(BCLK) then
       if write_d1='1' then
         HIGHRAM (to_integer(unsigned( BADDR_d1(addresswidth-1 downto 2))))<=BRDATA (31 downto 16);
         LOWRAM (to_integer(unsigned( BADDR_d1(addresswidth-1 downto 2))))<=BRDATA (15 downto 0);
       end if;
     end if;
   end process;

end;
