library ieee;
use ieee.STD_LOGIC_1164.all;

use ieee.numeric_std.all;

use work.SPB.all;

entity SPB_memory is
   generic(memory_type : string := "DEFAULT");
   port(BCLK          : in std_logic;
        BBUS_in       : in SPB_in;
        BBUS_out      : out SPB_out;

        imem_address  : in std_logic_vector(31 downto 0);
        imem_data     : out std_logic_vector(31 downto 0);
        imem_valid    : out std_logic);
end; --entity 
architecture logic of SPB_memory is
 
constant addresswidth : integer:= 13;
type ram_byte is array (0 to (2**(addresswidth-1)))
         of std_logic_vector (7 downto 0);

signal BADDR_d1: std_logic_vector (31 downto 0);
signal write_d1: std_logic;
signal BRDATA  : std_logic_vector (31 downto 0);

--block0  bit 31...24
signal RAM_BLOCK0:ram_byte:=(
X"3c",X"27",X"3c",X"24",X"3c",X"24",X"3c",X"27",X"3c",X"3c",X"36",X"24",X"24",X"ae",X"ae",X"8e",
X"32",X"16",X"00",X"8e",X"ae",X"10",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",
others=>X"00");

signal RAM_BLOCK1:ram_byte:=(
X"1c",X"9c",X"05",X"a5",X"04",X"84",X"1d",X"bd",X"10",X"14",X"94",X"11",X"12",X"11",X"12",X"13",
X"73",X"60",X"00",X"0a",X"0a",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",
others=>X"00");

signal RAM_BLOCK2:ram_byte:=(
X"00",X"80",X"00",X"00",X"00",X"02",X"00",X"02",X"20",X"20",X"00",X"00",X"00",X"00",X"00",X"00",
X"00",X"ff",X"00",X"00",X"00",X"ff",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",
others=>x"00");

signal RAM_BLOCK3:ram_byte:=(
X"01",X"70",X"00",X"7c",X"00",X"80",X"00",X"68",X"00",X"00",X"04",X"32",X"37",X"08",X"08",X"04",
X"01",X"fd",X"00",X"00",X"08",X"f9",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"0d",X"00",
others=>x"00");

  attribute RAM_STYLE: string;
  attribute RAM_STYLE of RAM_BLOCK0: signal is "block";
  attribute RAM_STYLE of RAM_BLOCK1: signal is "block";
  attribute RAM_STYLE of RAM_BLOCK2: signal is "block";
  attribute RAM_STYLE of RAM_BLOCK3: signal is "block";

type ram_halfword is array (0 to (2**(addresswidth-1)))
         of std_logic_vector (15 downto 0);

--high block  bit 31...16
signal HIGHRAM:ram_halfword:=(
X"3c1c",X"279c",X"3c05",X"24a5",X"3c04",X"2484",X"3c1d",X"27bd",
X"3c10",X"3c14",X"3694",X"2411",X"2412",X"ae11",X"ae12",X"8e13",
X"3273",X"1660",X"0000",X"8e0a",X"ae0a",X"1000",X"0000",X"0000",
X"0000",X"0000",X"0000",X"0000",X"0000",X"0000",X"0000",X"0000",
others=>X"0000");

--low block  bit 15...0
signal LOWRAM:ram_halfword:=(
X"0001",X"8070",X"0000",X"007c",X"0000",X"0280",X"0000",X"0268",
X"2000",X"2000",X"0004",X"0032",X"0037",X"0008",X"0008",X"0004",
X"0001",X"fffd",X"0000",X"0000",X"0008",X"fff9",X"0000",X"0000",
X"0000",X"0000",X"0000",X"0000",X"0000",X"0000",X"000d",X"0000",
others=>X"0000");

 begin

 imem_valid <='1';
 ram_proc: process(BCLK)

 begin
   if rising_edge(BCLK) then

     if BBUS_in.BWR = '1' and BBUS_in.BREADY='1' then
       if BBUS_in.BSEL(0) = '1' then
         if BBUS_in.BMASK(3)='1' then
           RAM_BLOCK0(to_integer(unsigned( BBUS_in.BADDR(addresswidth-1 downto 2))))<=BBUS_in.BWDATA(31 downto 24);
         end if;
         if BBUS_in.BMASK(2)='1' then
           RAM_BLOCK1(to_integer(unsigned( BBUS_in.BADDR(addresswidth-1 downto 2))))<=BBUS_in.BWDATA(23 downto 16);
         end if;
         if BBUS_in.BMASK(1)='1' then
           RAM_BLOCK2(to_integer(unsigned( BBUS_in.BADDR(addresswidth-1 downto 2))))<=BBUS_in.BWDATA(15 downto 8);
         end if;
         if BBUS_in.BMASK(0)='1' then
           RAM_BLOCK3(to_integer(unsigned( BBUS_in.BADDR(addresswidth-1 downto 2))))<=BBUS_in.BWDATA(7 downto 0);
         end if; 
       end if;
     end if;
   end if;
 end process; 
 
 
 BBUS_out.BREADYOUT<='1';

 process(BCLK) 
  begin
   if rising_edge(BCLK) then
     imem_data<=         HIGHRAM (to_integer(unsigned( imem_address(addresswidth-1 downto 2))))
        & LOWRAM (to_integer(unsigned( imem_address(addresswidth-1 downto 2))));
   end if;
  end process; 
 
 
 process(BCLK) 
  begin
   if rising_edge(BCLK) then
     BRDATA<=         RAM_BLOCK0 (to_integer(unsigned( BBUS_IN.BADDR(addresswidth-1 downto 2))))
        &RAM_BLOCK1 (to_integer(unsigned( BBUS_IN.BADDR(addresswidth-1 downto 2))))
        &RAM_BLOCK2 (to_integer(unsigned( BBUS_IN.BADDR(addresswidth-1 downto 2))))
        &RAM_BLOCK3 (to_integer(unsigned( BBUS_IN.BADDR(addresswidth-1 downto 2))));
   end if;
  end process; 

  BBUS_OUT.BRDATA<=BRDATA;

 process(BCLK) 
   begin 
     if rising_edge(BCLK) then  
       BADDR_d1<=BBUS_IN.BADDR; 
       if BBUS_in.BWR = '1' and BBUS_in.BSEL(0) ='1' then 
         write_d1<='1';
       else
         write_d1<='0';
       end if;
     end if;
   end process;

 process(BCLK)
   begin
     if rising_edge(BCLK) then
       if write_d1='1' then
         HIGHRAM (to_integer(unsigned( BADDR_d1(addresswidth-1 downto 2))))<=BRDATA (31 downto 16);
         LOWRAM (to_integer(unsigned( BADDR_d1(addresswidth-1 downto 2))))<=BRDATA (15 downto 0);
       end if;
     end if;
   end process;

end;
