library ieee;
use ieee.STD_LOGIC_1164.all;

use ieee.numeric_std.all;

use work.AHBlite.all;

entity ahb_memory is
   generic(memory_type : string := "DEFAULT");
   port(HCLK          : in std_logic;
        Hbus_in       : in AHB_in;
        Hbus_out      : out AHB_out;

        imem_address  : in std_logic_vector(31 downto 0);
        imem_data     : out std_logic_vector(31 downto 0);
        imem_valid    : out std_logic);
end; --entity 
architecture logic of ahb_memory is
 
constant addresswidth : integer:= 13;
type ram_byte is array (0 to (2**(addresswidth-1)))
         of std_logic_vector (7 downto 0);

--block0  bit 31...24
signal RAM_BLOCK0:ram_byte:=(
X"3c",X"27",X"3c",X"24",X"3c",X"24",X"3c",X"27",X"3c",X"27",X"0c",X"00",X"08",X"00",X"00",X"00",
X"27",X"af",X"03",X"3c",X"34",X"af",X"8f",X"8c",X"30",X"30",X"14",X"00",X"3c",X"34",X"90",X"a3",
X"83",X"03",X"8f",X"27",X"03",X"00",X"3c",X"27",X"03",X"27",X"af",X"af",X"af",X"03",X"af",X"3c",
X"3c",X"34",X"ac",X"3c",X"34",X"24",X"a0",X"3c",X"34",X"24",X"a0",X"3c",X"34",X"24",X"a0",X"24",
X"af",X"3c",X"34",X"8f",X"03",X"00",X"8f",X"ae",X"10",X"00",X"00",X"00",X"00",X"80",X"00",X"00",
X"00",X"00",X"00",X"00",X"00",others=>X"00");

signal RAM_BLOCK1:ram_byte:=(
X"1c",X"9c",X"05",X"a5",X"04",X"84",X"1d",X"bd",X"19",X"39",X"00",X"00",X"00",X"00",X"00",X"00",
X"bd",X"be",X"a0",X"02",X"42",X"c2",X"c2",X"42",X"42",X"42",X"40",X"00",X"02",X"42",X"42",X"c2",
X"c2",X"c0",X"be",X"bd",X"e0",X"00",X"1c",X"9c",X"99",X"bd",X"bf",X"be",X"b0",X"a0",X"bc",X"03",
X"02",X"42",X"62",X"02",X"43",X"02",X"62",X"02",X"43",X"02",X"62",X"02",X"43",X"02",X"62",X"02",
X"c2",X"02",X"50",X"99",X"20",X"00",X"dc",X"02",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",
X"00",X"00",X"00",X"00",X"00",others=>X"00");

signal RAM_BLOCK2:ram_byte:=(
X"00",X"81",X"00",X"01",X"00",X"03",X"00",X"03",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",
X"ff",X"00",X"f0",X"20",X"00",X"00",X"00",X"00",X"00",X"00",X"ff",X"00",X"20",X"00",X"00",X"00",
X"00",X"e8",X"00",X"00",X"00",X"00",X"00",X"80",X"e0",X"ff",X"00",X"00",X"00",X"f0",X"00",X"30",
X"87",X"43",X"00",X"40",X"00",X"00",X"00",X"40",X"00",X"00",X"00",X"40",X"00",X"00",X"00",X"00",
X"00",X"20",X"00",X"80",X"f8",X"00",X"00",X"00",X"ff",X"00",X"00",X"00",X"00",X"00",X"00",X"00",
X"00",X"00",X"00",X"00",X"00",others=>x"00");

signal RAM_BLOCK3:ram_byte:=(
X"01",X"20",X"00",X"50",X"00",X"50",X"00",X"38",X"00",X"98",X"26",X"00",X"0c",X"00",X"00",X"00",
X"e8",X"10",X"21",X"00",X"04",X"08",X"08",X"00",X"01",X"ff",X"fb",X"00",X"00",X"03",X"00",X"0c",
X"0c",X"21",X"10",X"18",X"08",X"00",X"01",X"88",X"21",X"d0",X"28",X"24",X"20",X"21",X"10",X"00",
X"65",X"21",X"00",X"00",X"10",X"04",X"00",X"00",X"13",X"0c",X"00",X"00",X"12",X"03",X"00",X"20",
X"18",X"00",X"08",X"2c",X"09",X"00",X"10",X"00",X"f8",X"00",X"00",X"00",X"00",X"00",X"00",X"00",
X"00",X"00",X"00",X"40",X"00",others=>x"00");

  attribute RAM_STYLE: string;
  attribute RAM_STYLE of RAM_BLOCK0: signal is "block";
  attribute RAM_STYLE of RAM_BLOCK1: signal is "block";
  attribute RAM_STYLE of RAM_BLOCK2: signal is "block";
  attribute RAM_STYLE of RAM_BLOCK3: signal is "block";

type ram_halfword is array (0 to (2**(addresswidth-1)))
         of std_logic_vector (15 downto 0);

--high block  bit 31...16
signal HIGHRAM:ram_halfword:=(
X"3c1c",X"279c",X"3c05",X"24a5",X"3c04",X"2484",X"3c1d",X"27bd",
X"3c19",X"2739",X"0c00",X"0000",X"0800",X"0000",X"0000",X"0000",
X"27bd",X"afbe",X"03a0",X"3c02",X"3442",X"afc2",X"8fc2",X"8c42",
X"3042",X"3042",X"1440",X"0000",X"3c02",X"3442",X"9042",X"a3c2",
X"83c2",X"03c0",X"8fbe",X"27bd",X"03e0",X"0000",X"3c1c",X"279c",
X"0399",X"27bd",X"afbf",X"afbe",X"afb0",X"03a0",X"afbc",X"3c03",
X"3c02",X"3442",X"ac62",X"3c02",X"3443",X"2402",X"a062",X"3c02",
X"3443",X"2402",X"a062",X"3c02",X"3443",X"2402",X"a062",X"2402",
X"afc2",X"3c02",X"3450",X"8f99",X"0320",X"0000",X"8fdc",X"ae02",
X"1000",X"0000",X"0000",X"0000",X"0000",X"8000",X"0000",X"0000",
X"0000",X"0000",X"0000",X"0000",X"0000",others=>X"0000");

--low block  bit 15...0
signal LOWRAM:ram_halfword:=(
X"0001",X"8120",X"0000",X"0150",X"0000",X"0350",X"0000",X"0338",
X"0000",X"0098",X"0026",X"0000",X"000c",X"0000",X"0000",X"0000",
X"ffe8",X"0010",X"f021",X"2000",X"0004",X"0008",X"0008",X"0000",
X"0001",X"00ff",X"fffb",X"0000",X"2000",X"0003",X"0000",X"000c",
X"000c",X"e821",X"0010",X"0018",X"0008",X"0000",X"0001",X"8088",
X"e021",X"ffd0",X"0028",X"0024",X"0020",X"f021",X"0010",X"3000",
X"8765",X"4321",X"0000",X"4000",X"0010",X"0004",X"0000",X"4000",
X"0013",X"000c",X"0000",X"4000",X"0012",X"0003",X"0000",X"0020",
X"0018",X"2000",X"0008",X"802c",X"f809",X"0000",X"0010",X"0000",
X"fff8",X"0000",X"0000",X"0000",X"0000",X"0000",X"0000",X"0000",
X"0000",X"0000",X"0000",X"0040",X"0000",others=>X"0000");

 begin

 imem_valid <='1';
 ram_proc: process(HCLK)

 begin
   if rising_edge(HCLK) then

     if HBus_in.HWRITE = '1' and Hbus_in.HTRANS(1)='1' then
       if Hbus_in.HSEL = "00000" then
         if HBUS_in.HMASK(3)='1' then
           RAM_BLOCK0(to_integer(unsigned( Hbus_in.HADDR(addresswidth-1 downto 2))))<=Hbus_in.HWDATA1(31 downto 24);
         end if;
         if HBUS_in.HMASK(2)='1' then
           RAM_BLOCK1(to_integer(unsigned( Hbus_in.HADDR(addresswidth-1 downto 2))))<=Hbus_in.HWDATA1(23 downto 16);
         end if;
         if HBUS_in.HMASK(1)='1' then
           RAM_BLOCK2(to_integer(unsigned( Hbus_in.HADDR(addresswidth-1 downto 2))))<=Hbus_in.HWDATA1(15 downto 8);
         end if;
         if HBUS_in.HMASK(0)='1' then
           RAM_BLOCK3(to_integer(unsigned( Hbus_in.HADDR(addresswidth-1 downto 2))))<=Hbus_in.HWDATA1(7 downto 0);
         end if; 
       end if;
     end if;
   end if;
 end process; 
 
 
 HBUS_out.HREADYOUT<='1';

 process(HCLK) 
  begin
   if rising_edge(HCLK) then
     imem_data<=         HIGHRAM (to_integer(unsigned( imem_address(addresswidth-1 downto 2))))
        & LOWRAM (to_integer(unsigned( imem_address(addresswidth-1 downto 2))));
   end if;
  end process; 
 
 
 process(HCLK) 
  begin
   if rising_edge(HCLK) then
     HBUS_OUT.HRDATA<=         RAM_BLOCK0 (to_integer(unsigned( HBUS_IN.HADDR(addresswidth-1 downto 2))))
        &RAM_BLOCK1 (to_integer(unsigned( HBUS_IN.HADDR(addresswidth-1 downto 2))))
        &RAM_BLOCK2 (to_integer(unsigned( HBUS_IN.HADDR(addresswidth-1 downto 2))))
        &RAM_BLOCK3 (to_integer(unsigned( HBUS_IN.HADDR(addresswidth-1 downto 2))));
   end if;
  end process; 
end;
