library ieee;
use ieee.STD_LOGIC_1164.all;

use ieee.numeric_std.all;

use work.SPB.all;

entity SPB_memory is
   generic(memory_type : string := "DEFAULT");
   port(BCLK          : in std_logic;
        BBUS_in       : in SPB_in;
        BBUS_out      : out SPB_out;

        imem_address  : in std_logic_vector(31 downto 0);
        imem_data     : out std_logic_vector(31 downto 0);
        imem_valid    : out std_logic);
end; --entity 
architecture logic of SPB_memory is
 
constant addresswidth : integer:= 13;
type ram_byte is array (0 to (2**(addresswidth-1)))
         of std_logic_vector (7 downto 0);

signal BADDR_d1: std_logic_vector (31 downto 0);
signal write_d1: std_logic;
signal BRDATA  : std_logic_vector (31 downto 0);

--block0  bit 31...24
signal RAM_BLOCK0:ram_byte:=(
X"3c",X"27",X"3c",X"24",X"3c",X"24",X"3c",X"27",X"3c",X"27",X"0c",X"00",X"08",X"27",X"af",X"03",
X"3c",X"34",X"af",X"8f",X"00",X"8c",X"00",X"30",X"30",X"14",X"00",X"3c",X"34",X"90",X"00",X"a3",
X"83",X"03",X"8f",X"27",X"03",X"00",X"27",X"af",X"af",X"af",X"03",X"3c",X"3c",X"34",X"ac",X"3c",
X"34",X"24",X"a0",X"3c",X"34",X"24",X"a0",X"3c",X"34",X"24",X"a0",X"24",X"af",X"3c",X"34",X"0c",
X"00",X"ae",X"10",X"00",X"00",others=>X"00");

signal RAM_BLOCK1:ram_byte:=(
X"1c",X"9c",X"05",X"a5",X"04",X"84",X"1d",X"bd",X"19",X"39",X"00",X"00",X"00",X"bd",X"be",X"a0",
X"02",X"42",X"c2",X"c2",X"00",X"42",X"00",X"42",X"42",X"40",X"00",X"02",X"42",X"42",X"00",X"c2",
X"c2",X"c0",X"be",X"bd",X"e0",X"00",X"bd",X"bf",X"be",X"b0",X"a0",X"02",X"03",X"63",X"43",X"02",
X"42",X"03",X"43",X"02",X"42",X"03",X"43",X"02",X"42",X"03",X"43",X"02",X"c2",X"02",X"50",X"00",
X"00",X"02",X"00",X"00",X"00",others=>X"00");

signal RAM_BLOCK2:ram_byte:=(
X"00",X"81",X"00",X"01",X"00",X"03",X"00",X"02",X"00",X"00",X"00",X"00",X"00",X"ff",X"00",X"f0",
X"20",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"ff",X"00",X"20",X"00",X"00",X"00",X"00",
X"00",X"e8",X"00",X"00",X"00",X"00",X"ff",X"00",X"00",X"00",X"f0",X"30",X"87",X"43",X"00",X"40",
X"00",X"00",X"00",X"40",X"00",X"00",X"00",X"40",X"00",X"00",X"00",X"00",X"00",X"20",X"00",X"00",
X"00",X"00",X"ff",X"00",X"00",others=>x"00");

signal RAM_BLOCK3:ram_byte:=(
X"01",X"00",X"00",X"10",X"00",X"10",X"00",X"f8",X"00",X"98",X"26",X"00",X"0c",X"f0",X"0c",X"21",
X"00",X"04",X"00",X"00",X"00",X"00",X"00",X"01",X"ff",X"f9",X"00",X"00",X"03",X"00",X"00",X"04",
X"04",X"21",X"0c",X"10",X"08",X"00",X"d8",X"24",X"20",X"1c",X"21",X"00",X"65",X"21",X"00",X"00",
X"10",X"04",X"00",X"00",X"13",X"0c",X"00",X"00",X"12",X"03",X"00",X"20",X"10",X"00",X"08",X"0d",
X"00",X"00",X"fa",X"00",X"00",others=>x"00");

  attribute RAM_STYLE: string;
  attribute RAM_STYLE of RAM_BLOCK0: signal is "block";
  attribute RAM_STYLE of RAM_BLOCK1: signal is "block";
  attribute RAM_STYLE of RAM_BLOCK2: signal is "block";
  attribute RAM_STYLE of RAM_BLOCK3: signal is "block";

type ram_halfword is array (0 to (2**(addresswidth-1)))
         of std_logic_vector (15 downto 0);

--high block  bit 31...16
signal HIGHRAM:ram_halfword:=(
X"3c1c",X"279c",X"3c05",X"24a5",X"3c04",X"2484",X"3c1d",X"27bd",
X"3c19",X"2739",X"0c00",X"0000",X"0800",X"27bd",X"afbe",X"03a0",
X"3c02",X"3442",X"afc2",X"8fc2",X"0000",X"8c42",X"0000",X"3042",
X"3042",X"1440",X"0000",X"3c02",X"3442",X"9042",X"0000",X"a3c2",
X"83c2",X"03c0",X"8fbe",X"27bd",X"03e0",X"0000",X"27bd",X"afbf",
X"afbe",X"afb0",X"03a0",X"3c02",X"3c03",X"3463",X"ac43",X"3c02",
X"3442",X"2403",X"a043",X"3c02",X"3442",X"2403",X"a043",X"3c02",
X"3442",X"2403",X"a043",X"2402",X"afc2",X"3c02",X"3450",X"0c00",
X"0000",X"ae02",X"1000",X"0000",X"0000",others=>X"0000");

--low block  bit 15...0
signal LOWRAM:ram_halfword:=(
X"0001",X"8100",X"0000",X"0110",X"0000",X"0310",X"0000",X"02f8",
X"0000",X"0098",X"0026",X"0000",X"000c",X"fff0",X"000c",X"f021",
X"2000",X"0004",X"0000",X"0000",X"0000",X"0000",X"0000",X"0001",
X"00ff",X"fff9",X"0000",X"2000",X"0003",X"0000",X"0000",X"0004",
X"0004",X"e821",X"000c",X"0010",X"0008",X"0000",X"ffd8",X"0024",
X"0020",X"001c",X"f021",X"3000",X"8765",X"4321",X"0000",X"4000",
X"0010",X"0004",X"0000",X"4000",X"0013",X"000c",X"0000",X"4000",
X"0012",X"0003",X"0000",X"0020",X"0010",X"2000",X"0008",X"000d",
X"0000",X"0000",X"fffa",X"0000",X"0000",others=>X"0000");

 begin

 imem_valid <='1';
 ram_proc: process(BCLK)

 begin
   if rising_edge(BCLK) then

     if BBUS_in.BWR = '1' and BBUS_in.BREADY='1' then
       if BBUS_in.BSEL(0) = '1' then
         if BBUS_in.BMASK(3)='1' then
           RAM_BLOCK0(to_integer(unsigned( BBUS_in.BADDR(addresswidth-1 downto 2))))<=BBUS_in.BWDATA(31 downto 24);
         end if;
         if BBUS_in.BMASK(2)='1' then
           RAM_BLOCK1(to_integer(unsigned( BBUS_in.BADDR(addresswidth-1 downto 2))))<=BBUS_in.BWDATA(23 downto 16);
         end if;
         if BBUS_in.BMASK(1)='1' then
           RAM_BLOCK2(to_integer(unsigned( BBUS_in.BADDR(addresswidth-1 downto 2))))<=BBUS_in.BWDATA(15 downto 8);
         end if;
         if BBUS_in.BMASK(0)='1' then
           RAM_BLOCK3(to_integer(unsigned( BBUS_in.BADDR(addresswidth-1 downto 2))))<=BBUS_in.BWDATA(7 downto 0);
         end if; 
       end if;
     end if;
   end if;
 end process; 
 
 
 BBUS_out.BREADYOUT<='1';

 process(BCLK) 
  begin
   if rising_edge(BCLK) then
     imem_data<=         HIGHRAM (to_integer(unsigned( imem_address(addresswidth-1 downto 2))))
        & LOWRAM (to_integer(unsigned( imem_address(addresswidth-1 downto 2))));
   end if;
  end process; 
 
 
 process(BCLK) 
  begin
   if rising_edge(BCLK) then
     BRDATA<=         RAM_BLOCK0 (to_integer(unsigned( BBUS_IN.BADDR(addresswidth-1 downto 2))))
        &RAM_BLOCK1 (to_integer(unsigned( BBUS_IN.BADDR(addresswidth-1 downto 2))))
        &RAM_BLOCK2 (to_integer(unsigned( BBUS_IN.BADDR(addresswidth-1 downto 2))))
        &RAM_BLOCK3 (to_integer(unsigned( BBUS_IN.BADDR(addresswidth-1 downto 2))));
   end if;
  end process; 

  BBUS_OUT.BRDATA<=BRDATA;

 process(BCLK) 
   begin 
     if rising_edge(BCLK) then  
       BADDR_d1<=BBUS_IN.BADDR; 
       if BBUS_in.BWR = '1' and BBUS_in.BSEL(0) ='1' then 
         write_d1<='1';
       else
         write_d1<='0';
       end if;
     end if;
   end process;

 process(BCLK)
   begin
     if rising_edge(BCLK) then
       if write_d1='1' then
         HIGHRAM (to_integer(unsigned( BADDR_d1(addresswidth-1 downto 2))))<=BRDATA (31 downto 16);
         LOWRAM (to_integer(unsigned( BADDR_d1(addresswidth-1 downto 2))))<=BRDATA (15 downto 0);
       end if;
     end if;
   end process;

end;
