library ieee;
use ieee.STD_LOGIC_1164.all;

use ieee.numeric_std.all;

use work.SPB.all;

entity SPB_memory is
   generic(memory_type : string := "DEFAULT");
   port(BCLK          : in std_logic;
        BBUS_in       : in SPB_in;
        BBUS_out      : out SPB_out;

        imem_address  : in std_logic_vector(31 downto 0);
        imem_data     : out std_logic_vector(31 downto 0);
        imem_valid    : out std_logic);
end; --entity 
architecture logic of SPB_memory is
 
constant addresswidth : integer:= 13;
type ram_byte is array (0 to (2**(addresswidth-1)))
         of std_logic_vector (7 downto 0);

signal BADDR_d1: std_logic_vector (31 downto 0);
signal write_d1: std_logic;
signal BRDATA  : std_logic_vector (31 downto 0);

--block0  bit 31...24
signal RAM_BLOCK0:ram_byte:=(
X"3c",X"27",X"3c",X"24",X"3c",X"24",X"3c",X"27",X"3c",X"27",X"0c",X"00",X"08",X"00",X"00",X"00",
X"27",X"af",X"03",X"af",X"24",X"a7",X"10",X"00",X"8f",X"00",X"84",X"00",X"00",X"8f",X"00",X"24",
X"84",X"00",X"00",X"af",X"8f",X"00",X"84",X"00",X"00",X"8f",X"00",X"24",X"84",X"00",X"00",X"af",
X"8f",X"00",X"24",X"84",X"00",X"00",X"8f",X"00",X"24",X"84",X"00",X"00",X"af",X"8f",X"00",X"24",
X"84",X"00",X"00",X"8f",X"00",X"24",X"84",X"00",X"00",X"af",X"8f",X"00",X"24",X"84",X"00",X"00",
X"8f",X"00",X"24",X"84",X"00",X"00",X"af",X"8f",X"00",X"24",X"84",X"00",X"00",X"8f",X"00",X"24",
X"84",X"00",X"00",X"af",X"8f",X"00",X"24",X"84",X"00",X"00",X"8f",X"00",X"24",X"84",X"00",X"00",
X"af",X"8f",X"00",X"24",X"84",X"00",X"00",X"8f",X"00",X"24",X"84",X"00",X"00",X"af",X"8f",X"8f",
X"00",X"00",X"af",X"8f",X"8f",X"00",X"00",X"af",X"8f",X"8f",X"00",X"00",X"af",X"8f",X"8f",X"00",
X"00",X"af",X"8f",X"00",X"30",X"8f",X"00",X"30",X"00",X"30",X"00",X"00",X"8f",X"00",X"a4",X"8f",
X"00",X"24",X"8f",X"00",X"30",X"8f",X"00",X"30",X"00",X"30",X"00",X"00",X"a4",X"8f",X"00",X"24",
X"97",X"00",X"00",X"8f",X"00",X"00",X"00",X"97",X"00",X"00",X"8f",X"00",X"00",X"00",X"00",X"97",
X"00",X"00",X"00",X"00",X"a4",X"8f",X"00",X"24",X"97",X"00",X"00",X"8f",X"00",X"00",X"00",X"97",
X"00",X"00",X"8f",X"00",X"00",X"00",X"00",X"97",X"00",X"00",X"00",X"00",X"a4",X"8f",X"00",X"24",
X"97",X"00",X"00",X"8f",X"00",X"00",X"00",X"97",X"00",X"00",X"8f",X"00",X"00",X"00",X"00",X"97",
X"00",X"00",X"8f",X"00",X"00",X"00",X"00",X"97",X"00",X"00",X"8f",X"00",X"00",X"00",X"00",X"97",
X"00",X"00",X"00",X"00",X"a4",X"8f",X"00",X"24",X"97",X"00",X"00",X"8f",X"00",X"00",X"00",X"97",
X"00",X"00",X"8f",X"00",X"00",X"00",X"00",X"97",X"00",X"00",X"8f",X"00",X"00",X"00",X"97",X"00",
X"00",X"8f",X"00",X"00",X"00",X"00",X"00",X"97",X"00",X"00",X"00",X"00",X"a4",X"8f",X"00",X"24",
X"97",X"00",X"00",X"8f",X"00",X"00",X"00",X"97",X"00",X"00",X"8f",X"00",X"00",X"00",X"00",X"97",
X"00",X"00",X"8f",X"00",X"00",X"00",X"00",X"97",X"00",X"00",X"8f",X"00",X"00",X"00",X"00",X"97",
X"00",X"00",X"00",X"00",X"a4",X"8f",X"00",X"24",X"97",X"00",X"00",X"8f",X"00",X"00",X"00",X"97",
X"00",X"00",X"8f",X"00",X"00",X"00",X"00",X"97",X"00",X"00",X"8f",X"00",X"00",X"00",X"97",X"00",
X"00",X"8f",X"00",X"00",X"00",X"00",X"00",X"97",X"00",X"00",X"00",X"00",X"a4",X"8f",X"00",X"24",
X"af",X"97",X"00",X"24",X"a7",X"97",X"00",X"14",X"00",X"8f",X"00",X"24",X"af",X"24",X"a7",X"10",
X"00",X"8f",X"00",X"84",X"00",X"00",X"8f",X"00",X"24",X"84",X"00",X"00",X"af",X"8f",X"00",X"84",
X"00",X"00",X"8f",X"00",X"24",X"84",X"00",X"00",X"af",X"8f",X"00",X"24",X"84",X"00",X"00",X"8f",
X"00",X"24",X"84",X"00",X"00",X"af",X"8f",X"00",X"24",X"84",X"00",X"00",X"8f",X"00",X"24",X"84",
X"00",X"00",X"af",X"8f",X"00",X"24",X"84",X"00",X"00",X"8f",X"00",X"24",X"84",X"00",X"00",X"af",
X"8f",X"00",X"24",X"84",X"00",X"00",X"8f",X"00",X"24",X"84",X"00",X"00",X"af",X"8f",X"00",X"24",
X"84",X"00",X"00",X"8f",X"00",X"24",X"84",X"00",X"00",X"af",X"8f",X"00",X"24",X"84",X"00",X"00",
X"8f",X"00",X"24",X"84",X"00",X"00",X"af",X"8f",X"8f",X"00",X"00",X"af",X"8f",X"8f",X"00",X"00",
X"af",X"8f",X"8f",X"00",X"00",X"af",X"8f",X"8f",X"00",X"00",X"af",X"8f",X"8f",X"00",X"00",X"97",
X"00",X"00",X"00",X"00",X"8f",X"00",X"a4",X"8f",X"00",X"24",X"8f",X"8f",X"00",X"00",X"97",X"00",
X"00",X"00",X"00",X"a4",X"8f",X"00",X"24",X"97",X"00",X"00",X"8f",X"00",X"00",X"00",X"97",X"00",
X"00",X"8f",X"00",X"00",X"00",X"00",X"97",X"00",X"00",X"00",X"00",X"a4",X"8f",X"00",X"24",X"97",
X"00",X"00",X"8f",X"00",X"00",X"00",X"97",X"00",X"00",X"8f",X"00",X"00",X"00",X"00",X"97",X"00",
X"00",X"00",X"00",X"a4",X"8f",X"00",X"24",X"97",X"00",X"00",X"8f",X"00",X"00",X"00",X"97",X"00",
X"00",X"8f",X"00",X"00",X"00",X"00",X"97",X"00",X"00",X"8f",X"00",X"00",X"00",X"00",X"97",X"00",
X"00",X"8f",X"00",X"00",X"00",X"00",X"97",X"00",X"00",X"00",X"00",X"a4",X"8f",X"00",X"24",X"97",
X"00",X"00",X"8f",X"00",X"00",X"00",X"97",X"00",X"00",X"8f",X"00",X"00",X"00",X"00",X"97",X"00",
X"00",X"8f",X"00",X"00",X"00",X"97",X"00",X"00",X"8f",X"00",X"00",X"00",X"00",X"00",X"97",X"00",
X"00",X"00",X"00",X"a4",X"8f",X"00",X"24",X"97",X"00",X"00",X"8f",X"00",X"00",X"00",X"97",X"00",
X"00",X"8f",X"00",X"00",X"00",X"00",X"97",X"00",X"00",X"8f",X"00",X"00",X"00",X"00",X"97",X"00",
X"00",X"8f",X"00",X"00",X"00",X"00",X"97",X"00",X"00",X"00",X"00",X"a4",X"8f",X"00",X"24",X"97",
X"00",X"00",X"8f",X"00",X"00",X"00",X"97",X"00",X"00",X"8f",X"00",X"00",X"00",X"00",X"97",X"00",
X"00",X"8f",X"00",X"00",X"00",X"97",X"00",X"00",X"8f",X"00",X"00",X"00",X"00",X"00",X"97",X"00",
X"00",X"00",X"00",X"a4",X"8f",X"00",X"24",X"af",X"97",X"00",X"24",X"a7",X"97",X"00",X"14",X"00",
X"03",X"8f",X"27",X"03",X"00",X"27",X"af",X"03",X"00",X"a3",X"3c",X"34",X"af",X"8f",X"00",X"90",
X"00",X"00",X"00",X"30",X"30",X"14",X"00",X"3c",X"34",X"93",X"00",X"a0",X"24",X"03",X"8f",X"27",
X"03",X"00",X"27",X"af",X"03",X"af",X"10",X"00",X"3c",X"34",X"af",X"8f",X"00",X"90",X"00",X"00",
X"00",X"30",X"30",X"14",X"00",X"3c",X"34",X"8f",X"00",X"80",X"00",X"a0",X"8f",X"00",X"24",X"af",
X"8f",X"00",X"80",X"00",X"14",X"00",X"03",X"8f",X"27",X"03",X"00",X"27",X"af",X"03",X"af",X"af",
X"af",X"8f",X"00",X"af",X"8f",X"00",X"04",X"00",X"8f",X"24",X"a0",X"8f",X"00",X"24",X"af",X"8f",
X"00",X"00",X"af",X"10",X"00",X"8f",X"8f",X"24",X"14",X"00",X"00",X"00",X"00",X"00",X"03",X"a0",
X"8f",X"24",X"14",X"00",X"00",X"00",X"00",X"af",X"8f",X"00",X"24",X"af",X"8f",X"00",X"14",X"00",
X"10",X"00",X"8f",X"00",X"03",X"80",X"00",X"30",X"24",X"30",X"00",X"00",X"8f",X"00",X"a0",X"8f",
X"00",X"24",X"af",X"8f",X"00",X"00",X"30",X"8f",X"00",X"24",X"af",X"14",X"00",X"8f",X"00",X"a0",
X"8f",X"03",X"8f",X"27",X"03",X"00",X"27",X"af",X"af",X"03",X"3c",X"24",X"0c",X"00",X"af",X"10",
X"00",X"af",X"10",X"00",X"8f",X"8f",X"00",X"00",X"3c",X"00",X"24",X"00",X"84",X"00",X"00",X"27",
X"00",X"00",X"0c",X"00",X"00",X"0c",X"00",X"24",X"0c",X"00",X"8f",X"00",X"24",X"af",X"8f",X"00",
X"28",X"14",X"00",X"24",X"0c",X"00",X"8f",X"00",X"24",X"af",X"8f",X"00",X"28",X"14",X"00",X"03",
X"8f",X"8f",X"27",X"03",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",
X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",
X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"05",X"00",X"03",X"05",X"00",X"00",others=>X"00");

signal RAM_BLOCK1:ram_byte:=(
X"1c",X"9c",X"05",X"a5",X"04",X"84",X"1d",X"bd",X"19",X"39",X"00",X"00",X"00",X"00",X"00",X"00",
X"bd",X"be",X"a0",X"c4",X"02",X"c2",X"00",X"00",X"c2",X"00",X"42",X"00",X"40",X"c2",X"00",X"42",
X"42",X"00",X"62",X"c2",X"c2",X"00",X"42",X"00",X"40",X"c2",X"00",X"42",X"42",X"00",X"62",X"c2",
X"c2",X"00",X"42",X"42",X"00",X"40",X"c2",X"00",X"42",X"42",X"00",X"62",X"c2",X"c2",X"00",X"42",
X"42",X"00",X"40",X"c2",X"00",X"42",X"42",X"00",X"62",X"c2",X"c2",X"00",X"42",X"42",X"00",X"40",
X"c2",X"00",X"42",X"42",X"00",X"62",X"c2",X"c2",X"00",X"42",X"42",X"00",X"40",X"c2",X"00",X"42",
X"42",X"00",X"62",X"c2",X"c2",X"00",X"42",X"42",X"00",X"40",X"c2",X"00",X"42",X"42",X"00",X"62",
X"c2",X"c2",X"00",X"42",X"42",X"00",X"40",X"c2",X"00",X"42",X"42",X"00",X"62",X"c2",X"c3",X"c2",
X"00",X"62",X"c2",X"c3",X"c2",X"00",X"62",X"c2",X"c3",X"c2",X"00",X"62",X"c2",X"c3",X"c2",X"00",
X"62",X"c2",X"c2",X"00",X"43",X"c2",X"00",X"42",X"62",X"42",X"02",X"03",X"c2",X"00",X"43",X"c2",
X"00",X"42",X"c3",X"00",X"64",X"c3",X"00",X"63",X"83",X"63",X"03",X"03",X"43",X"c2",X"00",X"42",
X"83",X"00",X"60",X"c3",X"00",X"83",X"00",X"83",X"00",X"60",X"c3",X"00",X"a3",X"00",X"83",X"83",
X"00",X"64",X"03",X"03",X"43",X"c2",X"00",X"42",X"83",X"00",X"60",X"c3",X"00",X"83",X"00",X"83",
X"00",X"60",X"c3",X"00",X"a3",X"00",X"83",X"83",X"00",X"64",X"03",X"03",X"43",X"c2",X"00",X"42",
X"83",X"00",X"60",X"c3",X"00",X"83",X"00",X"83",X"00",X"60",X"c3",X"00",X"a3",X"00",X"83",X"84",
X"00",X"80",X"c4",X"00",X"a4",X"00",X"64",X"84",X"00",X"80",X"c4",X"00",X"a4",X"00",X"64",X"83",
X"00",X"64",X"03",X"03",X"43",X"c2",X"00",X"42",X"83",X"00",X"60",X"c3",X"00",X"83",X"00",X"83",
X"00",X"60",X"c3",X"00",X"a3",X"00",X"83",X"83",X"00",X"60",X"c3",X"00",X"a3",X"00",X"83",X"00",
X"60",X"c3",X"00",X"c3",X"00",X"a3",X"83",X"83",X"00",X"64",X"03",X"03",X"43",X"c2",X"00",X"42",
X"83",X"00",X"60",X"c3",X"00",X"83",X"00",X"83",X"00",X"60",X"c3",X"00",X"a3",X"00",X"83",X"84",
X"00",X"80",X"c4",X"00",X"a4",X"00",X"64",X"84",X"00",X"80",X"c4",X"00",X"a4",X"00",X"64",X"83",
X"00",X"64",X"03",X"03",X"43",X"c2",X"00",X"42",X"83",X"00",X"60",X"c3",X"00",X"83",X"00",X"83",
X"00",X"60",X"c3",X"00",X"a3",X"00",X"83",X"83",X"00",X"60",X"c3",X"00",X"a3",X"00",X"83",X"00",
X"60",X"c3",X"00",X"c3",X"00",X"a3",X"83",X"83",X"00",X"64",X"03",X"03",X"43",X"c2",X"00",X"42",
X"c2",X"c2",X"00",X"42",X"c2",X"c2",X"00",X"40",X"00",X"c2",X"00",X"42",X"c2",X"02",X"c2",X"00",
X"00",X"c2",X"00",X"42",X"00",X"40",X"c2",X"00",X"42",X"42",X"00",X"62",X"c2",X"c2",X"00",X"42",
X"00",X"40",X"c2",X"00",X"42",X"42",X"00",X"62",X"c2",X"c2",X"00",X"42",X"42",X"00",X"40",X"c2",
X"00",X"42",X"42",X"00",X"62",X"c2",X"c2",X"00",X"42",X"42",X"00",X"40",X"c2",X"00",X"42",X"42",
X"00",X"62",X"c2",X"c2",X"00",X"42",X"42",X"00",X"40",X"c2",X"00",X"42",X"42",X"00",X"62",X"c2",
X"c2",X"00",X"42",X"42",X"00",X"40",X"c2",X"00",X"42",X"42",X"00",X"62",X"c2",X"c2",X"00",X"42",
X"42",X"00",X"40",X"c2",X"00",X"42",X"42",X"00",X"62",X"c2",X"c2",X"00",X"42",X"42",X"00",X"40",
X"c2",X"00",X"42",X"42",X"00",X"62",X"c2",X"c3",X"c2",X"00",X"62",X"c2",X"c3",X"c2",X"00",X"62",
X"c2",X"c3",X"c2",X"00",X"62",X"c2",X"c3",X"c2",X"00",X"62",X"c2",X"c3",X"c2",X"00",X"62",X"82",
X"00",X"43",X"02",X"03",X"c2",X"00",X"43",X"c2",X"00",X"42",X"c4",X"c3",X"00",X"83",X"83",X"00",
X"64",X"03",X"03",X"43",X"c2",X"00",X"42",X"83",X"00",X"60",X"c3",X"00",X"83",X"00",X"83",X"00",
X"60",X"c3",X"00",X"a3",X"00",X"83",X"83",X"00",X"64",X"03",X"03",X"43",X"c2",X"00",X"42",X"83",
X"00",X"60",X"c3",X"00",X"83",X"00",X"83",X"00",X"60",X"c3",X"00",X"a3",X"00",X"83",X"83",X"00",
X"64",X"03",X"03",X"43",X"c2",X"00",X"42",X"83",X"00",X"60",X"c3",X"00",X"83",X"00",X"83",X"00",
X"60",X"c3",X"00",X"a3",X"00",X"83",X"84",X"00",X"80",X"c4",X"00",X"a4",X"00",X"64",X"84",X"00",
X"80",X"c4",X"00",X"a4",X"00",X"64",X"83",X"00",X"64",X"03",X"03",X"43",X"c2",X"00",X"42",X"83",
X"00",X"60",X"c3",X"00",X"83",X"00",X"83",X"00",X"60",X"c3",X"00",X"a3",X"00",X"83",X"83",X"00",
X"60",X"c3",X"00",X"a3",X"00",X"83",X"00",X"60",X"c3",X"00",X"c3",X"00",X"a3",X"83",X"83",X"00",
X"64",X"03",X"03",X"43",X"c2",X"00",X"42",X"83",X"00",X"60",X"c3",X"00",X"83",X"00",X"83",X"00",
X"60",X"c3",X"00",X"a3",X"00",X"83",X"84",X"00",X"80",X"c4",X"00",X"a4",X"00",X"64",X"84",X"00",
X"80",X"c4",X"00",X"a4",X"00",X"64",X"83",X"00",X"64",X"03",X"03",X"43",X"c2",X"00",X"42",X"83",
X"00",X"60",X"c3",X"00",X"83",X"00",X"83",X"00",X"60",X"c3",X"00",X"a3",X"00",X"83",X"83",X"00",
X"60",X"c3",X"00",X"a3",X"00",X"83",X"00",X"60",X"c3",X"00",X"c3",X"00",X"a3",X"83",X"83",X"00",
X"64",X"03",X"03",X"43",X"c2",X"00",X"42",X"c2",X"c2",X"00",X"42",X"c2",X"c2",X"00",X"40",X"00",
X"c0",X"be",X"bd",X"e0",X"00",X"bd",X"be",X"a0",X"80",X"c2",X"02",X"42",X"c2",X"c2",X"00",X"42",
X"00",X"02",X"02",X"42",X"42",X"40",X"00",X"02",X"42",X"c3",X"00",X"43",X"02",X"c0",X"be",X"bd",
X"e0",X"00",X"bd",X"be",X"a0",X"c4",X"00",X"00",X"02",X"42",X"c2",X"c2",X"00",X"42",X"00",X"02",
X"02",X"42",X"42",X"40",X"00",X"02",X"42",X"c3",X"00",X"63",X"00",X"43",X"c2",X"00",X"42",X"c2",
X"c2",X"00",X"42",X"00",X"40",X"00",X"c0",X"be",X"bd",X"e0",X"00",X"bd",X"be",X"a0",X"c4",X"c5",
X"c0",X"c2",X"00",X"c2",X"c2",X"00",X"41",X"00",X"c2",X"03",X"43",X"c2",X"00",X"42",X"c2",X"c2",
X"00",X"02",X"c2",X"00",X"00",X"c2",X"c4",X"03",X"60",X"83",X"07",X"00",X"03",X"03",X"c2",X"43",
X"c3",X"02",X"40",X"62",X"07",X"00",X"00",X"c2",X"c2",X"00",X"42",X"c2",X"c2",X"00",X"40",X"00",
X"00",X"00",X"c2",X"00",X"c2",X"42",X"00",X"42",X"42",X"42",X"02",X"03",X"c2",X"00",X"43",X"c2",
X"00",X"42",X"c2",X"c2",X"00",X"02",X"42",X"c3",X"00",X"63",X"c3",X"40",X"00",X"c2",X"00",X"40",
X"c2",X"c0",X"be",X"bd",X"e0",X"00",X"bd",X"bf",X"be",X"a0",X"02",X"44",X"00",X"00",X"c0",X"00",
X"00",X"c0",X"00",X"00",X"c3",X"c2",X"00",X"62",X"02",X"03",X"42",X"62",X"42",X"00",X"40",X"c2",
X"60",X"40",X"00",X"00",X"40",X"00",X"00",X"04",X"00",X"00",X"c2",X"00",X"42",X"c2",X"c2",X"00",
X"42",X"40",X"00",X"04",X"00",X"00",X"c2",X"00",X"42",X"c2",X"c2",X"00",X"42",X"40",X"00",X"c0",
X"bf",X"be",X"bd",X"e0",X"00",X"00",X"00",X"00",X"28",X"21",X"1a",X"24",X"2b",X"21",X"1d",X"16",
X"2b",X"13",X"24",X"0f",X"24",X"1a",X"1d",X"13",X"21",X"1d",X"0f",X"1a",X"24",X"21",X"16",X"1a",
X"21",X"1d",X"21",X"0c",X"1a",X"1d",X"0c",X"08",X"00",X"3a",X"0a",X"25",X"8c",X"0d",X"00",others=>X"00");

signal RAM_BLOCK2:ram_byte:=(
X"00",X"90",X"00",X"10",X"00",X"12",X"00",X"12",X"00",X"0e",X"03",X"00",X"00",X"00",X"00",X"00",
X"ff",X"00",X"f0",X"00",X"00",X"00",X"01",X"00",X"00",X"00",X"00",X"00",X"18",X"00",X"00",X"00",
X"00",X"00",X"10",X"00",X"00",X"00",X"00",X"00",X"18",X"00",X"00",X"00",X"00",X"00",X"10",X"00",
X"00",X"00",X"00",X"00",X"00",X"18",X"00",X"00",X"00",X"00",X"00",X"10",X"00",X"00",X"00",X"00",
X"00",X"00",X"18",X"00",X"00",X"00",X"00",X"00",X"10",X"00",X"00",X"00",X"00",X"00",X"00",X"18",
X"00",X"00",X"00",X"00",X"00",X"10",X"00",X"00",X"00",X"00",X"00",X"00",X"18",X"00",X"00",X"00",
X"00",X"00",X"10",X"00",X"00",X"00",X"00",X"00",X"00",X"18",X"00",X"00",X"00",X"00",X"00",X"10",
X"00",X"00",X"00",X"00",X"00",X"00",X"18",X"00",X"00",X"00",X"00",X"00",X"10",X"00",X"00",X"00",
X"00",X"10",X"00",X"00",X"00",X"00",X"10",X"00",X"00",X"00",X"00",X"10",X"00",X"00",X"00",X"00",
X"10",X"00",X"00",X"00",X"ff",X"00",X"00",X"ff",X"10",X"ff",X"1c",X"1c",X"00",X"00",X"00",X"00",
X"00",X"00",X"00",X"00",X"ff",X"00",X"00",X"ff",X"18",X"ff",X"1c",X"1c",X"00",X"00",X"00",X"00",
X"80",X"00",X"20",X"00",X"00",X"00",X"20",X"80",X"00",X"28",X"00",X"00",X"00",X"18",X"20",X"80",
X"00",X"18",X"1c",X"1c",X"00",X"00",X"00",X"00",X"80",X"00",X"20",X"00",X"00",X"00",X"20",X"80",
X"00",X"28",X"00",X"00",X"00",X"18",X"20",X"80",X"00",X"18",X"1c",X"1c",X"00",X"00",X"00",X"00",
X"80",X"00",X"20",X"00",X"00",X"00",X"20",X"80",X"00",X"28",X"00",X"00",X"00",X"18",X"18",X"80",
X"00",X"28",X"00",X"00",X"00",X"20",X"18",X"80",X"00",X"28",X"00",X"00",X"00",X"20",X"20",X"80",
X"00",X"18",X"1c",X"1c",X"00",X"00",X"00",X"00",X"80",X"00",X"20",X"00",X"00",X"00",X"20",X"80",
X"00",X"28",X"00",X"00",X"00",X"18",X"20",X"80",X"00",X"28",X"00",X"00",X"00",X"28",X"80",X"00",
X"30",X"00",X"00",X"00",X"18",X"18",X"20",X"80",X"00",X"18",X"1c",X"1c",X"00",X"00",X"00",X"00",
X"80",X"00",X"20",X"00",X"00",X"00",X"20",X"80",X"00",X"28",X"00",X"00",X"00",X"18",X"18",X"80",
X"00",X"28",X"00",X"00",X"00",X"20",X"18",X"80",X"00",X"28",X"00",X"00",X"00",X"20",X"20",X"80",
X"00",X"18",X"1c",X"1c",X"00",X"00",X"00",X"00",X"80",X"00",X"20",X"00",X"00",X"00",X"20",X"80",
X"00",X"28",X"00",X"00",X"00",X"18",X"20",X"80",X"00",X"28",X"00",X"00",X"00",X"28",X"80",X"00",
X"30",X"00",X"00",X"00",X"18",X"18",X"20",X"80",X"00",X"18",X"1c",X"1c",X"00",X"00",X"00",X"00",
X"00",X"00",X"00",X"ff",X"00",X"00",X"00",X"fe",X"00",X"00",X"00",X"ff",X"00",X"00",X"00",X"01",
X"00",X"00",X"00",X"00",X"00",X"18",X"00",X"00",X"00",X"00",X"00",X"10",X"00",X"00",X"00",X"00",
X"00",X"18",X"00",X"00",X"00",X"00",X"00",X"10",X"00",X"00",X"00",X"00",X"00",X"00",X"18",X"00",
X"00",X"00",X"00",X"00",X"10",X"00",X"00",X"00",X"00",X"00",X"00",X"18",X"00",X"00",X"00",X"00",
X"00",X"10",X"00",X"00",X"00",X"00",X"00",X"00",X"18",X"00",X"00",X"00",X"00",X"00",X"10",X"00",
X"00",X"00",X"00",X"00",X"00",X"18",X"00",X"00",X"00",X"00",X"00",X"10",X"00",X"00",X"00",X"00",
X"00",X"00",X"18",X"00",X"00",X"00",X"00",X"00",X"10",X"00",X"00",X"00",X"00",X"00",X"00",X"18",
X"00",X"00",X"00",X"00",X"00",X"10",X"00",X"00",X"00",X"00",X"10",X"00",X"00",X"00",X"00",X"10",
X"00",X"00",X"00",X"00",X"10",X"00",X"00",X"00",X"00",X"10",X"00",X"00",X"00",X"00",X"18",X"80",
X"00",X"10",X"1c",X"1c",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"20",X"80",X"00",
X"18",X"1c",X"1c",X"00",X"00",X"00",X"00",X"80",X"00",X"20",X"00",X"00",X"00",X"20",X"80",X"00",
X"28",X"00",X"00",X"00",X"18",X"20",X"80",X"00",X"18",X"1c",X"1c",X"00",X"00",X"00",X"00",X"80",
X"00",X"20",X"00",X"00",X"00",X"20",X"80",X"00",X"28",X"00",X"00",X"00",X"18",X"20",X"80",X"00",
X"18",X"1c",X"1c",X"00",X"00",X"00",X"00",X"80",X"00",X"20",X"00",X"00",X"00",X"20",X"80",X"00",
X"28",X"00",X"00",X"00",X"18",X"18",X"80",X"00",X"28",X"00",X"00",X"00",X"20",X"18",X"80",X"00",
X"28",X"00",X"00",X"00",X"20",X"20",X"80",X"00",X"18",X"1c",X"1c",X"00",X"00",X"00",X"00",X"80",
X"00",X"20",X"00",X"00",X"00",X"20",X"80",X"00",X"28",X"00",X"00",X"00",X"18",X"20",X"80",X"00",
X"28",X"00",X"00",X"00",X"28",X"80",X"00",X"30",X"00",X"00",X"00",X"18",X"18",X"20",X"80",X"00",
X"18",X"1c",X"1c",X"00",X"00",X"00",X"00",X"80",X"00",X"20",X"00",X"00",X"00",X"20",X"80",X"00",
X"28",X"00",X"00",X"00",X"18",X"18",X"80",X"00",X"28",X"00",X"00",X"00",X"20",X"18",X"80",X"00",
X"28",X"00",X"00",X"00",X"20",X"20",X"80",X"00",X"18",X"1c",X"1c",X"00",X"00",X"00",X"00",X"80",
X"00",X"20",X"00",X"00",X"00",X"20",X"80",X"00",X"28",X"00",X"00",X"00",X"18",X"20",X"80",X"00",
X"28",X"00",X"00",X"00",X"28",X"80",X"00",X"30",X"00",X"00",X"00",X"18",X"18",X"20",X"80",X"00",
X"18",X"1c",X"1c",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"ff",X"00",X"00",X"00",X"fe",X"00",
X"e8",X"00",X"00",X"00",X"00",X"ff",X"00",X"f0",X"10",X"00",X"20",X"00",X"00",X"00",X"00",X"00",
X"00",X"16",X"16",X"00",X"00",X"ff",X"00",X"20",X"00",X"00",X"00",X"00",X"00",X"e8",X"00",X"00",
X"00",X"00",X"ff",X"00",X"f0",X"00",X"00",X"00",X"20",X"00",X"00",X"00",X"00",X"00",X"00",X"16",
X"16",X"00",X"00",X"ff",X"00",X"20",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",
X"00",X"00",X"00",X"00",X"ff",X"00",X"e8",X"00",X"00",X"00",X"00",X"ff",X"00",X"f0",X"00",X"00",
X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",
X"00",X"10",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"18",X"1e",X"1e",X"10",X"00",
X"00",X"00",X"00",X"00",X"00",X"18",X"10",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"ff",X"00",
X"00",X"00",X"00",X"00",X"10",X"00",X"00",X"00",X"00",X"00",X"1e",X"1e",X"00",X"00",X"00",X"00",
X"00",X"00",X"00",X"00",X"00",X"10",X"00",X"00",X"00",X"ff",X"00",X"ff",X"00",X"00",X"00",X"00",
X"00",X"e8",X"00",X"00",X"00",X"00",X"ff",X"00",X"00",X"f0",X"00",X"0f",X"00",X"00",X"00",X"00",
X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"18",X"00",X"18",X"0f",X"10",X"00",X"00",X"18",X"00",
X"20",X"28",X"03",X"00",X"20",X"03",X"00",X"00",X"03",X"00",X"00",X"00",X"00",X"00",X"00",X"00",
X"00",X"ff",X"00",X"00",X"03",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"ff",X"00",X"e8",
X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",
X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",
X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"00",X"0f",X"02",X"01",X"04",X"00",X"00",X"00",others=>x"00");

signal RAM_BLOCK3:ram_byte:=(
X"01",X"10",X"00",X"38",X"00",X"40",X"00",X"28",X"00",X"98",X"a6",X"00",X"0c",X"00",X"00",X"00",
X"d0",X"2c",X"21",X"30",X"08",X"24",X"6e",X"00",X"30",X"00",X"00",X"00",X"21",X"30",X"00",X"0e",
X"00",X"00",X"21",X"00",X"30",X"00",X"00",X"00",X"21",X"30",X"00",X"0e",X"00",X"00",X"23",X"20",
X"30",X"00",X"02",X"00",X"00",X"21",X"30",X"00",X"0c",X"00",X"00",X"21",X"04",X"30",X"00",X"02",
X"00",X"00",X"21",X"30",X"00",X"0c",X"00",X"00",X"23",X"1c",X"30",X"00",X"04",X"00",X"00",X"21",
X"30",X"00",X"0a",X"00",X"00",X"21",X"08",X"30",X"00",X"04",X"00",X"00",X"21",X"30",X"00",X"0a",
X"00",X"00",X"23",X"18",X"30",X"00",X"06",X"00",X"00",X"21",X"30",X"00",X"08",X"00",X"00",X"21",
X"0c",X"30",X"00",X"06",X"00",X"00",X"21",X"30",X"00",X"08",X"00",X"00",X"23",X"14",X"00",X"0c",
X"00",X"21",X"10",X"00",X"0c",X"00",X"23",X"00",X"04",X"08",X"00",X"21",X"0c",X"04",X"08",X"00",
X"23",X"04",X"10",X"00",X"ff",X"0c",X"00",X"ff",X"21",X"ff",X"00",X"03",X"30",X"00",X"00",X"30",
X"00",X"08",X"10",X"00",X"ff",X"0c",X"00",X"ff",X"23",X"ff",X"00",X"03",X"00",X"30",X"00",X"04",
X"14",X"00",X"21",X"00",X"00",X"18",X"12",X"16",X"00",X"21",X"04",X"00",X"18",X"12",X"21",X"18",
X"00",X"07",X"00",X"03",X"00",X"30",X"00",X"0c",X"16",X"00",X"21",X"00",X"00",X"18",X"12",X"14",
X"00",X"21",X"04",X"00",X"18",X"12",X"23",X"18",X"00",X"07",X"00",X"03",X"00",X"30",X"00",X"0e",
X"1a",X"00",X"21",X"20",X"00",X"18",X"12",X"1c",X"00",X"21",X"1c",X"00",X"18",X"12",X"23",X"1e",
X"00",X"21",X"18",X"00",X"18",X"12",X"21",X"20",X"00",X"21",X"14",X"00",X"18",X"12",X"23",X"18",
X"00",X"07",X"00",X"03",X"00",X"30",X"00",X"0a",X"1c",X"00",X"21",X"20",X"00",X"18",X"12",X"20",
X"00",X"21",X"1c",X"00",X"18",X"12",X"23",X"1e",X"00",X"21",X"14",X"00",X"18",X"12",X"1a",X"00",
X"21",X"18",X"00",X"18",X"12",X"21",X"21",X"18",X"00",X"07",X"00",X"03",X"00",X"30",X"00",X"06",
X"1e",X"00",X"21",X"20",X"00",X"18",X"12",X"1a",X"00",X"21",X"1c",X"00",X"18",X"12",X"23",X"20",
X"00",X"21",X"18",X"00",X"18",X"12",X"23",X"1c",X"00",X"21",X"14",X"00",X"18",X"12",X"23",X"18",
X"00",X"07",X"00",X"03",X"00",X"30",X"00",X"02",X"20",X"00",X"21",X"20",X"00",X"18",X"12",X"1e",
X"00",X"21",X"1c",X"00",X"18",X"12",X"21",X"1a",X"00",X"21",X"14",X"00",X"18",X"12",X"1c",X"00",
X"21",X"18",X"00",X"18",X"12",X"21",X"21",X"18",X"00",X"07",X"00",X"03",X"00",X"30",X"00",X"10",
X"30",X"24",X"00",X"ff",X"24",X"24",X"00",X"90",X"00",X"30",X"00",X"80",X"30",X"08",X"24",X"6c",
X"00",X"30",X"00",X"00",X"00",X"21",X"30",X"00",X"70",X"00",X"00",X"21",X"00",X"30",X"00",X"00",
X"00",X"21",X"30",X"00",X"70",X"00",X"00",X"23",X"20",X"30",X"00",X"10",X"00",X"00",X"21",X"30",
X"00",X"60",X"00",X"00",X"21",X"04",X"30",X"00",X"10",X"00",X"00",X"21",X"30",X"00",X"60",X"00",
X"00",X"23",X"1c",X"30",X"00",X"20",X"00",X"00",X"21",X"30",X"00",X"50",X"00",X"00",X"21",X"08",
X"30",X"00",X"20",X"00",X"00",X"21",X"30",X"00",X"50",X"00",X"00",X"23",X"18",X"30",X"00",X"30",
X"00",X"00",X"21",X"30",X"00",X"40",X"00",X"00",X"21",X"0c",X"30",X"00",X"30",X"00",X"00",X"21",
X"30",X"00",X"40",X"00",X"00",X"23",X"14",X"00",X"0c",X"00",X"21",X"10",X"00",X"0c",X"00",X"23",
X"00",X"04",X"08",X"00",X"21",X"0c",X"04",X"08",X"00",X"23",X"04",X"10",X"0c",X"00",X"21",X"22",
X"00",X"07",X"00",X"03",X"30",X"00",X"00",X"30",X"00",X"40",X"10",X"0c",X"00",X"23",X"22",X"00",
X"07",X"00",X"03",X"00",X"30",X"00",X"20",X"14",X"00",X"21",X"00",X"00",X"18",X"12",X"16",X"00",
X"21",X"04",X"00",X"18",X"12",X"21",X"24",X"00",X"07",X"00",X"03",X"00",X"30",X"00",X"60",X"16",
X"00",X"21",X"00",X"00",X"18",X"12",X"14",X"00",X"21",X"04",X"00",X"18",X"12",X"23",X"24",X"00",
X"07",X"00",X"03",X"00",X"30",X"00",X"70",X"1a",X"00",X"21",X"20",X"00",X"18",X"12",X"1c",X"00",
X"21",X"1c",X"00",X"18",X"12",X"23",X"1e",X"00",X"21",X"18",X"00",X"18",X"12",X"21",X"20",X"00",
X"21",X"14",X"00",X"18",X"12",X"23",X"24",X"00",X"07",X"00",X"03",X"00",X"30",X"00",X"50",X"1c",
X"00",X"21",X"20",X"00",X"18",X"12",X"20",X"00",X"21",X"1c",X"00",X"18",X"12",X"23",X"1e",X"00",
X"21",X"14",X"00",X"18",X"12",X"1a",X"00",X"21",X"18",X"00",X"18",X"12",X"21",X"21",X"24",X"00",
X"07",X"00",X"03",X"00",X"30",X"00",X"30",X"1e",X"00",X"21",X"20",X"00",X"18",X"12",X"1a",X"00",
X"21",X"1c",X"00",X"18",X"12",X"23",X"20",X"00",X"21",X"18",X"00",X"18",X"12",X"23",X"1c",X"00",
X"21",X"14",X"00",X"18",X"12",X"23",X"24",X"00",X"07",X"00",X"03",X"00",X"30",X"00",X"10",X"20",
X"00",X"21",X"20",X"00",X"18",X"12",X"1e",X"00",X"21",X"1c",X"00",X"18",X"12",X"21",X"1a",X"00",
X"21",X"14",X"00",X"18",X"12",X"1c",X"00",X"21",X"18",X"00",X"18",X"12",X"21",X"21",X"24",X"00",
X"07",X"00",X"03",X"00",X"30",X"00",X"02",X"30",X"24",X"00",X"ff",X"24",X"24",X"00",X"92",X"00",
X"21",X"2c",X"30",X"08",X"00",X"f0",X"0c",X"21",X"21",X"10",X"00",X"07",X"00",X"00",X"00",X"00",
X"00",X"00",X"03",X"ff",X"02",X"f7",X"00",X"00",X"08",X"10",X"00",X"00",X"01",X"21",X"0c",X"10",
X"08",X"00",X"f0",X"0c",X"21",X"10",X"19",X"00",X"00",X"07",X"00",X"00",X"00",X"00",X"00",X"00",
X"03",X"ff",X"02",X"f7",X"00",X"00",X"08",X"10",X"00",X"00",X"00",X"00",X"10",X"00",X"01",X"10",
X"10",X"00",X"00",X"00",X"e3",X"00",X"21",X"0c",X"10",X"08",X"00",X"d0",X"2c",X"21",X"30",X"34",
X"04",X"34",X"00",X"00",X"30",X"00",X"25",X"00",X"00",X"2d",X"00",X"00",X"00",X"01",X"00",X"30",
X"00",X"23",X"30",X"18",X"00",X"04",X"30",X"0a",X"02",X"1a",X"0d",X"10",X"00",X"03",X"21",X"08",
X"30",X"0a",X"02",X"1a",X"0d",X"10",X"12",X"30",X"04",X"00",X"01",X"04",X"30",X"00",X"e6",X"00",
X"12",X"00",X"04",X"00",X"21",X"08",X"00",X"ff",X"30",X"ff",X"00",X"03",X"00",X"00",X"00",X"00",
X"00",X"01",X"00",X"04",X"00",X"2b",X"ff",X"04",X"00",X"ff",X"04",X"e6",X"00",X"00",X"00",X"00",
X"34",X"21",X"2c",X"30",X"08",X"00",X"c0",X"3c",X"38",X"21",X"00",X"a0",X"10",X"00",X"10",X"2a",
X"00",X"14",X"1b",X"00",X"14",X"10",X"00",X"21",X"00",X"40",X"a0",X"21",X"00",X"00",X"21",X"18",
X"21",X"21",X"4b",X"00",X"21",X"22",X"00",X"20",X"05",X"00",X"14",X"00",X"01",X"14",X"14",X"00",
X"08",X"e2",X"00",X"0d",X"05",X"00",X"10",X"00",X"08",X"10",X"10",X"00",X"40",X"d3",X"00",X"21",
X"3c",X"38",X"40",X"08",X"00",X"00",X"00",X"00",X"21",X"16",X"28",X"1a",X"1a",X"16",X"2b",X"24",
X"28",X"24",X"21",X"1a",X"2b",X"21",X"1d",X"04",X"21",X"1a",X"21",X"04",X"21",X"1a",X"1d",X"0c",
X"28",X"1a",X"0c",X"04",X"21",X"16",X"04",X"00",X"a0",X"2a",X"1b",X"b4",X"03",X"00",X"00",others=>x"00");

  attribute RAM_STYLE: string;
  attribute RAM_STYLE of RAM_BLOCK0: signal is "block";
  attribute RAM_STYLE of RAM_BLOCK1: signal is "block";
  attribute RAM_STYLE of RAM_BLOCK2: signal is "block";
  attribute RAM_STYLE of RAM_BLOCK3: signal is "block";

type ram_halfword is array (0 to (2**(addresswidth-1)))
         of std_logic_vector (15 downto 0);

--high block  bit 31...16
signal HIGHRAM:ram_halfword:=(
X"3c1c",X"279c",X"3c05",X"24a5",X"3c04",X"2484",X"3c1d",X"27bd",
X"3c19",X"2739",X"0c00",X"0000",X"0800",X"0000",X"0000",X"0000",
X"27bd",X"afbe",X"03a0",X"afc4",X"2402",X"a7c2",X"1000",X"0000",
X"8fc2",X"0000",X"8442",X"0000",X"0040",X"8fc2",X"0000",X"2442",
X"8442",X"0000",X"0062",X"afc2",X"8fc2",X"0000",X"8442",X"0000",
X"0040",X"8fc2",X"0000",X"2442",X"8442",X"0000",X"0062",X"afc2",
X"8fc2",X"0000",X"2442",X"8442",X"0000",X"0040",X"8fc2",X"0000",
X"2442",X"8442",X"0000",X"0062",X"afc2",X"8fc2",X"0000",X"2442",
X"8442",X"0000",X"0040",X"8fc2",X"0000",X"2442",X"8442",X"0000",
X"0062",X"afc2",X"8fc2",X"0000",X"2442",X"8442",X"0000",X"0040",
X"8fc2",X"0000",X"2442",X"8442",X"0000",X"0062",X"afc2",X"8fc2",
X"0000",X"2442",X"8442",X"0000",X"0040",X"8fc2",X"0000",X"2442",
X"8442",X"0000",X"0062",X"afc2",X"8fc2",X"0000",X"2442",X"8442",
X"0000",X"0040",X"8fc2",X"0000",X"2442",X"8442",X"0000",X"0062",
X"afc2",X"8fc2",X"0000",X"2442",X"8442",X"0000",X"0040",X"8fc2",
X"0000",X"2442",X"8442",X"0000",X"0062",X"afc2",X"8fc3",X"8fc2",
X"0000",X"0062",X"afc2",X"8fc3",X"8fc2",X"0000",X"0062",X"afc2",
X"8fc3",X"8fc2",X"0000",X"0062",X"afc2",X"8fc3",X"8fc2",X"0000",
X"0062",X"afc2",X"8fc2",X"0000",X"3043",X"8fc2",X"0000",X"3042",
X"0062",X"3042",X"0002",X"0003",X"8fc2",X"0000",X"a443",X"8fc2",
X"0000",X"2442",X"8fc3",X"0000",X"3064",X"8fc3",X"0000",X"3063",
X"0083",X"3063",X"0003",X"0003",X"a443",X"8fc2",X"0000",X"2442",
X"9783",X"0000",X"0060",X"8fc3",X"0000",X"0083",X"0000",X"9783",
X"0000",X"0060",X"8fc3",X"0000",X"00a3",X"0000",X"0083",X"9783",
X"0000",X"0064",X"0003",X"0003",X"a443",X"8fc2",X"0000",X"2442",
X"9783",X"0000",X"0060",X"8fc3",X"0000",X"0083",X"0000",X"9783",
X"0000",X"0060",X"8fc3",X"0000",X"00a3",X"0000",X"0083",X"9783",
X"0000",X"0064",X"0003",X"0003",X"a443",X"8fc2",X"0000",X"2442",
X"9783",X"0000",X"0060",X"8fc3",X"0000",X"0083",X"0000",X"9783",
X"0000",X"0060",X"8fc3",X"0000",X"00a3",X"0000",X"0083",X"9784",
X"0000",X"0080",X"8fc4",X"0000",X"00a4",X"0000",X"0064",X"9784",
X"0000",X"0080",X"8fc4",X"0000",X"00a4",X"0000",X"0064",X"9783",
X"0000",X"0064",X"0003",X"0003",X"a443",X"8fc2",X"0000",X"2442",
X"9783",X"0000",X"0060",X"8fc3",X"0000",X"0083",X"0000",X"9783",
X"0000",X"0060",X"8fc3",X"0000",X"00a3",X"0000",X"0083",X"9783",
X"0000",X"0060",X"8fc3",X"0000",X"00a3",X"0000",X"9783",X"0000",
X"0060",X"8fc3",X"0000",X"00c3",X"0000",X"00a3",X"0083",X"9783",
X"0000",X"0064",X"0003",X"0003",X"a443",X"8fc2",X"0000",X"2442",
X"9783",X"0000",X"0060",X"8fc3",X"0000",X"0083",X"0000",X"9783",
X"0000",X"0060",X"8fc3",X"0000",X"00a3",X"0000",X"0083",X"9784",
X"0000",X"0080",X"8fc4",X"0000",X"00a4",X"0000",X"0064",X"9784",
X"0000",X"0080",X"8fc4",X"0000",X"00a4",X"0000",X"0064",X"9783",
X"0000",X"0064",X"0003",X"0003",X"a443",X"8fc2",X"0000",X"2442",
X"9783",X"0000",X"0060",X"8fc3",X"0000",X"0083",X"0000",X"9783",
X"0000",X"0060",X"8fc3",X"0000",X"00a3",X"0000",X"0083",X"9783",
X"0000",X"0060",X"8fc3",X"0000",X"00a3",X"0000",X"9783",X"0000",
X"0060",X"8fc3",X"0000",X"00c3",X"0000",X"00a3",X"0083",X"9783",
X"0000",X"0064",X"0003",X"0003",X"a443",X"8fc2",X"0000",X"2442",
X"afc2",X"97c2",X"0000",X"2442",X"a7c2",X"97c2",X"0000",X"1440",
X"0000",X"8fc2",X"0000",X"2442",X"afc2",X"2402",X"a7c2",X"1000",
X"0000",X"8fc2",X"0000",X"8442",X"0000",X"0040",X"8fc2",X"0000",
X"2442",X"8442",X"0000",X"0062",X"afc2",X"8fc2",X"0000",X"8442",
X"0000",X"0040",X"8fc2",X"0000",X"2442",X"8442",X"0000",X"0062",
X"afc2",X"8fc2",X"0000",X"2442",X"8442",X"0000",X"0040",X"8fc2",
X"0000",X"2442",X"8442",X"0000",X"0062",X"afc2",X"8fc2",X"0000",
X"2442",X"8442",X"0000",X"0040",X"8fc2",X"0000",X"2442",X"8442",
X"0000",X"0062",X"afc2",X"8fc2",X"0000",X"2442",X"8442",X"0000",
X"0040",X"8fc2",X"0000",X"2442",X"8442",X"0000",X"0062",X"afc2",
X"8fc2",X"0000",X"2442",X"8442",X"0000",X"0040",X"8fc2",X"0000",
X"2442",X"8442",X"0000",X"0062",X"afc2",X"8fc2",X"0000",X"2442",
X"8442",X"0000",X"0040",X"8fc2",X"0000",X"2442",X"8442",X"0000",
X"0062",X"afc2",X"8fc2",X"0000",X"2442",X"8442",X"0000",X"0040",
X"8fc2",X"0000",X"2442",X"8442",X"0000",X"0062",X"afc2",X"8fc3",
X"8fc2",X"0000",X"0062",X"afc2",X"8fc3",X"8fc2",X"0000",X"0062",
X"afc2",X"8fc3",X"8fc2",X"0000",X"0062",X"afc2",X"8fc3",X"8fc2",
X"0000",X"0062",X"afc2",X"8fc3",X"8fc2",X"0000",X"0062",X"9782",
X"0000",X"0043",X"0002",X"0003",X"8fc2",X"0000",X"a443",X"8fc2",
X"0000",X"2442",X"8fc4",X"8fc3",X"0000",X"0083",X"9783",X"0000",
X"0064",X"0003",X"0003",X"a443",X"8fc2",X"0000",X"2442",X"9783",
X"0000",X"0060",X"8fc3",X"0000",X"0083",X"0000",X"9783",X"0000",
X"0060",X"8fc3",X"0000",X"00a3",X"0000",X"0083",X"9783",X"0000",
X"0064",X"0003",X"0003",X"a443",X"8fc2",X"0000",X"2442",X"9783",
X"0000",X"0060",X"8fc3",X"0000",X"0083",X"0000",X"9783",X"0000",
X"0060",X"8fc3",X"0000",X"00a3",X"0000",X"0083",X"9783",X"0000",
X"0064",X"0003",X"0003",X"a443",X"8fc2",X"0000",X"2442",X"9783",
X"0000",X"0060",X"8fc3",X"0000",X"0083",X"0000",X"9783",X"0000",
X"0060",X"8fc3",X"0000",X"00a3",X"0000",X"0083",X"9784",X"0000",
X"0080",X"8fc4",X"0000",X"00a4",X"0000",X"0064",X"9784",X"0000",
X"0080",X"8fc4",X"0000",X"00a4",X"0000",X"0064",X"9783",X"0000",
X"0064",X"0003",X"0003",X"a443",X"8fc2",X"0000",X"2442",X"9783",
X"0000",X"0060",X"8fc3",X"0000",X"0083",X"0000",X"9783",X"0000",
X"0060",X"8fc3",X"0000",X"00a3",X"0000",X"0083",X"9783",X"0000",
X"0060",X"8fc3",X"0000",X"00a3",X"0000",X"9783",X"0000",X"0060",
X"8fc3",X"0000",X"00c3",X"0000",X"00a3",X"0083",X"9783",X"0000",
X"0064",X"0003",X"0003",X"a443",X"8fc2",X"0000",X"2442",X"9783",
X"0000",X"0060",X"8fc3",X"0000",X"0083",X"0000",X"9783",X"0000",
X"0060",X"8fc3",X"0000",X"00a3",X"0000",X"0083",X"9784",X"0000",
X"0080",X"8fc4",X"0000",X"00a4",X"0000",X"0064",X"9784",X"0000",
X"0080",X"8fc4",X"0000",X"00a4",X"0000",X"0064",X"9783",X"0000",
X"0064",X"0003",X"0003",X"a443",X"8fc2",X"0000",X"2442",X"9783",
X"0000",X"0060",X"8fc3",X"0000",X"0083",X"0000",X"9783",X"0000",
X"0060",X"8fc3",X"0000",X"00a3",X"0000",X"0083",X"9783",X"0000",
X"0060",X"8fc3",X"0000",X"00a3",X"0000",X"9783",X"0000",X"0060",
X"8fc3",X"0000",X"00c3",X"0000",X"00a3",X"0083",X"9783",X"0000",
X"0064",X"0003",X"0003",X"a443",X"8fc2",X"0000",X"2442",X"afc2",
X"97c2",X"0000",X"2442",X"a7c2",X"97c2",X"0000",X"1440",X"0000",
X"03c0",X"8fbe",X"27bd",X"03e0",X"0000",X"27bd",X"afbe",X"03a0",
X"0080",X"a3c2",X"3c02",X"3442",X"afc2",X"8fc2",X"0000",X"9042",
X"0000",X"0002",X"0002",X"3042",X"3042",X"1440",X"0000",X"3c02",
X"3442",X"93c3",X"0000",X"a043",X"2402",X"03c0",X"8fbe",X"27bd",
X"03e0",X"0000",X"27bd",X"afbe",X"03a0",X"afc4",X"1000",X"0000",
X"3c02",X"3442",X"afc2",X"8fc2",X"0000",X"9042",X"0000",X"0002",
X"0002",X"3042",X"3042",X"1440",X"0000",X"3c02",X"3442",X"8fc3",
X"0000",X"8063",X"0000",X"a043",X"8fc2",X"0000",X"2442",X"afc2",
X"8fc2",X"0000",X"8042",X"0000",X"1440",X"0000",X"03c0",X"8fbe",
X"27bd",X"03e0",X"0000",X"27bd",X"afbe",X"03a0",X"afc4",X"afc5",
X"afc0",X"8fc2",X"0000",X"afc2",X"8fc2",X"0000",X"0441",X"0000",
X"8fc2",X"2403",X"a043",X"8fc2",X"0000",X"2442",X"afc2",X"8fc2",
X"0000",X"0002",X"afc2",X"1000",X"0000",X"8fc2",X"8fc4",X"2403",
X"1460",X"0083",X"0007",X"0000",X"0003",X"0003",X"03c2",X"a043",
X"8fc3",X"2402",X"1440",X"0062",X"0007",X"0000",X"0000",X"afc2",
X"8fc2",X"0000",X"2442",X"afc2",X"8fc2",X"0000",X"1440",X"0000",
X"1000",X"0000",X"8fc2",X"0000",X"03c2",X"8042",X"0000",X"3042",
X"2442",X"3042",X"0002",X"0003",X"8fc2",X"0000",X"a043",X"8fc2",
X"0000",X"2442",X"afc2",X"8fc2",X"0000",X"0002",X"3042",X"8fc3",
X"0000",X"2463",X"afc3",X"1440",X"0000",X"8fc2",X"0000",X"a040",
X"8fc2",X"03c0",X"8fbe",X"27bd",X"03e0",X"0000",X"27bd",X"afbf",
X"afbe",X"03a0",X"3c02",X"2444",X"0c00",X"0000",X"afc0",X"1000",
X"0000",X"afc0",X"1000",X"0000",X"8fc3",X"8fc2",X"0000",X"0062",
X"3c02",X"0003",X"2442",X"0062",X"8442",X"0000",X"0040",X"27c2",
X"0060",X"0040",X"0c00",X"0000",X"0040",X"0c00",X"0000",X"2404",
X"0c00",X"0000",X"8fc2",X"0000",X"2442",X"afc2",X"8fc2",X"0000",
X"2842",X"1440",X"0000",X"2404",X"0c00",X"0000",X"8fc2",X"0000",
X"2442",X"afc2",X"8fc2",X"0000",X"2842",X"1440",X"0000",X"03c0",
X"8fbf",X"8fbe",X"27bd",X"03e0",X"0000",X"0000",X"0000",X"0000",
X"0028",X"0021",X"001a",X"0024",X"002b",X"0021",X"001d",X"0016",
X"002b",X"0013",X"0024",X"000f",X"0024",X"001a",X"001d",X"0013",
X"0021",X"001d",X"000f",X"001a",X"0024",X"0021",X"0016",X"001a",
X"0021",X"001d",X"0021",X"000c",X"001a",X"001d",X"000c",X"0008",
X"0000",X"053a",X"000a",X"0325",X"058c",X"000d",X"0000",others=>X"0000");

--low block  bit 15...0
signal LOWRAM:ram_halfword:=(
X"0001",X"9010",X"0000",X"1038",X"0000",X"1240",X"0000",X"1228",
X"0000",X"0e98",X"03a6",X"0000",X"000c",X"0000",X"0000",X"0000",
X"ffd0",X"002c",X"f021",X"0030",X"0008",X"0024",X"016e",X"0000",
X"0030",X"0000",X"0000",X"0000",X"1821",X"0030",X"0000",X"000e",
X"0000",X"0000",X"1021",X"0000",X"0030",X"0000",X"0000",X"0000",
X"1821",X"0030",X"0000",X"000e",X"0000",X"0000",X"1023",X"0020",
X"0030",X"0000",X"0002",X"0000",X"0000",X"1821",X"0030",X"0000",
X"000c",X"0000",X"0000",X"1021",X"0004",X"0030",X"0000",X"0002",
X"0000",X"0000",X"1821",X"0030",X"0000",X"000c",X"0000",X"0000",
X"1023",X"001c",X"0030",X"0000",X"0004",X"0000",X"0000",X"1821",
X"0030",X"0000",X"000a",X"0000",X"0000",X"1021",X"0008",X"0030",
X"0000",X"0004",X"0000",X"0000",X"1821",X"0030",X"0000",X"000a",
X"0000",X"0000",X"1023",X"0018",X"0030",X"0000",X"0006",X"0000",
X"0000",X"1821",X"0030",X"0000",X"0008",X"0000",X"0000",X"1021",
X"000c",X"0030",X"0000",X"0006",X"0000",X"0000",X"1821",X"0030",
X"0000",X"0008",X"0000",X"0000",X"1023",X"0014",X"0000",X"000c",
X"0000",X"1021",X"0010",X"0000",X"000c",X"0000",X"1023",X"0000",
X"0004",X"0008",X"0000",X"1021",X"000c",X"0004",X"0008",X"0000",
X"1023",X"0004",X"0010",X"0000",X"ffff",X"000c",X"0000",X"ffff",
X"1021",X"ffff",X"1c00",X"1c03",X"0030",X"0000",X"0000",X"0030",
X"0000",X"0008",X"0010",X"0000",X"ffff",X"000c",X"0000",X"ffff",
X"1823",X"ffff",X"1c00",X"1c03",X"0000",X"0030",X"0000",X"0004",
X"8014",X"0000",X"2021",X"0000",X"0000",X"0018",X"2012",X"8016",
X"0000",X"2821",X"0004",X"0000",X"0018",X"1812",X"2021",X"8018",
X"0000",X"1807",X"1c00",X"1c03",X"0000",X"0030",X"0000",X"000c",
X"8016",X"0000",X"2021",X"0000",X"0000",X"0018",X"2012",X"8014",
X"0000",X"2821",X"0004",X"0000",X"0018",X"1812",X"2023",X"8018",
X"0000",X"1807",X"1c00",X"1c03",X"0000",X"0030",X"0000",X"000e",
X"801a",X"0000",X"2021",X"0020",X"0000",X"0018",X"2012",X"801c",
X"0000",X"2821",X"001c",X"0000",X"0018",X"1812",X"1823",X"801e",
X"0000",X"2821",X"0018",X"0000",X"0018",X"2012",X"1821",X"8020",
X"0000",X"2821",X"0014",X"0000",X"0018",X"2012",X"2023",X"8018",
X"0000",X"1807",X"1c00",X"1c03",X"0000",X"0030",X"0000",X"000a",
X"801c",X"0000",X"2021",X"0020",X"0000",X"0018",X"2012",X"8020",
X"0000",X"2821",X"001c",X"0000",X"0018",X"1812",X"2023",X"801e",
X"0000",X"2821",X"0014",X"0000",X"0018",X"2812",X"801a",X"0000",
X"3021",X"0018",X"0000",X"0018",X"1812",X"1821",X"2021",X"8018",
X"0000",X"1807",X"1c00",X"1c03",X"0000",X"0030",X"0000",X"0006",
X"801e",X"0000",X"2021",X"0020",X"0000",X"0018",X"2012",X"801a",
X"0000",X"2821",X"001c",X"0000",X"0018",X"1812",X"1823",X"8020",
X"0000",X"2821",X"0018",X"0000",X"0018",X"2012",X"1823",X"801c",
X"0000",X"2821",X"0014",X"0000",X"0018",X"2012",X"2023",X"8018",
X"0000",X"1807",X"1c00",X"1c03",X"0000",X"0030",X"0000",X"0002",
X"8020",X"0000",X"2021",X"0020",X"0000",X"0018",X"2012",X"801e",
X"0000",X"2821",X"001c",X"0000",X"0018",X"1812",X"2021",X"801a",
X"0000",X"2821",X"0014",X"0000",X"0018",X"2812",X"801c",X"0000",
X"3021",X"0018",X"0000",X"0018",X"1812",X"1821",X"2021",X"8018",
X"0000",X"1807",X"1c00",X"1c03",X"0000",X"0030",X"0000",X"0010",
X"0030",X"0024",X"0000",X"ffff",X"0024",X"0024",X"0000",X"fe90",
X"0000",X"0030",X"0000",X"ff80",X"0030",X"0008",X"0024",X"016c",
X"0000",X"0030",X"0000",X"0000",X"0000",X"1821",X"0030",X"0000",
X"0070",X"0000",X"0000",X"1021",X"0000",X"0030",X"0000",X"0000",
X"0000",X"1821",X"0030",X"0000",X"0070",X"0000",X"0000",X"1023",
X"0020",X"0030",X"0000",X"0010",X"0000",X"0000",X"1821",X"0030",
X"0000",X"0060",X"0000",X"0000",X"1021",X"0004",X"0030",X"0000",
X"0010",X"0000",X"0000",X"1821",X"0030",X"0000",X"0060",X"0000",
X"0000",X"1023",X"001c",X"0030",X"0000",X"0020",X"0000",X"0000",
X"1821",X"0030",X"0000",X"0050",X"0000",X"0000",X"1021",X"0008",
X"0030",X"0000",X"0020",X"0000",X"0000",X"1821",X"0030",X"0000",
X"0050",X"0000",X"0000",X"1023",X"0018",X"0030",X"0000",X"0030",
X"0000",X"0000",X"1821",X"0030",X"0000",X"0040",X"0000",X"0000",
X"1021",X"000c",X"0030",X"0000",X"0030",X"0000",X"0000",X"1821",
X"0030",X"0000",X"0040",X"0000",X"0000",X"1023",X"0014",X"0000",
X"000c",X"0000",X"1021",X"0010",X"0000",X"000c",X"0000",X"1023",
X"0000",X"0004",X"0008",X"0000",X"1021",X"000c",X"0004",X"0008",
X"0000",X"1023",X"0004",X"0010",X"000c",X"0000",X"1821",X"8022",
X"0000",X"1007",X"1c00",X"1c03",X"0030",X"0000",X"0000",X"0030",
X"0000",X"0040",X"0010",X"000c",X"0000",X"2023",X"8022",X"0000",
X"1807",X"1c00",X"1c03",X"0000",X"0030",X"0000",X"0020",X"8014",
X"0000",X"2021",X"0000",X"0000",X"0018",X"2012",X"8016",X"0000",
X"2821",X"0004",X"0000",X"0018",X"1812",X"2021",X"8024",X"0000",
X"1807",X"1c00",X"1c03",X"0000",X"0030",X"0000",X"0060",X"8016",
X"0000",X"2021",X"0000",X"0000",X"0018",X"2012",X"8014",X"0000",
X"2821",X"0004",X"0000",X"0018",X"1812",X"2023",X"8024",X"0000",
X"1807",X"1c00",X"1c03",X"0000",X"0030",X"0000",X"0070",X"801a",
X"0000",X"2021",X"0020",X"0000",X"0018",X"2012",X"801c",X"0000",
X"2821",X"001c",X"0000",X"0018",X"1812",X"1823",X"801e",X"0000",
X"2821",X"0018",X"0000",X"0018",X"2012",X"1821",X"8020",X"0000",
X"2821",X"0014",X"0000",X"0018",X"2012",X"2023",X"8024",X"0000",
X"1807",X"1c00",X"1c03",X"0000",X"0030",X"0000",X"0050",X"801c",
X"0000",X"2021",X"0020",X"0000",X"0018",X"2012",X"8020",X"0000",
X"2821",X"001c",X"0000",X"0018",X"1812",X"2023",X"801e",X"0000",
X"2821",X"0014",X"0000",X"0018",X"2812",X"801a",X"0000",X"3021",
X"0018",X"0000",X"0018",X"1812",X"1821",X"2021",X"8024",X"0000",
X"1807",X"1c00",X"1c03",X"0000",X"0030",X"0000",X"0030",X"801e",
X"0000",X"2021",X"0020",X"0000",X"0018",X"2012",X"801a",X"0000",
X"2821",X"001c",X"0000",X"0018",X"1812",X"1823",X"8020",X"0000",
X"2821",X"0018",X"0000",X"0018",X"2012",X"1823",X"801c",X"0000",
X"2821",X"0014",X"0000",X"0018",X"2012",X"2023",X"8024",X"0000",
X"1807",X"1c00",X"1c03",X"0000",X"0030",X"0000",X"0010",X"8020",
X"0000",X"2021",X"0020",X"0000",X"0018",X"2012",X"801e",X"0000",
X"2821",X"001c",X"0000",X"0018",X"1812",X"2021",X"801a",X"0000",
X"2821",X"0014",X"0000",X"0018",X"2812",X"801c",X"0000",X"3021",
X"0018",X"0000",X"0018",X"1812",X"1821",X"2021",X"8024",X"0000",
X"1807",X"1c00",X"1c03",X"0000",X"0030",X"0000",X"0002",X"0030",
X"0024",X"0000",X"ffff",X"0024",X"0024",X"0000",X"fe92",X"0000",
X"e821",X"002c",X"0030",X"0008",X"0000",X"fff0",X"000c",X"f021",
X"1021",X"0010",X"2000",X"0007",X"0000",X"0000",X"0000",X"0000",
X"0000",X"1600",X"1603",X"00ff",X"0002",X"fff7",X"0000",X"2000",
X"0008",X"0010",X"0000",X"0000",X"0001",X"e821",X"000c",X"0010",
X"0008",X"0000",X"fff0",X"000c",X"f021",X"0010",X"0019",X"0000",
X"2000",X"0007",X"0000",X"0000",X"0000",X"0000",X"0000",X"1600",
X"1603",X"00ff",X"0002",X"fff7",X"0000",X"2000",X"0008",X"0010",
X"0000",X"0000",X"0000",X"0000",X"0010",X"0000",X"0001",X"0010",
X"0010",X"0000",X"0000",X"0000",X"ffe3",X"0000",X"e821",X"000c",
X"0010",X"0008",X"0000",X"ffd0",X"002c",X"f021",X"0030",X"0034",
X"0004",X"0034",X"0000",X"0000",X"0030",X"0000",X"0025",X"0000",
X"0000",X"002d",X"0000",X"0000",X"0000",X"0001",X"0000",X"0030",
X"0000",X"1023",X"0030",X"0018",X"0000",X"0004",X"0030",X"000a",
X"0002",X"001a",X"000d",X"1810",X"1e00",X"1e03",X"1021",X"0008",
X"0030",X"000a",X"0002",X"001a",X"000d",X"1810",X"1012",X"0030",
X"0004",X"0000",X"0001",X"0004",X"0030",X"0000",X"ffe6",X"0000",
X"0012",X"0000",X"0004",X"0000",X"1021",X"0008",X"0000",X"00ff",
X"0030",X"00ff",X"1e00",X"1e03",X"0000",X"0000",X"0000",X"0000",
X"0000",X"0001",X"0000",X"0004",X"0000",X"102b",X"00ff",X"0004",
X"0000",X"ffff",X"0004",X"ffe6",X"0000",X"0000",X"0000",X"0000",
X"0034",X"e821",X"002c",X"0030",X"0008",X"0000",X"ffc0",X"003c",
X"0038",X"f021",X"0000",X"0fa0",X"0010",X"0000",X"0010",X"002a",
X"0000",X"0014",X"001b",X"0000",X"0014",X"0010",X"0000",X"1821",
X"0000",X"1840",X"0fa0",X"1021",X"0000",X"0000",X"1821",X"0018",
X"2021",X"2821",X"034b",X"0000",X"2021",X"0322",X"0000",X"0020",
X"0305",X"0000",X"0014",X"0000",X"0001",X"0014",X"0014",X"0000",
X"0008",X"ffe2",X"0000",X"000d",X"0305",X"0000",X"0010",X"0000",
X"0008",X"0010",X"0010",X"0000",X"0040",X"ffd3",X"0000",X"e821",
X"003c",X"0038",X"0040",X"0008",X"0000",X"0000",X"0000",X"0000",
X"0021",X"0016",X"0028",X"001a",X"001a",X"0016",X"002b",X"0024",
X"0028",X"0024",X"0021",X"001a",X"002b",X"0021",X"001d",X"0004",
X"0021",X"001a",X"0021",X"0004",X"0021",X"001a",X"001d",X"000c",
X"0028",X"001a",X"000c",X"0004",X"0021",X"0016",X"0004",X"0000",
X"0fa0",X"022a",X"011b",X"04b4",X"0003",X"0000",X"0000",others=>X"0000");

 begin

 imem_valid <='1';
 ram_proc: process(BCLK)

 begin
   if rising_edge(BCLK) then

     if BBUS_in.BWR = '1' and BBUS_in.BREADY='1' then
       if BBUS_in.BSEL(0) = '1' then
         if BBUS_in.BMASK(3)='1' then
           RAM_BLOCK0(to_integer(unsigned( BBUS_in.BADDR(addresswidth-1 downto 2))))<=BBUS_in.BWDATA(31 downto 24);
         end if;
         if BBUS_in.BMASK(2)='1' then
           RAM_BLOCK1(to_integer(unsigned( BBUS_in.BADDR(addresswidth-1 downto 2))))<=BBUS_in.BWDATA(23 downto 16);
         end if;
         if BBUS_in.BMASK(1)='1' then
           RAM_BLOCK2(to_integer(unsigned( BBUS_in.BADDR(addresswidth-1 downto 2))))<=BBUS_in.BWDATA(15 downto 8);
         end if;
         if BBUS_in.BMASK(0)='1' then
           RAM_BLOCK3(to_integer(unsigned( BBUS_in.BADDR(addresswidth-1 downto 2))))<=BBUS_in.BWDATA(7 downto 0);
         end if; 
       end if;
     end if;
   end if;
 end process; 
 
 
 BBUS_out.BREADYOUT<='1';

 process(BCLK) 
  begin
   if rising_edge(BCLK) then
     imem_data<=         HIGHRAM (to_integer(unsigned( imem_address(addresswidth-1 downto 2))))
        & LOWRAM (to_integer(unsigned( imem_address(addresswidth-1 downto 2))));
   end if;
  end process; 
 
 
 process(BCLK) 
  begin
   if rising_edge(BCLK) then
     BRDATA<=         RAM_BLOCK0 (to_integer(unsigned( BBUS_IN.BADDR(addresswidth-1 downto 2))))
        &RAM_BLOCK1 (to_integer(unsigned( BBUS_IN.BADDR(addresswidth-1 downto 2))))
        &RAM_BLOCK2 (to_integer(unsigned( BBUS_IN.BADDR(addresswidth-1 downto 2))))
        &RAM_BLOCK3 (to_integer(unsigned( BBUS_IN.BADDR(addresswidth-1 downto 2))));
   end if;
  end process; 

  BBUS_OUT.BRDATA<=BRDATA;

 process(BCLK) 
   begin 
     if rising_edge(BCLK) then  
       BADDR_d1<=BBUS_IN.BADDR; 
       if BBUS_in.BWR = '1' and BBUS_in.BSEL(0) ='1' then 
         write_d1<='1';
       else
         write_d1<='0';
       end if;
     end if;
   end process;

 process(BCLK)
   begin
     if rising_edge(BCLK) then
       if write_d1='1' then
         HIGHRAM (to_integer(unsigned( BADDR_d1(addresswidth-1 downto 2))))<=BRDATA (31 downto 16);
         LOWRAM (to_integer(unsigned( BADDR_d1(addresswidth-1 downto 2))))<=BRDATA (15 downto 0);
       end if;
     end if;
   end process;

end;
