//convert.c by Steve Rhoads 4/26/01
//Now uses the ELF format (get gccmips_elf.zip)
//set $gp and zero .sbss and .bss
//Reads test.axf and creates code.txt

//addition by Rene Doss 5/5/11
//the generated outputfile use interface simple pipeline bus
//the program is the intial value of ram
//can be used as boot rom in Mais CPU

#include <stdio.h>
#include <stdlib.h>
#include <string.h>


#define BUF_SIZE (1024*1024*4) 
/*Assumes running on PC little endian*/
#ifndef USE_BIG_ENDIAN
#define ntohl(A) (((A)>>24)|(((A)&0x00ff0000)>>8)|(((A)&0xff00)<<8)|((A)<<24))
#define ntohs(A) (uint16)((((A)&0xff00)>>8)|((A)<<8))
#else
#define ntohl(A) A
#define ntohs(A) A
#endif

#define EI_NIDENT 16
#define SHT_PROGBITS 1
#define SHT_STRTAB 3
#define SHT_NOBITS 8

typedef unsigned int   uint32;
typedef unsigned short uint16;
typedef unsigned char  uint8;

typedef struct
{
   uint8 e_ident[EI_NIDENT];
   uint16 e_e_type;
   uint16 e_machine;
   uint32 e_version;
   uint32 e_entry;
   uint32 e_phoff;
   uint32 e_shoff;
   uint32 e_flags;
   uint16 e_ehsize;
   uint16 e_phentsize;
   uint16 e_phnum;
   uint16 e_shentsize;
   uint16 e_shnum;
   uint16 e_shstrndx;
} ElfHeader;

typedef struct
{
   uint32 p_type;
   uint32 p_offset;
   uint32 p_vaddr;
   uint32 p_paddr;
   uint32 p_filesz;
   uint32 p_memsz;
   uint32 p_flags;
   uint32 p_align;
} Elf32_Phdr;

typedef struct
{
   uint32 sh_name;
   uint32 sh_type;
   uint32 sh_flags;
   uint32 sh_addr;
   uint32 sh_offset;
   uint32 sh_size;
   uint32 sh_link;
   uint32 sh_info;
   uint32 sh_addralign;
   uint32 sh_entsize;
} Elf32_Shdr;

typedef struct 
{
   uint32 ri_gprmask;
   uint32 ri_cprmask[4];
   uint32 ri_gp_value;
} ELF_RegInfo;

#define PT_MIPS_REGINFO  0x70000000
#define SHT_MIPS_REGINFO 0x70000006
void help(void)
{
   printf("todo   Hilfetext einblenden!!!!!!!!!\n");
}


void set_low(uint8 *ptr, uint32 address, uint32 value)
{
   uint32 opcode;
   opcode = *(uint32 *)(ptr + address);
   opcode = ntohl(opcode);
   opcode = (opcode & 0xffff0000) | (value & 0xffff);
   opcode = ntohl(opcode);
   *(uint32 *)(ptr + address) = opcode;
}

int main(int argc, char *argv[])
{
char filenameinput[512];
if (argc== 1) {help(); exit(1);};



   strcpy(filenameinput,argv[1]);

   int addresswidth=12;  // 4k
   if (argc==3)
	{
		if (!strcmp(argv[2],"8k")) addresswidth=13;
		if (!strcmp(argv[2],"16k")) addresswidth=14;
		if (!strcmp(argv[2],"32k")) addresswidth=15;
		if (!strcmp(argv[2],"64k")) addresswidth=16;
		if (!strcmp(argv[2],"128k")) addresswidth=17;
	}	

 printf("options are 8k, 16k, 32k, 64k, 128k\n");
 printf("addresswidth: %d\n", addresswidth);
		
   FILE *infile, *outfile, *txtfile,*vhdlfile;
   uint8 *buf, *code;
   long size, stack_pointer;
   uint32 length, d, i,a, gp_ptr = 0, gp_ptr_backup = 0;
   uint32 bss_start = 0, bss_end = 0;

   ElfHeader *elfHeader;
   Elf32_Phdr *elfProgram;
   ELF_RegInfo *elfRegInfo;
   Elf32_Shdr *elfSection;
   (void)argc; 
   (void)argv;
   (void)stack_pointer;

  
  // infile = fopen("test.axf", "rb");
     infile = fopen(filenameinput, "rb");
   if(infile == NULL)
   {
      printf("%s\n",filenameinput);
      return 0;
   }
 printf("%s -> code.txt & test.bin & iram.vhd\n",filenameinput);
   buf = (uint8*)malloc(BUF_SIZE);
   size = (int)fread(buf, 1, BUF_SIZE, infile);
   fclose(infile);
   code = (uint8*)malloc(BUF_SIZE);
   memset(code, 0, BUF_SIZE);

   elfHeader = (ElfHeader *)buf;
   if(strncmp((char*)elfHeader->e_ident + 1, "ELF", 3))
   {
      printf("Error:  Not an ELF file!\n");
      printf("Use the gccmips_elf.zip from opencores/projects/plasma!\n");
      return -1;
   }

   elfHeader->e_entry = ntohl(elfHeader->e_entry);
   elfHeader->e_phoff = ntohl(elfHeader->e_phoff);
   elfHeader->e_shoff = ntohl(elfHeader->e_shoff);
   elfHeader->e_flags = ntohl(elfHeader->e_flags);
   elfHeader->e_phentsize = ntohs(elfHeader->e_phentsize);
   elfHeader->e_phnum = ntohs(elfHeader->e_phnum);
   elfHeader->e_shentsize = ntohs(elfHeader->e_shentsize);
   elfHeader->e_shnum = ntohs(elfHeader->e_shnum);
   printf("Entry=0x%x ", elfHeader->e_entry);
   length = 0;

   for(i = 0; i < elfHeader->e_phnum; ++i)
   {
      elfProgram = (Elf32_Phdr *)(buf + elfHeader->e_phoff +
                         elfHeader->e_phentsize * i);
      elfProgram->p_type = ntohl(elfProgram->p_type);
      elfProgram->p_offset = ntohl(elfProgram->p_offset);
      elfProgram->p_vaddr = ntohl(elfProgram->p_vaddr);
      elfProgram->p_filesz = ntohl(elfProgram->p_filesz);
      elfProgram->p_memsz = ntohl(elfProgram->p_memsz);
      elfProgram->p_flags = ntohl(elfProgram->p_flags);

      elfProgram->p_vaddr -= elfHeader->e_entry;

      if(elfProgram->p_type == PT_MIPS_REGINFO)
      {
         elfRegInfo = (ELF_RegInfo*)(buf + elfProgram->p_offset);
         gp_ptr = ntohl(elfRegInfo->ri_gp_value);
      }
      if(elfProgram->p_vaddr < BUF_SIZE)
      {
         //printf("[0x%x,0x%x,0x%x,0x%x,0x%x]\n", elfProgram->p_vaddr,
         //   elfProgram->p_offset, elfProgram->p_filesz, elfProgram->p_memsz,
         //   elfProgram->p_flags);
         memcpy(code + elfProgram->p_vaddr, buf + elfProgram->p_offset,
                 elfProgram->p_filesz);
         length = elfProgram->p_vaddr + elfProgram->p_filesz;
         //printf("length = %d 0x%x\n", length, length);
      }
   }

   for(i = 0; i < elfHeader->e_shnum; ++i)
   {
      elfSection = (Elf32_Shdr *)(buf + elfHeader->e_shoff +
                         elfHeader->e_shentsize * i);
      elfSection->sh_name = ntohl(elfSection->sh_name);
      elfSection->sh_type = ntohl(elfSection->sh_type);
      elfSection->sh_addr = ntohl(elfSection->sh_addr);
      elfSection->sh_offset = ntohl(elfSection->sh_offset);
      elfSection->sh_size = ntohl(elfSection->sh_size);

      if(elfSection->sh_type == SHT_MIPS_REGINFO)
      {
         elfRegInfo = (ELF_RegInfo*)(buf + elfSection->sh_offset);
         gp_ptr = ntohl(elfRegInfo->ri_gp_value);
      }
      if(elfSection->sh_type == SHT_PROGBITS)
      {
         //printf("elfSection->sh_addr=0x%x\n", elfSection->sh_addr);
         if(elfSection->sh_addr > gp_ptr_backup)
            gp_ptr_backup = elfSection->sh_addr;
      }
      if(elfSection->sh_type == SHT_NOBITS)
      {
         if(bss_start == 0)
         {
            bss_start = elfSection->sh_addr;
         }
         bss_end = elfSection->sh_addr + elfSection->sh_size;
      }
   }

   if(length > bss_start - elfHeader->e_entry)
   {
      length = bss_start - elfHeader->e_entry;
   }
   if(bss_start == length)
   {
      bss_start = length;
      bss_end = length + 4;
   }
   if(gp_ptr == 0)
      gp_ptr = gp_ptr_backup + 0x7ff0;

#if 0
   /*Initialize the $gp register for sdata and sbss */
   printf("gp_ptr=0x%x ", gp_ptr);
   /*modify the first opcodes in boot.asm */
   /*modify the lui opcode */
   set_low(code, 0, gp_ptr >> 16);
   /*modify the ori opcode */
   set_low(code, 4, gp_ptr & 0xffff);

   /*Clear .sbss and .bss */
   printf("sbss=0x%x bss_end=0x%x\nlength=0x%x ", bss_start, bss_end, length);
   set_low(code, 8, bss_start >> 16);
   set_low(code, 12, bss_start & 0xffff);
   set_low(code, 16, bss_end >> 16);
   set_low(code, 20, bss_end & 0xffff);

   /*Set stack pointer */
   if(elfHeader->e_entry < 0x10000000)
      stack_pointer = bss_end + 512;
   else
      stack_pointer = bss_end + 1024 * 4;
   stack_pointer &= ~7;
   printf("SP=0x%x\n", stack_pointer);
   set_low(code, 24, stack_pointer >> 16);
   set_low(code, 28, stack_pointer & 0xffff);
#endif

   /*write out test.bin */
   outfile = fopen("test.bin", "wb");
   fwrite(code, length, 1, outfile);
   fclose(outfile);

   /*write out code.txt */
   txtfile = fopen("code.txt", "w");
   for(i = 0; i <= length; i += 4)
   {
      d = ntohl(*(uint32 *)(code + i));
      fprintf(txtfile, "%8.8x\n", d);
   }
   fclose(txtfile);
 
   
  
 
 
 /*write out */
    vhdlfile =fopen("spb_memory.vhd","w");

   fprintf(vhdlfile,"library ieee;\n");
   fprintf(vhdlfile,"use ieee.STD_LOGIC_1164.all;\n\n");
   fprintf(vhdlfile,"use ieee.numeric_std.all;\n\n");
   fprintf(vhdlfile,"use work.SPB.all;\n\n");
      
   fprintf(vhdlfile,"entity SPB_memory is\n");
   fprintf(vhdlfile,"   generic(memory_type : string := \"DEFAULT\");\n");
   fprintf(vhdlfile,"   port(BCLK          : in std_logic;\n");      
   fprintf(vhdlfile,"        BBUS_in       : in SPB_in;\n");      
   fprintf(vhdlfile,"        BBUS_out      : out SPB_out;\n\n");

   fprintf(vhdlfile,"        imem_address  : in std_logic_vector(31 downto 0);\n");
   fprintf(vhdlfile,"        imem_data     : out std_logic_vector(31 downto 0);\n");
   fprintf(vhdlfile,"        imem_valid    : out std_logic);\n");
   fprintf(vhdlfile,"end; --entity \n");

   fprintf(vhdlfile,"architecture logic of SPB_memory is\n \n");
   
   fprintf(vhdlfile,"constant addresswidth : integer:= %d;\n" ,addresswidth);
   fprintf(vhdlfile,"type ram_byte is array (0 to (2**(addresswidth-1)))\n");		
   fprintf(vhdlfile,"         of std_logic_vector (7 downto 0);\n\n");

   fprintf(vhdlfile,"signal BADDR_d1: std_logic_vector (31 downto 0);\n");
   fprintf(vhdlfile,"signal write_d1: std_logic;\n");
   fprintf(vhdlfile,"signal BRDATA  : std_logic_vector (31 downto 0);\n\n");

   fprintf(vhdlfile,"--block0  bit 31...24\n");
   fprintf(vhdlfile,"signal RAM_BLOCK0:ram_byte:=(\n");
	a=0;
   for(i = 0; i <= length; i += 4)
   {
      fprintf(vhdlfile, "X\"%2.2x\",", *(code+i));
     a++;
     if (a==16)  /* line wrapping */
		{a=0; fprintf(vhdlfile,"\n");}		
	 }
	 fprintf(vhdlfile,"others=>X\"00\");\n\n");	

  fprintf(vhdlfile,"signal RAM_BLOCK1:ram_byte:=(\n");
	a=0;
   for(i = 1; i <= length+1; i += 4)
   {
      fprintf(vhdlfile, "X\"%2.2x\",", *(code+i));
     a++;
     if (a==16)  /* line wrapping */
		{a=0; fprintf(vhdlfile,"\n");}		
	 }
	 fprintf(vhdlfile,"others=>X\"00\");\n\n");	

   fprintf(vhdlfile,"signal RAM_BLOCK2:ram_byte:=(\n");
	a=0;
   for(i = 2; i <= length+2; i += 4)
   {
      fprintf(vhdlfile, "X\"%2.2x\",", *(code+i));
     a++;
     if (a==16)  /* line wrapping */
		{a=0; fprintf(vhdlfile,"\n");}		
	 }
	 fprintf(vhdlfile,"others=>x\"00\");\n\n");	
	 
   fprintf(vhdlfile,"signal RAM_BLOCK3:ram_byte:=(\n");
	a=0;
   for(i = 3; i <= length+3; i += 4)
   {
      fprintf(vhdlfile, "X\"%2.2x\",", *(code+i));
     a++;
     if (a==16)  /* line wrapping */
		{a=0; fprintf(vhdlfile,"\n");}		
	 }
	 fprintf(vhdlfile,"others=>x\"00\");\n\n");
     
     
     fprintf(vhdlfile,"  attribute RAM_STYLE: string;\n");
     fprintf(vhdlfile,"  attribute RAM_STYLE of RAM_BLOCK0: signal is \"block\";\n");
     fprintf(vhdlfile,"  attribute RAM_STYLE of RAM_BLOCK1: signal is \"block\";\n");
     fprintf(vhdlfile,"  attribute RAM_STYLE of RAM_BLOCK2: signal is \"block\";\n");
     fprintf(vhdlfile,"  attribute RAM_STYLE of RAM_BLOCK3: signal is \"block\";\n\n");

   fprintf(vhdlfile,"type ram_halfword is array (0 to (2**(addresswidth-1)))\n");
   
		
   fprintf(vhdlfile,"         of std_logic_vector (15 downto 0);\n\n");

   fprintf(vhdlfile,"--high block  bit 31...16\n");
   fprintf(vhdlfile,"signal HIGHRAM:ram_halfword:=(\n");
	a=0;
   for(i = 0; i <= length; i += 4)
   {
      fprintf(vhdlfile, "X\"%2.2x%2.2x\",", *(code+i),*(code+i+1));
     a++;
     if (a==8)  /* line wrapping */
		{a=0; fprintf(vhdlfile,"\n");}		
	 }
	 fprintf(vhdlfile,"others=>X\"0000\");\n\n");	

     
   fprintf(vhdlfile,"--low block  bit 15...0\n");     
   fprintf(vhdlfile,"signal LOWRAM:ram_halfword:=(\n");
 	a=0;
   for(i = 2; i <= length+2; i += 4)
   {
      fprintf(vhdlfile, "X\"%2.2x%2.2x\",", *(code+i),*(code+i+1));
     a++;
     if (a==8)  /* line wrapping */
		{a=0; fprintf(vhdlfile,"\n");}		
	 }
	 fprintf(vhdlfile,"others=>X\"0000\");\n\n");	
    
     
     
     fprintf(vhdlfile," begin\n\n");
     fprintf(vhdlfile," imem_valid <='1';\n");
     
     fprintf(vhdlfile," ram_proc: process(BCLK)\n\n");
     fprintf(vhdlfile," begin\n");
     fprintf(vhdlfile,"   if rising_edge(BCLK) then\n\n");
     fprintf(vhdlfile,"     if BBUS_in.BWR = '1' and BBUS_in.BREADY='1' then\n");
     fprintf(vhdlfile,"       if BBUS_in.BSEL(0) = '1' then\n");
     fprintf(vhdlfile,"         if BBUS_in.BMASK(3)='1' then\n");
     fprintf(vhdlfile,"           RAM_BLOCK0(to_integer(unsigned( BBUS_in.BADDR(addresswidth-1 downto 2))))<=BBUS_in.BWDATA(31 downto 24);\n");
     fprintf(vhdlfile,"         end if;\n");
     fprintf(vhdlfile,"         if BBUS_in.BMASK(2)='1' then\n");
     fprintf(vhdlfile,"           RAM_BLOCK1(to_integer(unsigned( BBUS_in.BADDR(addresswidth-1 downto 2))))<=BBUS_in.BWDATA(23 downto 16);\n");
     fprintf(vhdlfile,"         end if;\n");
     fprintf(vhdlfile,"         if BBUS_in.BMASK(1)='1' then\n");
     fprintf(vhdlfile,"           RAM_BLOCK2(to_integer(unsigned( BBUS_in.BADDR(addresswidth-1 downto 2))))<=BBUS_in.BWDATA(15 downto 8);\n");
     fprintf(vhdlfile,"         end if;\n");
     fprintf(vhdlfile,"         if BBUS_in.BMASK(0)='1' then\n");
     fprintf(vhdlfile,"           RAM_BLOCK3(to_integer(unsigned( BBUS_in.BADDR(addresswidth-1 downto 2))))<=BBUS_in.BWDATA(7 downto 0);\n");
     fprintf(vhdlfile,"         end if; \n");
    
     fprintf(vhdlfile,"       end if;\n");
     fprintf(vhdlfile,"     end if;\n");
     fprintf(vhdlfile,"   end if;\n");  
     fprintf(vhdlfile," end process; \n \n \n");
        
    
     fprintf(vhdlfile," BBUS_out.BREADYOUT<='1';\n\n");  
 
 

     fprintf(vhdlfile," process(BCLK) \n");
     fprintf(vhdlfile,"  begin\n");
     fprintf(vhdlfile,"   if rising_edge(BCLK) then\n");
       fprintf(vhdlfile,"     imem_data<=");
   
     fprintf(vhdlfile,"         HIGHRAM (to_integer(unsigned( imem_address(addresswidth-1 downto 2))))\n");
     fprintf(vhdlfile,"        & LOWRAM (to_integer(unsigned( imem_address(addresswidth-1 downto 2))));\n");
     fprintf(vhdlfile,"   end if;\n");
     fprintf(vhdlfile,"  end process; \n \n \n");
    
 
     fprintf(vhdlfile," process(BCLK) \n");
     fprintf(vhdlfile,"  begin\n");
     fprintf(vhdlfile,"   if rising_edge(BCLK) then\n");
     fprintf(vhdlfile,"     BRDATA<=");
     fprintf(vhdlfile,"         RAM_BLOCK0 (to_integer(unsigned( BBUS_IN.BADDR(addresswidth-1 downto 2))))\n");
     fprintf(vhdlfile,"        &RAM_BLOCK1 (to_integer(unsigned( BBUS_IN.BADDR(addresswidth-1 downto 2))))\n");
     fprintf(vhdlfile,"        &RAM_BLOCK2 (to_integer(unsigned( BBUS_IN.BADDR(addresswidth-1 downto 2))))\n");
     fprintf(vhdlfile,"        &RAM_BLOCK3 (to_integer(unsigned( BBUS_IN.BADDR(addresswidth-1 downto 2))));\n");
     fprintf(vhdlfile,"   end if;\n");
     fprintf(vhdlfile,"  end process; \n\n");   

     fprintf(vhdlfile,"  BBUS_OUT.BRDATA<=BRDATA;\n\n");


     fprintf(vhdlfile," process(BCLK) \n");
     fprintf(vhdlfile,"   begin \n");
     fprintf(vhdlfile,"     if rising_edge(BCLK) then  \n");
     fprintf(vhdlfile,"       BADDR_d1<=BBUS_IN.BADDR; \n");
     fprintf(vhdlfile,"       if BBUS_in.BWR = '1' and BBUS_in.BSEL(0) ='1' then \n");
     fprintf(vhdlfile,"         write_d1<='1';\n");
     fprintf(vhdlfile,"       else\n");
     fprintf(vhdlfile,"         write_d1<='0';\n");
     fprintf(vhdlfile,"       end if;\n");
     fprintf(vhdlfile,"     end if;\n");
     fprintf(vhdlfile,"   end process;\n\n");

     fprintf(vhdlfile," process(BCLK)\n");
     fprintf(vhdlfile,"   begin\n");
     fprintf(vhdlfile,"     if rising_edge(BCLK) then\n");
     fprintf(vhdlfile,"       if write_d1='1' then\n");
     fprintf(vhdlfile,"         HIGHRAM (to_integer(unsigned( BADDR_d1(addresswidth-1 downto 2))))<=BRDATA (31 downto 16);\n");
     fprintf(vhdlfile,"         LOWRAM (to_integer(unsigned( BADDR_d1(addresswidth-1 downto 2))))<=BRDATA (15 downto 0);\n");
     fprintf(vhdlfile,"       end if;\n");
     fprintf(vhdlfile,"     end if;\n");
     fprintf(vhdlfile,"   end process;\n\n");


    
    
    
    fprintf(vhdlfile,"end;\n");

	 
   fclose(vhdlfile);
  free(buf);
   printf("length=%d=0x%x\n", length, length);

   return 0;
}

